#!/bin/bash


# Instalación de Docker 
yum install docker

# Configuración del cortafuegos del sistema
sudo firewall-cmd --permanent --zone=trusted --add-source=127.0.0.1/32
sudo firewall-cmd --permanent --zone=trusted --add-port=5432/tcp
sudo firewall-cmd --reload

# Creación del directorio donde se mapeará el sistema de archivos del contendor
# de PostgreSQL, y aplicación del contexto SELinux
sudo mkdir -p /opt/dockyard/postgres
sudo chcon -Rt svirt_sandbox_file_t /opt/dockyard/postgres

# Creación del contenedor PostgreSQL
docker run --name postgresql-tfg-uoc \
-p 5432:5432 \
-v /opt/dockyard/postgres:/var/lib/postgresql \
-e POSTGRES_PASSWORD=palocariz \
-d postgres:9.6.2

# Creación del contenedor PgAdmin
docker run --name pgadmin4-tfg-uoc \
-p 5050:5050 \
-e POSTGRES_PASSWORD=palocariz \
-d fenglc/pgadmin4



# Activación del servicio y los contenedores
systemctl start  docker.service
docker start postgresql-tfg-uoc
docker start pgadmin4-tfg-uoc


# Creación de directorios para los tablespaces desde dentro del contenedor
docker exec postgresql-tfg-uoc \
 mkdir -p  /var/lib/postgresql/tablespaces/tb_general & \
 mkdir -p  /var/lib/postgresql/tablespaces/tb_documentos & \
 mkdir -p  /var/lib/postgresql/tablespaces/tb_letras & \
 mkdir -p  /var/lib/postgresql/tablespaces/tb_idx & \
 mkdir -p  /var/lib/postgresql/tablespaces/tb_dwh & \
 chown -R postgres:postgres  /var/lib/postgresql/tablespaces
 

# Configuración de las credenciales para el uso de psql sin autenticación
   # "hostname:port:database:user:password"
echo "localhost:5432:carnaval:postgres:palocariz" > /root/.pgpass

chmod 0600 /root/.pgpass


# Creación

psql -h localhost -U postgres -f 01_database.sql

psql -h localhost -U postgres -d "carnaval" -f 02_tablas.sql

psql -h localhost -U postgres -d "carnaval" -f 03_indices.sql

psql -h localhost -U postgres -d "carnaval" -f 04_roles.sql

psql -h localhost -U postgres -d "carnaval" -f 05_SP_01_tablasLookup.sql

psql -h localhost -U postgres -d "carnaval" -f 05_SP_02_Documento.sql

psql -h localhost -U postgres -d "carnaval" -f 05_SP_03_Entidad.sql

psql -h localhost -U postgres -d "carnaval" -f 05_SP_04_Evento.sql

psql -h localhost -U postgres -d "carnaval" -f 05_SP_05_AgrupacionAnio.sql

psql -h localhost -U postgres -d "carnaval" -f 05_SP_06_Obra.sql

psql -h localhost -U postgres -d "carnaval" -f 05_SP_061_ObraLiteraria.sql

psql -h localhost -U postgres -d "carnaval" -f 05_SP_062_ObraMusical.sql

psql -h localhost -U postgres -d "carnaval" -f 05_SP_063_ObraGrafica.sql

psql -h localhost -U postgres -d "carnaval" -f 05_SP_071_Letra.sql

psql -h localhost -U postgres -d "carnaval" -f 05_SP_072_Musica.sql

psql -h localhost -U postgres -d "carnaval" -f 05_SP_073_Disenio.sql

psql -h localhost -U postgres -d "carnaval" -f 06_vistas.sql


# Creación DWH

psql -h localhost -U postgres -d "carnaval" -f 07_DWH_tablas_vistas.sql

psql -h localhost -U postgres -d "carnaval" -f 07_DWH_triggers.sql

psql -h localhost -U postgres -d "carnaval" -f 08_DWH_SP_premiosconcurso.sql

psql -h localhost -U postgres -d "carnaval" -f 08_DWH_SP_funcionesentidad.sql

psql -h localhost -U postgres -d "carnaval" -f 08_DWH_SP_soportedocumental.sql

psql -h localhost -U postgres -d "carnaval" -f 08_DWH_SP_temasrecurrentes.sql


# Conjunto de inserciones por tabla

psql -h localhost -U postgres -d "carnaval" -f 09_inserts_01_ModalidadDescriptorActividad.sql

psql -h localhost -U postgres -d "carnaval" -f 09_inserts_01_Tema.sql

psql -h localhost -U postgres -d "carnaval" -f 09_inserts_02_Agrupacion.sql

psql -h localhost -U postgres -d "carnaval" -f 09_inserts_02_Empresa.sql

psql -h localhost -U postgres -d "carnaval" -f 09_inserts_02_Persona.sql

psql -h localhost -U postgres -d "carnaval" -f 09_inserts_03_Evento.sql

psql -h localhost -U postgres -d "carnaval" -f 09_inserts_041_Agrupacion_Anio.sql

psql -h localhost -U postgres -d "carnaval" -f 09_inserts_042_Agrupacion_Intrumento.sql

psql -h localhost -U postgres -d "carnaval" -f 09_inserts_043_AgrupacionAnio_Cargo.sql

psql -h localhost -U postgres -d "carnaval" -f 09_inserts_043_AgrupacionAnio_Orquesta.sql

psql -h localhost -U postgres -d "carnaval" -f 09_inserts_043_AgrupacionAnio_Voz.sql

psql -h localhost -U postgres -d "carnaval" -f 09_inserts_044_Agrupacion_Participa.sql

psql -h localhost -U postgres -d "carnaval" -f 09_inserts_061_Obra.sql

psql -h localhost -U postgres -d "carnaval" -f 09_inserts_062_Obra_Agrup.sql

psql -h localhost -U postgres -d "carnaval" -f 09_inserts_071_Letra.sql

psql -h localhost -U postgres -d "carnaval" -f 09_inserts_071_Letra_Tema.sql

psql -h localhost -U postgres -d "carnaval" -f 09_inserts_072_Musica.sql

psql -h localhost -U postgres -d "carnaval" -f 09_inserts_073_Disenio.sql



# Conjunto de pruebas automatizadas

psql -h localhost -U postgres -d "carnaval" -f 10_pruebas_01_tablasLookup.sql > resultadosBBDD.txt 2>&1
psql -h localhost -U postgres -d "carnaval" -f 10_pruebas_02_Documento.sql >> resultadosBBDD.txt  2>&1
psql -h localhost -U postgres -d "carnaval" -f 10_pruebas_03_Entidad.sql >> resultadosBBDD.txt 2>&1
psql -h localhost -U postgres -d "carnaval" -f 10_pruebas_04_Evento.sql >> resultadosBBDD.txt 2>&1
psql -h localhost -U postgres -d "carnaval" -f 10_pruebas_05_AgrupacionAnio.sql >> resultadosBBDD.txt 2>&1
psql -h localhost -U postgres -d "carnaval" -f 10_pruebas_061_ObraLiteraria.sql >> resultadosBBDD.txt 2>&1
psql -h localhost -U postgres -d "carnaval" -f 10_pruebas_062_ObraMusical.sql >> resultadosBBDD.txt 2>&1
psql -h localhost -U postgres -d "carnaval" -f 10_pruebas_063_ObraGrafica.sql >> resultadosBBDD.txt 2>&1
psql -h localhost -U postgres -d "carnaval" -f 10_pruebas_071_Letra.sql >> resultadosBBDD.txt 2>&1
psql -h localhost -U postgres -d "carnaval" -f 10_pruebas_072_Musica.sql >> resultadosBBDD.txt 2>&1
psql -h localhost -U postgres -d "carnaval" -f 10_pruebas_073_Disenio.sql >> resultadosBBDD.txt 2>&1


psql -h localhost -U postgres -d "carnaval" -f 10_pruebas_080_DWH.sql >> resultadosBBDD.txt 2>&1

