-- TFG BBDD Carnaval de Cádiz - Paloma Cortés


-- Creación de las tablas




set default_tablespace = 'tb_general';
set default_with_oids = false;



-- Creación de esquema dedicado

CREATE SCHEMA general_accc AUTHORIZATION general_accc_ROLE;


--
-- Creación de secuencias
--

CREATE SEQUENCE general_accc.seq_entidad start 100000;
CREATE SEQUENCE general_accc.seq_agrupacionanio start 100000;
CREATE SEQUENCE general_accc.seq_obra start 100000;
CREATE SEQUENCE general_accc.seq_documento start 100000;

CREATE SEQUENCE general_accc.seq_modalidad start 1;
CREATE SEQUENCE general_accc.seq_instrumento start 1;
CREATE SEQUENCE general_accc.seq_usa start 1;
CREATE SEQUENCE general_accc.seq_evento start 1;
CREATE SEQUENCE general_accc.seq_premio start 1;
CREATE SEQUENCE general_accc.seq_rol start 1;
CREATE SEQUENCE general_accc.seq_cargo start 1;
CREATE SEQUENCE general_accc.seq_actividad start 1;
CREATE SEQUENCE general_accc.seq_actividad_empresa start 1;
CREATE SEQUENCE general_accc.seq_descriptor start 1;
CREATE SEQUENCE general_accc.seq_descriptor_documento start 1;
CREATE SEQUENCE general_accc.seq_depositolegal start 1;
CREATE SEQUENCE general_accc.seq_letra start 1;
CREATE SEQUENCE general_accc.seq_musica start 1;
CREATE SEQUENCE general_accc.seq_disenio start 1;
CREATE SEQUENCE general_accc.seq_tema start 1;
CREATE SEQUENCE general_accc.seq_tema_letra start 1;
CREATE SEQUENCE general_accc.seq_tema_disenio start 1;
CREATE SEQUENCE general_accc.seq_crea start 1;


--
-- Creación de tablas e índices, y enumeraciones donde son utilizadas
--

--
-- Nombre: entidad; Tipo: ENUM; Schema: general_accc; Tablespace: tb_global
--
CREATE TYPE general_accc.enum_entidad AS ENUM ('Persona', 'Agrupacion', 'Empresa');

--
-- Nombre: entidad; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.entidad (
    id integer NOT NULL DEFAULT nextval('general_accc.seq_entidad'::regclass),
    tipo_entidad general_accc.enum_entidad NOT NULL,

    CONSTRAINT pk_entidad PRIMARY KEY (id) USING INDEX TABLESPACE tb_idx

);



--
-- Nombre: agrupacion; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.agrupacion (
    entidad integer NOT NULL,
    localidad character varying(50) NOT NULL,
    provincia character varying(50) NOT NULL,
    alias character varying(50),
    contacto character varying(100),
    observaciones text,

    CONSTRAINT pk_agrupacion PRIMARY KEY (entidad) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT uq_agrupacion_alias UNIQUE (alias) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT chk_agrupacion_localidad CHECK (char_length(trim(localidad)) <> 0),

    CONSTRAINT chk_agrupacion_provincia CHECK (char_length(trim(provincia)) <> 0),

    CONSTRAINT fk_agrupacion_entidad FOREIGN KEY (entidad)
      REFERENCES general_accc.entidad (id) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE RESTRICT
);



--
-- Nombre: modalidad; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.modalidad (
    id integer NOT NULL DEFAULT nextval('general_accc.seq_modalidad'::regclass),
    modalidad character varying(30) NOT NULL,

    CONSTRAINT pk_modalidad PRIMARY KEY (id) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT uq_modalidad_modalidad UNIQUE (modalidad) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT chk_modalidad_modalidad CHECK (char_length(trim(modalidad)) <> 0)
);



--
-- Nombre: enum_categoria; Tipo: ENUM; Schema: general_accc; Tablespace: tb_global
--
CREATE TYPE general_accc.enum_categoria AS ENUM ('Infantil', 'Juvenil', 'Junior', 'Adultos');

--
-- Nombre: enum_composicion; Tipo: ENUM; Schema: general_accc; Tablespace: tb_global
--
CREATE TYPE general_accc.enum_composicion AS ENUM ('Desconocida', 'Femenina', 'Masculina', 'Mixta');

--
-- Nombre: agrupacion_anio; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.agrupacion_anio (
    entidad integer NOT NULL,
    id integer NOT NULL DEFAULT nextval('general_accc.seq_agrupacionanio'::regclass),
    nombre character varying(100) NOT NULL,
    anio integer NOT NULL,
    categoria general_accc.enum_categoria NOT NULL,
    modalidad integer NOT NULL,
    submodalidad character varying(100),
    composicion general_accc.enum_composicion NOT NULL,
    num_componentes integer DEFAULT 0,
    presupuesto numeric(10,0) DEFAULT 0.0,
    lugar_ensayo character varying(100),
    observaciones text,
    en_prensa boolean NOT NULL DEFAULT false,
    en_documento_oficial boolean NOT NULL DEFAULT false,
    fuentes character varying(500),
    observaciones_gestion text,

    CONSTRAINT pk_agrupacionanio PRIMARY KEY (id) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT uq_agrupacionanio_nombre UNIQUE (nombre) USING INDEX TABLESPACE tb_idx,
    CONSTRAINT uq_agrupacionanio_entidadnombre UNIQUE (entidad, nombre) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT chk_agrupacionanio_nombre CHECK (char_length(trim(nombre)) <> 0),
    CONSTRAINT chk_agrupacionanio_anio CHECK (anio > 0  AND  anio < 10000),
    CONSTRAINT chk_agrupacionanio_numcomponentes CHECK (num_componentes > 0),
    CONSTRAINT chk_agrupacionanio_presupuesto CHECK (presupuesto > 0.0),

    CONSTRAINT fk_agrupacionanio_entidad FOREIGN KEY (entidad)
      REFERENCES general_accc.agrupacion (entidad) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE RESTRICT,

    CONSTRAINT fk_agrupacionanio_modalidad FOREIGN KEY (modalidad)
      REFERENCES general_accc.modalidad (id) MATCH SIMPLE
      ON UPDATE RESTRICT ON DELETE RESTRICT
);



--
-- Nombre: instrumento; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.instrumento (
    id integer NOT NULL DEFAULT nextval('general_accc.seq_instrumento'::regclass),
    instrumento character varying(50) NOT NULL,

    CONSTRAINT pk_instrumento PRIMARY KEY (id) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT uq_instrumento_instrumento UNIQUE (instrumento) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT chk_instrumento_instrumento CHECK (char_length(trim(instrumento)) <> 0)
);



--
-- Nombre: usa; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.usa (
    id integer NOT NULL DEFAULT nextval('general_accc.seq_usa'::regclass),
    instrumento integer NOT NULL,
    agrupacion_anio integer NOT NULL,
    total integer DEFAULT 0,

    CONSTRAINT pk_usa PRIMARY KEY (id) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT uq_usa_instrumentoagrupacionanio UNIQUE (instrumento, agrupacion_anio) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT chk_usa_total CHECK (total > -1),

    CONSTRAINT fk_usa_instrumento FOREIGN KEY (instrumento)
      REFERENCES general_accc.instrumento (id) MATCH SIMPLE
      ON UPDATE RESTRICT ON DELETE RESTRICT,

    CONSTRAINT fk_usa_agrupacionanio FOREIGN KEY (agrupacion_anio)
      REFERENCES general_accc.agrupacion_anio (id) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE CASCADE
);



--
-- Nombre: enum_evento; Tipo: ENUM; Schema: general_accc; Tablespace: tb_global
--
CREATE TYPE general_accc.enum_evento AS ENUM ('Concurso', 'Premio', 'Homenaje', 'Efemérides');

--
-- Nombre: evento; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.evento (
    id integer NOT NULL DEFAULT nextval('general_accc.seq_evento'::regclass),
    tipo_evento general_accc.enum_evento NOT NULL,
    anio integer NOT NULL,
    nombre character varying(50) NOT NULL,
    localidad character varying(50) NOT NULL,
    provincia character varying(50) NOT NULL,
    ubicacion character varying(100),
    descripcion text,

    CONSTRAINT pk_evento PRIMARY KEY (id) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT uq_evento_anionombrelocalidad UNIQUE (anio, nombre, localidad) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT chk_evento_nombre CHECK (char_length(trim(nombre)) <> 0),
    CONSTRAINT chk_evento_localidad CHECK (char_length(trim(localidad)) <> 0),
    CONSTRAINT chk_evento_provincia CHECK (char_length(trim(provincia)) <> 0),

    CONSTRAINT chk_evento_anio CHECK (anio > 0  AND  anio < 10000)
);



--
-- Nombre: concurso; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.concurso (
    evento integer NOT NULL,
    categoria general_accc.enum_categoria NOT NULL,

    CONSTRAINT pk_concurso PRIMARY KEY (evento) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT fk_concurso_evento FOREIGN KEY (evento)
      REFERENCES general_accc.evento (id) MATCH SIMPLE
      ON UPDATE RESTRICT ON DELETE RESTRICT
);



--
-- Nombre: ; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.premio (
    id integer NOT NULL DEFAULT nextval('general_accc.seq_premio'::regclass),
    concurso integer NOT NULL,
    modalidad integer NOT NULL,
    premio integer NOT NULL,
    cuantia numeric(10,0) DEFAULT 0.0,
    observaciones text,

    CONSTRAINT pk_premio PRIMARY KEY (id) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT uq_premio_concursomodalidadpremio UNIQUE (concurso, modalidad, premio) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT chk_premio_premio CHECK (premio > 0),
    CONSTRAINT chk_premio_cuantia CHECK (cuantia > -1.0),

    CONSTRAINT fk_premio_concurso FOREIGN KEY (concurso)
      REFERENCES general_accc.concurso (evento) MATCH SIMPLE
      ON UPDATE RESTRICT ON DELETE RESTRICT,

    CONSTRAINT fk_premio_modalidad FOREIGN KEY (modalidad)
      REFERENCES general_accc.modalidad (id) MATCH SIMPLE
      ON UPDATE RESTRICT ON DELETE RESTRICT
);



--
-- Nombre: participa; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.participa (
    agrupacion_anio integer NOT NULL,
    concurso integer NOT NULL,
    clasificacion integer,
    puntos integer,
    fase character varying(20),

    CONSTRAINT pk_participa PRIMARY KEY (agrupacion_anio, concurso) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT chk_participa_clasificacion CHECK (clasificacion > 0),
    CONSTRAINT chk_participa_puntos CHECK (puntos > -1),

    CONSTRAINT fk_participa_agrupacionanio FOREIGN KEY (agrupacion_anio)
      REFERENCES general_accc.agrupacion_anio (id) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE RESTRICT,

    CONSTRAINT fk_participa_concurso FOREIGN KEY (concurso)
      REFERENCES general_accc.concurso (evento) MATCH SIMPLE
      ON UPDATE RESTRICT ON DELETE RESTRICT
);



--
-- Nombre: persona; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.persona (
    entidad integer NOT NULL,
    nombre character varying(30) NOT NULL,
    apellido1 character varying(30) NOT NULL,
    apellido2 character varying(30),
    alias character varying(50),
    dni character varying(9),
    telefono character varying(20),
    domicilio character varying(150),
    localidad character varying(50),
    provincia character varying(50),
    cp character varying(6),
    profesion character varying(100),
    fec_nac date,
    loc_nac character varying(50),
    prov_nac character varying(50),
    historia text,
    observaciones text,

    CONSTRAINT pk_persona PRIMARY KEY (entidad) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT uq_persona_dni UNIQUE (dni) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT chk_persona_nombre CHECK (char_length(trim(nombre)) <> 0),
    CONSTRAINT chk_persona_apellido1 CHECK (char_length(trim(apellido1)) <> 0),

    CONSTRAINT fk_persona_entidad FOREIGN KEY (entidad)
      REFERENCES general_accc.entidad (id) MATCH SIMPLE
      ON UPDATE RESTRICT ON DELETE RESTRICT
);



--
-- Nombre: enum_rol; Tipo: ENUM; Schema: general_accc; Tablespace: tb_global
--
CREATE TYPE general_accc.enum_rol AS ENUM ('Cargo', 'Voz', 'Orquesta');

--
-- Nombre: rol; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.rol (
    id integer NOT NULL DEFAULT nextval('general_accc.seq_rol'::regclass),
    persona integer NOT NULL,
    agrupacion_anio integer NOT NULL,
    tipo_rol general_accc.enum_rol NOT NULL,

    CONSTRAINT pk_rol PRIMARY KEY (id) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT uq_rol_personaagrupacionaniotiporol UNIQUE (persona, agrupacion_anio, tipo_rol) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT fk_rol_persona FOREIGN KEY (persona)
      REFERENCES general_accc.persona (entidad) MATCH SIMPLE
      ON UPDATE RESTRICT ON DELETE RESTRICT,

    CONSTRAINT fk_rol_agrupacionanio FOREIGN KEY (agrupacion_anio)
      REFERENCES general_accc.agrupacion_anio (id) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE RESTRICT
);



--
-- Nombre: enum_cargo; Tipo: ENUM; Schema: general_accc; Tablespace: tb_global
--
CREATE TYPE general_accc.enum_cargo AS ENUM ('Presidente', 'Director', 'Representante legal', 'Colaborador', 'Relaciones públicas', 'Postulante');

--
-- Nombre: cargo; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.cargo (
    id integer NOT NULL DEFAULT nextval('general_accc.seq_cargo'::regclass),
    rol integer NOT NULL,
    tipo_cargo general_accc.enum_cargo NOT NULL,
    observaciones text,

    CONSTRAINT pk_cargo PRIMARY KEY (id) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT uq_cargo_roltipocargo UNIQUE (rol, tipo_cargo) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT fk_cargo_rol FOREIGN KEY (rol)
      REFERENCES general_accc.rol (id) MATCH SIMPLE
      ON UPDATE RESTRICT ON DELETE RESTRICT
);



--
-- Nombre: enum_voz; Tipo: ENUM; Schema: general_accc; Tablespace: tb_global
--
CREATE TYPE general_accc.enum_voz AS ENUM ('Tenor', 'Bajo', 'Segunda', 'Octavilla', 'Contralto', 'Barítono');

--
-- Nombre: voz; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.voz (
    rol integer NOT NULL,
    tipo_voz general_accc.enum_voz NOT NULL,

    CONSTRAINT pk_voz PRIMARY KEY (rol) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT fk_voz_rol FOREIGN KEY (rol)
      REFERENCES general_accc.rol (id) MATCH SIMPLE
      ON UPDATE RESTRICT ON DELETE RESTRICT
);



--
-- Nombre: orquesta; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.orquesta (
    rol integer NOT NULL,
    instrumento integer NOT NULL,

    CONSTRAINT pk_orquesta PRIMARY KEY (rol) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT fk_orquesta_rol FOREIGN KEY (rol)
      REFERENCES general_accc.rol (id) MATCH SIMPLE
      ON UPDATE RESTRICT ON DELETE RESTRICT,

    CONSTRAINT fk_orquesta_instrumento FOREIGN KEY (instrumento)
      REFERENCES general_accc.instrumento (id) MATCH SIMPLE
      ON UPDATE RESTRICT ON DELETE RESTRICT
);



--
-- Nombre: empresa; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.empresa (
    entidad integer NOT NULL,
    nombre character varying(50) NOT NULL,
    localidad character varying(50) NOT NULL,
    provincia character varying(50) NOT NULL,
    cif character varying(10),
    domicilio character varying(100),
    telefono character varying(30),
    contacto character varying(100),

    CONSTRAINT pk_empresa PRIMARY KEY (entidad) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT uq_empresa_nombrelocalidadprovincia UNIQUE (nombre, localidad, provincia) USING INDEX TABLESPACE tb_idx,
    CONSTRAINT uq_empresa_cif UNIQUE (cif) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT chk_empresa_nombre CHECK (char_length(trim(nombre)) <> 0),
    CONSTRAINT chk_empresa_localidad CHECK (char_length(trim(localidad)) <> 0),
    CONSTRAINT chk_empresa_provincia CHECK (char_length(trim(provincia)) <> 0),

    CONSTRAINT fk_empresa_entidad FOREIGN KEY (entidad)
      REFERENCES general_accc.entidad (id) MATCH SIMPLE
      ON UPDATE RESTRICT ON DELETE RESTRICT
);



--
-- Nombre: actividad; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.actividad (
    id integer NOT NULL DEFAULT nextval('general_accc.seq_actividad'::regclass),
    actividad character varying(50) NOT NULL,

    CONSTRAINT pk_actividad PRIMARY KEY (id) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT uq_actividad_actividad UNIQUE (actividad) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT chk_actividad_actividad CHECK (char_length(trim(actividad)) <> 0)
);



--
-- Nombre: actividad_empresa; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.actividad_empresa (
    id integer NOT NULL DEFAULT nextval('general_accc.seq_actividad_empresa'::regclass),
    actividad integer NOT NULL,
    empresa integer NOT NULL,

    CONSTRAINT pk_actividadempresa PRIMARY KEY (id) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT uq_actividadempresa_actividadempresa UNIQUE (actividad, empresa) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT fk_actividadempresa_actividad FOREIGN KEY (actividad)
      REFERENCES general_accc.actividad (id) MATCH SIMPLE
      ON UPDATE RESTRICT ON DELETE RESTRICT,

    CONSTRAINT fk_actividadempresa_empresa FOREIGN KEY (empresa)
      REFERENCES general_accc.empresa (entidad) MATCH SIMPLE
      ON UPDATE RESTRICT ON DELETE CASCADE
);



--
-- Nombre: enum_tipodocumento; Tipo: ENUM; Schema: general_accc; Tablespace: tb_global
--
CREATE TYPE general_accc.enum_tipodocumento AS ENUM ('Texto','Imagen','Audio','Video','Expediente','Serie','Fondo','Colección');

--
-- Nombre: enum_soporte; Tipo: ENUM; Schema: general_accc; Tablespace: tb_global
--
CREATE TYPE general_accc.enum_tiposoporte AS ENUM ('No disponible','Papel', 'Libro', 'Electrónico', 'Fotografía', 'Negativo', 'Casete', 'CD', 'Vinilo SP', 'Vinilo LP', 'VHS', 'DVD');

--
-- Nombre: enum_niveldescripcion; Tipo: ENUM; Schema: general_accc; Tablespace: tb_global
--
CREATE TYPE general_accc.enum_niveldescripcion AS ENUM ('Fondo', 'Subfondo', 'Serie', 'Unidad documental compuesta', 'Unidad documental simple');

--
-- Nombre: documento; Tipo: TABLE; Schema: general_accc; Tablespace: tb_documentos
--
CREATE TABLE general_accc.documento (
    id integer NOT NULL DEFAULT nextval('general_accc.seq_documento'::regclass),
    "3_1_1_codigo_referencia" character varying(100) DEFAULT 'ES.ACCC//'::character varying,
    "3_1_2_titulo" character varying(500) NOT NULL,
    "3_1_3_fecha_s" character varying(300) NOT NULL,
    tipo_documento general_accc.enum_tipodocumento NOT NULL,
    tipo_soporte general_accc.enum_tiposoporte NOT NULL,
    ubicacion character varying(50),
    deposito character varying(50),
    estante character varying(50),
    url character varying(500),
    "3_1_4_nivel_descripcion" general_accc.enum_niveldescripcion NOT NULL,
    "3_1_5_volumen_soporte" text,
    "3_2_1_nombre_s_productor_es" text,
    "3_2_2_historia_institucional_resenia_biografica" text,
    "3_2_3_historia_archivistica" text,
    "3_2_4_forma_ingreso" text,
    "3_3_1_alcance_contenido" text,
    "3_3_2_valoracion_seleccion_eliminacion" text,
    "3_3_3_nuevos_ingresos" text,
    "3_3_4_organizacion" text,
    "3_4_1_condiciones_acceso" text,
    "3_4_2_condiciones_reproduccion" text,
    "3_4_3_lengua_documentos" text,
    "3_4_4_caracteristicas_fisicas_requisitos_tecnicos" text,
    "3_4_5_instrumentos_descripcion" text,
    "3_5_1_existencia_localizacion_documentos_originales" text,
    "3_5_2_existencia_localizacion_copias" text,
    "3_5_3_unidades_descripcion_relacionadas" text,
    "3_5_4_nota_publicaciones" text,
    "3_6_1_notas" text,
    "3_7_1_nota_archivero" text,
    "3_7_2_reglas_normas" text,
    "3_7_3_fecha_s_descripcion_es" text,
    esta_descrito boolean NOT NULL DEFAULT false,
    esta_digitalizado boolean NOT NULL DEFAULT false,
    en_deposito boolean NOT NULL DEFAULT false,
    observaciones_gestion text,

    CONSTRAINT pk_documento PRIMARY KEY (id) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT uq_documento_codigoreferencia UNIQUE ("3_1_1_codigo_referencia") USING INDEX TABLESPACE tb_idx,

    CONSTRAINT chk_documento_titulo CHECK (char_length(trim("3_1_2_titulo")) <> 0),
    CONSTRAINT chk_documento_fecha CHECK (char_length(trim("3_1_3_fecha_s")) <> 0)

) TABLESPACE tb_documentos;



--
-- Nombre: descriptor; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.descriptor (
    id integer NOT NULL DEFAULT nextval('general_accc.seq_descriptor'::regclass),
    descriptor character varying(30) NOT NULL,

    CONSTRAINT pk_descriptor PRIMARY KEY (id) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT uq_descriptor_descriptor UNIQUE (descriptor) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT chk_descriptor_descriptor CHECK (char_length(trim(descriptor)) <> 0)
);



--
-- Nombre: descriptor_documento; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.descriptor_documento (
    id integer NOT NULL DEFAULT nextval('general_accc.seq_descriptor_documento'::regclass),
    descriptor integer NOT NULL,
    documento integer NOT NULL,

    CONSTRAINT pk_descriptordocumento PRIMARY KEY (id) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT uq_descriptordocumento_descriptordocumento UNIQUE (descriptor, documento) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT fk_descriptordocumento_descriptor FOREIGN KEY (descriptor)
      REFERENCES general_accc.descriptor (id) MATCH SIMPLE
      ON UPDATE RESTRICT ON DELETE RESTRICT,

    CONSTRAINT fk_descriptordocumento_documento FOREIGN KEY (documento)
      REFERENCES general_accc.documento (id) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE CASCADE
);



--
-- Nombre: deposito_legal; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.deposito_legal (
    id integer NOT NULL DEFAULT nextval('general_accc.seq_depositolegal'::regclass),
    documento integer NOT NULL,
    deposito_legal character varying(50) NOT NULL,
    edicion character varying(10),
    anio integer,
    generador character varying(200),

    CONSTRAINT pk_depositolegal PRIMARY KEY (id, documento) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT uq_depositolegal_depositolegal UNIQUE (deposito_legal) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT chk_depositolegal_depositolegal CHECK (char_length(trim(deposito_legal)) <> 0),
    CONSTRAINT chk_depositolegal_anio CHECK (anio > 0  AND  anio < 10000),

    CONSTRAINT fk_depositolegal_documento FOREIGN KEY (documento)
      REFERENCES general_accc.documento (id) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE CASCADE
);



--
-- Nombre: documento_evento; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.documento_evento (
    documento integer NOT NULL,
    evento integer NOT NULL,

    CONSTRAINT pk_documentoevento PRIMARY KEY (documento, evento) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT fk_documentoevento FOREIGN KEY (documento)
      REFERENCES general_accc.documento (id) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE RESTRICT,

    CONSTRAINT fk_documentoevento_evento FOREIGN KEY (evento)
      REFERENCES general_accc.evento (id) MATCH SIMPLE
      ON UPDATE RESTRICT ON DELETE RESTRICT
);



--
-- Nombre: enum_obra; Tipo: ENUM; Schema: general_accc; Tablespace: tb_global
--
CREATE TYPE general_accc.enum_obra AS ENUM ('Literaria', 'Musical', 'Grafica');

--
-- Nombre: obra; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.obra (
    signatura integer NOT NULL DEFAULT nextval('general_accc.seq_obra'::regclass),
    tipo_obra general_accc.enum_obra NOT NULL,

    CONSTRAINT pk_obra PRIMARY KEY (signatura) USING INDEX TABLESPACE tb_idx
);



--
-- Nombre: documento_obra; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.documento_obra (
    documento integer NOT NULL,
    obra integer NOT NULL,

    CONSTRAINT pk_documentoobra PRIMARY KEY (documento, obra) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT fk_documentoobra_documento FOREIGN KEY (documento)
      REFERENCES general_accc.documento (id) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE RESTRICT,

    CONSTRAINT fk_documentoobra_obra FOREIGN KEY (obra)
      REFERENCES general_accc.obra (signatura) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE RESTRICT
);



--
-- Nombre: literaria; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.literaria (
    obra integer NOT NULL,
    repertorio character varying(100),

    CONSTRAINT pk_literaria PRIMARY KEY (obra) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT fk_literaria_obra FOREIGN KEY (obra)
      REFERENCES general_accc.obra (signatura) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE RESTRICT
);



--
-- Nombre: musical; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.musical (
    obra integer NOT NULL,
    repertorio character varying(100),

    CONSTRAINT pk_musical PRIMARY KEY (obra) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT fk_musical_obra FOREIGN KEY (obra)
      REFERENCES general_accc.obra (signatura) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE RESTRICT
);



--
-- Nombre: grafica; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.grafica (
    obra integer NOT NULL,

    CONSTRAINT pk_grafica PRIMARY KEY (obra) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT fk_grafica_obra FOREIGN KEY (obra)
      REFERENCES general_accc.obra (signatura) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE RESTRICT
);



--
-- Nombre: enum_genero_letra; Tipo: ENUM; Schema: general_accc; Tablespace: tb_global
--
CREATE TYPE general_accc.enum_genero_letra AS ENUM ('Presentación', 'Pasodoble', 'Tango', 'Cuplé', 'Popurrí', 'Parodia', 'Pregón');

--
-- Nombre: letra; Tipo: TABLE; Schema: general_accc; Tablespace: tb_letras
--
CREATE TABLE general_accc.letra (
    id integer NOT NULL DEFAULT nextval('general_accc.seq_letra'::regclass),
    obra_literaria integer NOT NULL,
    signatura integer NOT NULL,
    autoria_letra integer NOT NULL,
    genero general_accc.enum_genero_letra NOT NULL,
    transcripcion text,

    CONSTRAINT pk_letra PRIMARY KEY (id) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT uq_letra_obrasignatura UNIQUE (obra_literaria, signatura) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT fk_letra_obraliteraria FOREIGN KEY (obra_literaria)
      REFERENCES general_accc.literaria (obra) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE RESTRICT,

    CONSTRAINT fk_letra_autoria FOREIGN KEY (autoria_letra)
      REFERENCES general_accc.entidad (id) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE RESTRICT

) TABLESPACE tb_letras;



--
-- Nombre: documento_letra; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.documento_letra (
    documento integer NOT NULL,
    letra integer NOT NULL,

    CONSTRAINT pk_documentoletra PRIMARY KEY (documento) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT fk_documentoletra_documento FOREIGN KEY (documento)
      REFERENCES general_accc.documento (id) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE RESTRICT,

    CONSTRAINT fk_documentoletra_letra FOREIGN KEY (letra)
      REFERENCES general_accc.letra (id) MATCH SIMPLE
      ON UPDATE RESTRICT ON DELETE RESTRICT
);



--
-- Nombre: enum_genero_musica; Tipo: ENUM; Schema: general_accc; Tablespace: tb_global
--
CREATE TYPE general_accc.enum_genero_musica AS ENUM ('Presentación', 'Pasodoble', 'Tango', 'Cuplé', 'Popurrí', 'Rumba', 'Bolero', 'Vals', 'Polca', 'Habanera', 'Kake-Walh');

--
-- Nombre: musica; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.musica (
    id integer NOT NULL DEFAULT nextval('general_accc.seq_musica'::regclass),
    obra_musical integer NOT NULL,
    signatura integer NOT NULL,
    autoria_musica integer NOT NULL,
    genero general_accc.enum_genero_musica NOT NULL,

    CONSTRAINT pk_musica PRIMARY KEY (id) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT uq_musica_obrasignatura UNIQUE (obra_musical, signatura) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT uq_musica_obragenero UNIQUE (obra_musical, genero) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT fk_musica_obramusical FOREIGN KEY (obra_musical)
      REFERENCES general_accc.musical (obra) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE RESTRICT,

    CONSTRAINT fk_musica_autoria FOREIGN KEY (autoria_musica)
      REFERENCES general_accc.entidad (id) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE RESTRICT
);



--
-- Nombre: documento_musica; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.documento_musica (
    documento integer NOT NULL,
    musica integer NOT NULL,

    CONSTRAINT pk_documentomusica PRIMARY KEY (documento) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT fk_documentomusica_documento FOREIGN KEY (documento)
      REFERENCES general_accc.documento (id) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE RESTRICT,

    CONSTRAINT fk_documentomusica_musica FOREIGN KEY (musica)
      REFERENCES general_accc.musica (id) MATCH SIMPLE
      ON UPDATE RESTRICT ON DELETE RESTRICT
);



--
-- Nombre: enum_disenio; Tipo: ENUM; Schema: general_accc; Tablespace: tb_global
--
CREATE TYPE general_accc.enum_disenio AS ENUM ('Tipo', 'Disfraz', 'Atrezo', 'Forillo', 'Caracterización');

--
-- Nombre: disenio; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.disenio (
    id integer NOT NULL DEFAULT nextval('general_accc.seq_disenio'::regclass),
    obra_grafica integer NOT NULL,
    signatura integer NOT NULL,
    autoria_disenio integer NOT NULL,
    tipo_disenio general_accc.enum_disenio NOT NULL,

    CONSTRAINT pk_disenio PRIMARY KEY (id) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT uq_disenio_obrasignatura UNIQUE (obra_grafica, signatura) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT uq_disenio_obratipodisenio UNIQUE (obra_grafica, tipo_disenio) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT fk_disenio_obragrafica FOREIGN KEY (obra_grafica)
      REFERENCES general_accc.grafica (obra) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE RESTRICT,

    CONSTRAINT fk_disenio_autoria FOREIGN KEY (autoria_disenio)
      REFERENCES general_accc.entidad (id) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE RESTRICT
);



--
-- Nombre: documento_disenio; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.documento_disenio (
    documento integer NOT NULL,
    disenio integer NOT NULL,

    CONSTRAINT pk_documentodisenio PRIMARY KEY (documento) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT fk_documentodisenio_documento FOREIGN KEY (documento)
      REFERENCES general_accc.documento (id) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE RESTRICT,

    CONSTRAINT fk_documentodisenio_disenio FOREIGN KEY (disenio)
      REFERENCES general_accc.disenio (id) MATCH SIMPLE
      ON UPDATE RESTRICT ON DELETE RESTRICT
);



--
-- Nombre: tema; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.tema (
    id integer NOT NULL DEFAULT nextval('general_accc.seq_tema'::regclass),
    tema character varying (50) NOT NULL,
    descripcion character varying (500),

    CONSTRAINT pk_tema PRIMARY KEY (id) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT uq_tema_tema UNIQUE (tema) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT chk_tema_tema CHECK (char_length(trim(tema)) <> 0)
);



--
-- Nombre: tema_letra; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.tema_letra (
    id integer NOT NULL DEFAULT nextval('general_accc.seq_tema_letra'::regclass),
    tema integer NOT NULL,
    letra integer NOT NULL,

    CONSTRAINT pk_temaletra PRIMARY KEY (id) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT uq_temaletra_temaletra UNIQUE (tema, letra) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT fk_temaletra_tema FOREIGN KEY (tema)
      REFERENCES general_accc.tema (id) MATCH SIMPLE
      ON UPDATE RESTRICT ON DELETE RESTRICT,

    CONSTRAINT fk_temaletra_letra FOREIGN KEY (letra)
      REFERENCES general_accc.letra (id) MATCH SIMPLE
      ON UPDATE RESTRICT ON DELETE CASCADE
);



--
-- Nombre: tema_disenio; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.tema_disenio (
    id integer NOT NULL DEFAULT nextval('general_accc.seq_tema_disenio'::regclass),
    tema integer NOT NULL,
    disenio integer NOT NULL,

    CONSTRAINT pk_temadisenio PRIMARY KEY (id) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT uq_temadisenio_temadisenio UNIQUE (tema, disenio) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT fk_temadisenio_tema FOREIGN KEY (tema)
      REFERENCES general_accc.tema (id) MATCH SIMPLE
      ON UPDATE RESTRICT ON DELETE RESTRICT,

    CONSTRAINT fk_temadisenio_disenio FOREIGN KEY (disenio)
      REFERENCES general_accc.disenio (id) MATCH SIMPLE
      ON UPDATE RESTRICT ON DELETE CASCADE
);



--
-- Nombre: crea; Tipo: TABLE; Schema: general_accc; Tablespace: tb_global
--
CREATE TABLE general_accc.crea (
    id integer NOT NULL DEFAULT nextval('general_accc.seq_crea'::regclass),
    disenio integer NOT NULL,
    entidad integer NOT NULL,
    coste numeric(10,0) DEFAULT 0.0,
    observaciones text,

    CONSTRAINT pk_crea PRIMARY KEY (id) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT uq_crea_disenioentidad UNIQUE (disenio, entidad) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT chk_crea_coste CHECK (coste > -1.0),

    CONSTRAINT fk_crea_disenio FOREIGN KEY (disenio)
      REFERENCES general_accc.disenio (id) MATCH SIMPLE
      ON UPDATE RESTRICT ON DELETE RESTRICT,

    CONSTRAINT fk_crea_entidad FOREIGN KEY (entidad)
      REFERENCES general_accc.entidad (id) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE RESTRICT
);



--
-- Nombre: agrupacion_anio; Tipo: ALTER TABLE; Schema: general_accc; Tablespace: tb_global
--
-- Se incluyen las claves externas que modelan las relaciones de interpretación.
--

ALTER TABLE general_accc.agrupacion_anio ADD COLUMN interpreta_literaria integer;
ALTER TABLE general_accc.agrupacion_anio ADD CONSTRAINT uq_agrupacionanio_interpretaliteraria UNIQUE (interpreta_literaria) USING INDEX TABLESPACE tb_idx;
ALTER TABLE general_accc.agrupacion_anio ADD CONSTRAINT fk_agrupacionanio_interpretaliteraria FOREIGN KEY (interpreta_literaria)
      REFERENCES general_accc.literaria (obra) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE general_accc.agrupacion_anio ADD COLUMN interpreta_musical integer;
ALTER TABLE general_accc.agrupacion_anio ADD CONSTRAINT uq_agrupacionanio_interpretamusical UNIQUE (interpreta_musical) USING INDEX TABLESPACE tb_idx;
ALTER TABLE general_accc.agrupacion_anio ADD CONSTRAINT fk_agrupacionanio_interpretamusical FOREIGN KEY (interpreta_musical)
      REFERENCES general_accc.musical (obra) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE general_accc.agrupacion_anio ADD COLUMN utiliza_tipo integer;
ALTER TABLE general_accc.agrupacion_anio ADD CONSTRAINT uq_agrupacionanio_utilizatipo UNIQUE (utiliza_tipo) USING INDEX TABLESPACE tb_idx;
ALTER TABLE general_accc.agrupacion_anio ADD CONSTRAINT fk_agrupacionanio_utilizatipo FOREIGN KEY (utiliza_tipo)
      REFERENCES general_accc.grafica (obra) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE RESTRICT;
