-- TFG BBDD Carnaval de Cádiz - Paloma Cortés


-- Procedimientos almacenados para altas, baja y modificación en las tablas de
-- soporte: Modalidad, Descriptor, Tema, Instrumento y Actividad.




--
-- Nombre: alta_modalidad; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.alta_modalidad (in_modalidad character varying)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
BEGIN

    IF in_modalidad IS NULL OR char_length(trim(in_modalidad)) = 0 THEN
        RAISE EXCEPTION 'La modalidad es un valor obligatorio';
    END IF;

    SELECT id INTO auxInteger FROM general_accc.modalidad WHERE upper(modalidad) LIKE '%' || upper(in_modalidad) || '%';
    IF NOT FOUND THEN
        INSERT INTO general_accc.modalidad (modalidad) VALUES (in_modalidad);
        RETURN 'OK';
    END IF;

    RAISE EXCEPTION 'La modalidad ya existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;



--
-- Nombre: modifica_modalidad; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.modifica_modalidad (in_id integer,
                                                            in_modalidad character varying)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
BEGIN

    SELECT id INTO auxInteger FROM general_accc.modalidad WHERE id = in_id;
    IF FOUND THEN

        SELECT id INTO auxInteger
            FROM general_accc.modalidad
            WHERE upper(modalidad) LIKE '%' || upper(in_modalidad) || '%'
                AND id <> in_id;
        IF NOT FOUND THEN
            UPDATE general_accc.modalidad SET modalidad = in_modalidad WHERE id = in_id;
            RETURN 'OK';
        ELSE
            RAISE EXCEPTION 'La modalidad indicada ya existe.';
        END IF;
    END IF;

    RAISE EXCEPTION 'El identificador de la modalidad no existe o no se ha indicado.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;



--
-- Nombre: baja_modalidad; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_modalidad (in_id integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
BEGIN

    SELECT id INTO auxInteger FROM general_accc.modalidad WHERE id = in_id;
    IF FOUND THEN
        DELETE FROM general_accc.modalidad WHERE id = in_id;
        RETURN 'OK';
    END IF;

    RAISE EXCEPTION 'No existe la modalidad con identificador: %.', in_id;

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;



--
-- Nombre: alta_descriptor; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.alta_descriptor (in_descriptor character varying)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
BEGIN

    SELECT id INTO auxInteger FROM general_accc.descriptor WHERE upper(descriptor) LIKE '%' || upper(in_descriptor) || '%';
    IF NOT FOUND THEN
        INSERT INTO general_accc.descriptor (descriptor) VALUES (in_descriptor);
        RETURN 'OK';
    END IF;

    RAISE EXCEPTION 'El descriptor ya existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;



--
-- Nombre: modifica_descriptor; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.modifica_descriptor (in_id integer,
                                                             in_descriptor character varying)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
BEGIN

    SELECT id INTO auxInteger FROM general_accc.descriptor WHERE id = in_id;
    IF FOUND THEN
        SELECT id INTO auxInteger
            FROM general_accc.descriptor
            WHERE upper(descriptor) LIKE '%' || upper(in_descriptor) || '%'
                AND id <> in_id;
        IF NOT FOUND THEN
            UPDATE general_accc.descriptor SET descriptor = in_descriptor WHERE id = in_id;
            RETURN 'OK';
        ELSE
            RAISE EXCEPTION 'El descriptor indicado ya existe.';
        END IF;
    END IF;

    RAISE EXCEPTION 'El identificador del descriptor no existe o no se ha indicado.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;



--
-- Nombre: baja_descriptor; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_descriptor (in_id integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
BEGIN

    SELECT id INTO auxInteger FROM general_accc.descriptor WHERE id = in_id;
    IF FOUND THEN
        DELETE FROM general_accc.descriptor WHERE id = in_id;
        RETURN 'OK';
    END IF;

    RAISE EXCEPTION 'No existe el descriptor con identificador: %.', in_id;

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;



--
-- Nombre: alta_tema; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.alta_tema (in_tema character varying,
                                                   in_descripcion character varying)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
BEGIN

    SELECT id INTO auxInteger FROM general_accc.tema WHERE upper(tema) LIKE '%' || upper(in_tema) || '%';
    IF NOT FOUND THEN
        INSERT INTO general_accc.tema (tema, descripcion) VALUES (in_tema, in_descripcion);
        RETURN 'OK';
    END IF;

    RAISE EXCEPTION 'El tema ya existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;



--
-- Nombre: modifica_tema; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.modifica_tema (in_id integer,
                                                       in_tema character varying,
                                                       in_descripcion character varying)
RETURNS character varying AS $$
DECLARE
    was general_accc.tema%ROWTYPE;
DECLARE
    auxInteger integer;
BEGIN

    SELECT * INTO was FROM general_accc.tema WHERE id = in_id;
    IF FOUND AND in_tema IS NOT NULL AND char_length(trim(in_tema)) <> 0 THEN
        SELECT id INTO auxInteger
            FROM general_accc.tema
            WHERE upper(tema) LIKE '%' || upper(in_tema) || '%'
                AND id <> in_id;
        IF NOT FOUND THEN
            UPDATE general_accc.tema SET
                                     tema = COALESCE(in_tema, was.tema),
                                     descripcion = COALESCE(in_descripcion, was.descripcion)
                WHERE id = in_id;
            RETURN 'OK';
        ELSE
            RAISE EXCEPTION 'El tema indicado ya existe.';
        END IF;
    END IF;

    RAISE EXCEPTION 'El identificador del tema no existe o no se ha indicado.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;



--
-- Nombre: baja_tema; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_tema (in_id integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
BEGIN

    SELECT id INTO auxInteger FROM general_accc.tema WHERE id = in_id;
    IF FOUND THEN
        DELETE FROM general_accc.tema WHERE id = in_id;
        RETURN 'OK';
    END IF;

    RAISE EXCEPTION 'No existe el tema con identificador: %.', in_id;

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: alta_instrumento; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.alta_instrumento (in_instrumento character varying)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
BEGIN

    IF in_instrumento IS NULL OR char_length(trim(in_instrumento)) = 0 THEN
        RAISE EXCEPTION 'Instrumento es un valor obligatorio';
    END IF;

    SELECT id INTO auxInteger FROM general_accc.instrumento WHERE upper(instrumento) LIKE '%' || upper(in_instrumento) || '%';
    IF NOT FOUND THEN
        INSERT INTO general_accc.instrumento (instrumento) VALUES (in_instrumento);
        RETURN 'OK';
    END IF;

    RAISE EXCEPTION 'El instrumento ya existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: modifica_instrumento; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.modifica_instrumento (in_id integer,
                                                              in_instrumento character varying)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
BEGIN

    SELECT id INTO auxInteger FROM general_accc.instrumento WHERE id = in_id;
    IF FOUND THEN
        SELECT id INTO auxInteger
            FROM general_accc.instrumento
            WHERE upper(instrumento) LIKE '%' || upper(in_instrumento) || '%'
                AND id <> in_id;
        IF NOT FOUND THEN
            UPDATE general_accc.instrumento SET
                                            instrumento = in_instrumento
                WHERE id = in_id;
            RETURN 'OK';
        END IF;
        RAISE EXCEPTION 'El instrumento indicado ya existe.';
    END IF;

    RAISE EXCEPTION 'El identificador del instrumento no existe o no se ha indicado.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: baja_instrumento; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_instrumento (in_id integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
BEGIN

    SELECT id INTO auxInteger FROM general_accc.instrumento WHERE id = in_id;
    IF FOUND THEN
        DELETE FROM general_accc.instrumento WHERE id = in_id;
        RETURN 'OK';
    END IF;

    RAISE EXCEPTION 'No existe el instrumento con identificador: %.', in_id;

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: alta_actividad; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.alta_actividad (in_actividad character varying)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
BEGIN

    SELECT id INTO auxInteger FROM general_accc.actividad WHERE upper(actividad) LIKE '%' || upper(in_actividad) || '%';
    IF NOT FOUND THEN
        INSERT INTO general_accc.actividad (actividad) VALUES (in_actividad);
        RETURN 'OK';
    END IF;

    RAISE EXCEPTION 'La actividad ya existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: modifica_actividad; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.modifica_actividad (in_id integer,
                                                            in_actividad character varying)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
BEGIN

    SELECT id INTO auxInteger FROM general_accc.actividad WHERE id = in_id;
    IF FOUND THEN
        SELECT id INTO auxInteger
            FROM general_accc.actividad
            WHERE upper(actividad) LIKE '%' || upper(in_actividad) || '%'
                AND id <> in_id;
        IF NOT FOUND THEN
            UPDATE general_accc.actividad SET actividad = in_actividad WHERE id = in_id;
            RETURN 'OK';
        ELSE
            RAISE EXCEPTION 'La actividad indicada ya existe.';
        END IF;
    END IF;

    RAISE EXCEPTION 'El identificador de la actividad no existe o no se ha indicado.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: baja_actividad; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_actividad (in_id integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
BEGIN

    SELECT id INTO auxInteger FROM general_accc.actividad WHERE id = in_id;
    IF FOUND THEN
        DELETE FROM general_accc.actividad WHERE id = in_id;
        RETURN 'OK';
    END IF;

    RAISE EXCEPTION 'No existe la actividad con identificador: %.', in_id;

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;


