-- TFG BBDD Carnaval de Cádiz - Paloma Cortés


-- Creación de procedimientos almacenados para altas, baja y modificación
-- de la tabla Documento y sus directamente vinculadas, Descriptor y DepositoLegal




--
-- Nombre: __alta_documento; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.__alta_documento (in_id integer,
                                                          in_3_1_1 character varying,
                                                          in_3_1_2 character varying,
                                                          in_3_1_3 character varying,
                                                          in_tipo_documento character varying,
                                                          in_tipo_soporte character varying,
                                                          in_ubicacion character varying,
                                                          in_deposito character varying,
                                                          in_estante character varying,
                                                          in_url character varying,
                                                          in_3_1_4 character varying,
                                                          in_3_1_5 text,
                                                          in_3_2_1 text,
                                                          in_3_2_2 text,
                                                          in_3_2_3 text,
                                                          in_3_2_4 text,
                                                          in_3_3_1 text,
                                                          in_3_3_2 text,
                                                          in_3_3_3 text,
                                                          in_3_3_4 text,
                                                          in_3_4_1 text,
                                                          in_3_4_2 text,
                                                          in_3_4_3 text,
                                                          in_3_4_4 text,
                                                          in_3_4_5 text,
                                                          in_3_5_1 text,
                                                          in_3_5_2 text,
                                                          in_3_5_3 text,
                                                          in_3_5_4 text,
                                                          in_3_6_1 text,
                                                          in_3_7_1 text,
                                                          in_3_7_2 text,
                                                          in_3_7_3 text,
                                                          in_descriptor integer,
                                                          in_esta_descrito boolean,
                                                          in_esta_digitalizado boolean,
                                                          in_en_deposito boolean,
                                                          in_observaciones_gestion text)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
BEGIN
    SELECT id INTO auxInteger FROM general_accc.documento WHERE id = in_id;
    IF NOT FOUND THEN

        IF in_esta_descrito IS NULL THEN
            in_esta_descrito := false;
        END IF;
        IF in_esta_digitalizado IS NULL THEN
            in_esta_digitalizado := false;
        END IF;
        IF in_en_deposito IS NULL THEN
            in_en_deposito := false;
        END IF;

        -- Aquí llega obligatoriamente el identificador de documento y el título,
        -- de forma que se fuerza la llamada desde los procedimientos
        -- almacenados de evento y obra, lo que es necesario para que envíen el
        -- Código de referencia (3.1.1) compuesto
        IF in_id IS NULL OR in_3_1_1 IS NULL OR char_length(trim(in_3_1_1)) = 0 THEN
            RAISE EXCEPTION 'No se ha recibido el identificador o el título del documento.';
        END IF;
        IF in_3_1_2 IS NULL OR char_length(trim(in_3_1_2)) = 0 THEN
            RAISE EXCEPTION 'El título es obligatorio.';
        END IF;
        IF in_descriptor IS NULL THEN
            RAISE EXCEPTION 'El descriptor es obligatorio.';
        END IF;

        IF in_esta_digitalizado AND (in_url IS NULL OR char_length(trim(in_url)) = 0) THEN
            RAISE EXCEPTION 'La URL es valor obligatorio si se ha indicado que está digitalizado.';
        END IF;

        IF in_en_deposito AND
                         (in_ubicacion IS NULL OR char_length(trim(in_ubicacion)) = 0) AND
                         (in_deposito IS NULL OR char_length(trim(in_deposito)) = 0) AND
                         (in_estante IS NULL OR char_length(trim(in_estante)) = 0) THEN
            RAISE EXCEPTION 'Algún dato sobre la localización es obligatorio si se ha indicado en depósito.';

        ELSEIF (in_esta_digitalizado OR in_en_deposito)
                AND in_tipo_soporte = 'No disponible' THEN
            RAISE EXCEPTION 'Se debe indicar el tipo de soporte si se cuenta con un documento tangible.';
        END IF;

        IF in_3_1_4 = 'Unidad documental simple'
                AND (in_tipo_documento = 'Expediente' OR
                     in_tipo_documento = 'Serie' OR
                     in_tipo_documento = 'Fondo' OR
                     in_tipo_documento = 'Colección') THEN
            RAISE EXCEPTION 'Las unidades documentales simples solo se pueden vincular a los tipos: Texto, Imagen, Audio y Video.';

        END IF;

        BEGIN
            INSERT INTO general_accc.documento (id,
                                                "3_1_1_codigo_referencia",
                                                "3_1_2_titulo",
                                                "3_1_3_fecha_s",
                                                tipo_documento,
                                                tipo_soporte,
                                                ubicacion,
                                                deposito,
                                                estante,
                                                url,
                                                "3_1_4_nivel_descripcion",
                                                "3_1_5_volumen_soporte",
                                                "3_2_1_nombre_s_productor_es",
                                                "3_2_2_historia_institucional_resenia_biografica",
                                                "3_2_3_historia_archivistica",
                                                "3_2_4_forma_ingreso",
                                                "3_3_1_alcance_contenido",
                                                "3_3_2_valoracion_seleccion_eliminacion",
                                                "3_3_3_nuevos_ingresos",
                                                "3_3_4_organizacion",
                                                "3_4_1_condiciones_acceso",
                                                "3_4_2_condiciones_reproduccion",
                                                "3_4_3_lengua_documentos",
                                                "3_4_4_caracteristicas_fisicas_requisitos_tecnicos",
                                                "3_4_5_instrumentos_descripcion",
                                                "3_5_1_existencia_localizacion_documentos_originales",
                                                "3_5_2_existencia_localizacion_copias",
                                                "3_5_3_unidades_descripcion_relacionadas",
                                                "3_5_4_nota_publicaciones",
                                                "3_6_1_notas",
                                                "3_7_1_nota_archivero",
                                                "3_7_2_reglas_normas",
                                                "3_7_3_fecha_s_descripcion_es",
                                                esta_descrito,
                                                esta_digitalizado,
                                                en_deposito,
                                                observaciones_gestion)
                VALUES (in_id, in_3_1_1, in_3_1_2, in_3_1_3,
                        in_tipo_documento::general_accc.enum_tipodocumento,
                        in_tipo_soporte::general_accc.enum_tiposoporte,
                        in_ubicacion, in_deposito, in_estante, in_url,
                        in_3_1_4::general_accc.enum_niveldescripcion, in_3_1_5,
                        in_3_2_1, in_3_2_2, in_3_2_3, in_3_2_4, in_3_3_1, in_3_3_2,
                        in_3_3_3, in_3_3_4, in_3_4_1, in_3_4_2, in_3_4_3, in_3_4_4,
                        in_3_4_5, in_3_5_1, in_3_5_2, in_3_5_3, in_3_5_4, in_3_6_1,
                        in_3_7_1, in_3_7_2, in_3_7_3, in_esta_descrito, in_esta_digitalizado,
                    	in_en_deposito, in_observaciones_gestion);

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha creado el documento. (%) %', SQLSTATE, SQLERRM;
        END;

        BEGIN
            SELECT general_accc.alta_descriptor_documento(in_descriptor, in_id)
                INTO auxCharacter;
        EXCEPTION WHEN others THEN
            RAISE EXCEPTION '%', SQLERRM;
        END;

        RETURN 'OK';

    END IF;

    RAISE EXCEPTION 'El identificador del documento ya existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: __modifica_documento; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.__modifica_documento (in_id integer,
                                                              in_3_1_1 character varying,
                                                              in_3_1_2 character varying,
                                                              in_3_1_3 character varying,
                                                              in_tipo_documento character varying,
                                                              in_tipo_soporte character varying,
                                                              in_ubicacion character varying,
                                                              in_deposito character varying,
                                                              in_estante character varying,
                                                              in_url character varying,
                                                              in_3_1_4 character varying,
                                                              in_3_1_5 text,
                                                              in_3_2_1 text,
                                                              in_3_2_2 text,
                                                              in_3_2_3 text,
                                                              in_3_2_4 text,
                                                              in_3_3_1 text,
                                                              in_3_3_2 text,
                                                              in_3_3_3 text,
                                                              in_3_3_4 text,
                                                              in_3_4_1 text,
                                                              in_3_4_2 text,
                                                              in_3_4_3 text,
                                                              in_3_4_4 text,
                                                              in_3_4_5 text,
                                                              in_3_5_1 text,
                                                              in_3_5_2 text,
                                                              in_3_5_3 text,
                                                              in_3_5_4 text,
                                                              in_3_6_1 text,
                                                              in_3_7_1 text,
                                                              in_3_7_2 text,
                                                              in_3_7_3 text,
                                                              in_esta_descrito boolean,
                                                              in_esta_digitalizado boolean,
                                                              in_en_deposito boolean,
                                                              in_observaciones_gestion text)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
    was general_accc.documento%ROWTYPE;
BEGIN

    SELECT * INTO was FROM general_accc.documento WHERE id = in_id;
    IF FOUND THEN


        IF in_esta_digitalizado AND (in_url IS NOT NULL AND char_length(trim(in_url)) = 0) THEN
            RAISE EXCEPTION 'La URL es valor obligatorio si se ha indicado que esta digitalizado.';
        END IF;

        IF in_en_deposito AND
                         (
                            (in_ubicacion IS NOT NULL AND char_length(trim(in_ubicacion)) = 0) OR
                            (in_deposito IS NOT NULL AND char_length(trim(in_deposito)) = 0) OR
                            (in_estante IS NOT NULL AND char_length(trim(in_estante)) = 0)
                         ) THEN
            -- Con este condicional se trata de controlar si alguno de los
            -- campos es recibido vacio

            RAISE EXCEPTION 'Algún dato sobre la localización es obligatorio si se ha indicado en depósito.';

        ELSEIF (in_esta_digitalizado OR in_en_deposito)
                        AND in_tipo_soporte IS NOT NULL AND in_tipo_soporte = 'No disponible' THEN
            RAISE EXCEPTION 'Se debe indicar el tipo de soporte si se cuenta con un documento tangible.';

        ELSEIF in_3_1_4 IS NOT NULL AND in_3_1_4 = 'Unidad documental simple' AND
                        (in_tipo_documento IS NOT NULL AND in_tipo_documento = 'Expediente' OR
                         in_tipo_documento IS NOT NULL AND in_tipo_documento = 'Serie' OR
                         in_tipo_documento IS NOT NULL AND in_tipo_documento = 'Fondo' OR
                         in_tipo_documento IS NOT NULL AND in_tipo_documento = 'Colección'
                        ) THEN
            RAISE EXCEPTION 'Las unidades documentales simples solo se pueden vincular a los tipos: Texto, Imagen, Audio y Video.';

        END IF;

        BEGIN
            UPDATE general_accc.documento SET
                                          "3_1_1_codigo_referencia" = COALESCE(in_3_1_1, was."3_1_1_codigo_referencia"),
                                          "3_1_2_titulo" = COALESCE(in_3_1_2, was."3_1_2_titulo"),
                                          "3_1_3_fecha_s" = COALESCE(in_3_1_3, was."3_1_3_fecha_s"),
                                          tipo_documento = COALESCE(in_tipo_documento::general_accc.enum_tipodocumento, was.tipo_documento),
                                          tipo_soporte = COALESCE(in_tipo_soporte::general_accc.enum_tiposoporte, was.tipo_soporte),
                                          ubicacion = COALESCE(in_ubicacion, was.ubicacion),
                                          deposito = COALESCE(in_deposito, was.deposito),
                                          estante = COALESCE(in_estante, was.estante),
                                          url = COALESCE(in_url, was.url),
                                          "3_1_4_nivel_descripcion" = COALESCE(in_3_1_4::general_accc.enum_niveldescripcion, was."3_1_4_nivel_descripcion"),
                                          "3_1_5_volumen_soporte" = COALESCE(in_3_1_5, was."3_1_5_volumen_soporte"),
                                          "3_2_1_nombre_s_productor_es" = COALESCE(in_3_2_1, was."3_2_1_nombre_s_productor_es"),
                                          "3_2_2_historia_institucional_resenia_biografica" = COALESCE(in_3_2_2, was."3_2_2_historia_institucional_resenia_biografica"),
                                          "3_2_3_historia_archivistica" = COALESCE(in_3_2_3, was."3_2_3_historia_archivistica"),
                                          "3_2_4_forma_ingreso" = COALESCE(in_3_2_4, was."3_2_4_forma_ingreso"),
                                          "3_3_1_alcance_contenido" = COALESCE(in_3_3_1, was."3_3_1_alcance_contenido"),
                                          "3_3_2_valoracion_seleccion_eliminacion" = COALESCE(in_3_3_2, was."3_3_2_valoracion_seleccion_eliminacion"),
                                          "3_3_3_nuevos_ingresos" = COALESCE(in_3_3_3, was."3_3_3_nuevos_ingresos"),
                                          "3_3_4_organizacion" = COALESCE(in_3_3_4, was."3_3_4_organizacion"),
                                          "3_4_1_condiciones_acceso" = COALESCE(in_3_4_1, was."3_4_1_condiciones_acceso"),
                                          "3_4_2_condiciones_reproduccion" = COALESCE(in_3_4_2, was."3_4_2_condiciones_reproduccion"),
                                          "3_4_3_lengua_documentos" = COALESCE(in_3_4_3, was."3_4_3_lengua_documentos"),
                                          "3_4_4_caracteristicas_fisicas_requisitos_tecnicos" = COALESCE(in_3_4_4, was."3_4_4_caracteristicas_fisicas_requisitos_tecnicos"),
                                          "3_4_5_instrumentos_descripcion" = COALESCE(in_3_4_5, was."3_4_5_instrumentos_descripcion"),
                                          "3_5_1_existencia_localizacion_documentos_originales" = COALESCE(in_3_5_1, was."3_5_1_existencia_localizacion_documentos_originales"),
                                          "3_5_2_existencia_localizacion_copias" = COALESCE(in_3_5_2, was."3_5_2_existencia_localizacion_copias"),
                                          "3_5_3_unidades_descripcion_relacionadas" = COALESCE(in_3_5_3, was."3_5_3_unidades_descripcion_relacionadas"),
                                          "3_5_4_nota_publicaciones" = COALESCE(in_3_5_4, was."3_5_4_nota_publicaciones"),
                                          "3_6_1_notas" = COALESCE(in_3_6_1, was."3_6_1_notas"),
                                          "3_7_1_nota_archivero" = COALESCE(in_3_7_1, was."3_7_1_nota_archivero"),
                                          "3_7_2_reglas_normas" = COALESCE(in_3_7_2, was."3_7_2_reglas_normas"),
                                          "3_7_3_fecha_s_descripcion_es" = COALESCE(in_3_7_3, was."3_7_3_fecha_s_descripcion_es"),
                                          esta_descrito = COALESCE(in_esta_descrito, was.esta_descrito),
                                          esta_digitalizado = COALESCE(in_esta_digitalizado, was.esta_digitalizado),
                                          en_deposito = COALESCE(in_en_deposito, was.en_deposito),
                                          observaciones_gestion = COALESCE(in_observaciones_gestion, was.observaciones_gestion)
                WHERE id = in_id;
                -- Mediante esta función no se actualizan los descriptores
                -- porque podría haber varios
            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha actualizado el documento. (%) %', SQLSTATE, SQLERRM;
        END;

    END IF;
    RAISE EXCEPTION 'El identificador del documento no ha sido recibido o no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: __baja_documento; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.__baja_documento (in_documento integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    was general_accc.documento%ROWTYPE;
BEGIN

    SELECT * INTO was FROM general_accc.documento WHERE id = in_documento;
    IF FOUND THEN

        -- Se eliminará si no aparece en ningún registro de las siguientes relaciones
        -- el vinculo habrá sido retirado por la función que llama a esta

        SELECT COUNT(doc) INTO auxInteger FROM
        (
            SELECT d_e.documento AS doc FROM general_accc.documento_evento AS d_e WHERE d_e.documento = in_documento
            UNION
            SELECT d_o.documento AS doc FROM general_accc.documento_obra AS d_o WHERE d_o.documento = in_documento
            UNION
            SELECT d_l.documento AS doc FROM general_accc.documento_letra AS d_l WHERE d_l.documento = in_documento
            UNION
            SELECT d_m.documento AS doc FROM general_accc.documento_musica AS d_m WHERE d_m.documento = in_documento
            UNION
            SELECT d_d.documento AS doc FROM general_accc.documento_disenio AS d_d WHERE d_d.documento = in_documento
        ) AS auxQuery;

        IF auxInteger = 0 THEN

            -- Si la relación que se ha eliminado era la última se impedirá en
            -- caso de tener algún documento tangible asociado
            IF was.esta_digitalizado THEN
                RAISE EXCEPTION 'La descripción del documento refiere a un documento electrónico localizado.';
            END IF;
            IF was.en_deposito AND
                             (was.ubicacion IS NOT NULL OR was.deposito IS NULL OR was.estante IS NULL) THEN
                RAISE EXCEPTION 'La descripción del documento refiere a un documento físico localizado.';
            END IF;

            -- Si existen registros en deposito_legal evitarán la eliminación
            -- por restricción de integridad referencial

            BEGIN
                DELETE FROM general_accc.documento WHERE id = in_documento;
                -- Los descriptores asociados desaparecen por la restricción
                -- ON DELETE CASCADE
                RETURN 'OK';

            EXCEPTION WHEN others THEN
                RAISE EXCEPTION 'No se ha eliminado el documento. (%) %', SQLSTATE, SQLERRM;
            END;
        ELSE
            -- Este mensaje es informativo de la realidad del documento dentro
            -- de la BBDD, luego las funciones que llaman aquí actuarán según sus
            -- premisas específicas
            RAISE EXCEPTION 'El documento aún refiere a algún elemento del inventario.';

        END IF;
    END IF;

    RAISE EXCEPTION 'El documento no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: modifica_id_documento; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.modifica_id_documento (in_id_nuevo integer,
                                                               in_id_viejo integer)
RETURNS character varying AS $$
DECLARE
    auxCharacter character varying;
BEGIN

    SELECT "3_1_1_codigo_referencia" INTO auxCharacter
        FROM general_accc.documento
        WHERE id = in_id_viejo;

    IF FOUND AND in_id_nuevo < 100000 THEN
                               -- Valor inicial de la secuencia de Documento

        -- Sustituye el identificador en el campo "3_1_1_codigo_referencia"
        auxCharacter := replace(auxCharacter, in_id_viejo::character varying, in_id_nuevo::character varying);

     	UPDATE general_accc.documento SET
                                      id = in_id_nuevo,
                                      "3_1_1_codigo_referencia" = auxCharacter
            WHERE id = in_id_viejo;

        RETURN 'OK';
    END IF;
    RAISE EXCEPTION 'El identificador ya está en uso o no puede utilizarse.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: alta_depositolegal_documento; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.alta_depositolegal_documento (in_documento integer,
                                                                      in_deposito_legal character varying,
                                                                      in_edicion character varying,
                                                                      in_anio integer,
                                                                      in_generador character varying)
RETURNS character varying AS $$
DECLARE
    b1 boolean;
    b2 boolean;
BEGIN

    SELECT esta_digitalizado, en_deposito INTO b1, b2
        FROM general_accc.documento
        WHERE id = in_documento;

    IF b1 OR b2 THEN

        BEGIN
            INSERT INTO general_accc.deposito_legal(documento, deposito_legal, edicion, anio, generador)
                VALUES (in_documento, in_deposito_legal, in_edicion, in_anio, in_generador);

            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha creado el registro. (%) %', SQLSTATE, SQLERRM;
        END;

    ELSE
        RAISE EXCEPTION 'El documento no tiene representación tangible, electrónica o física.';

    END IF;

    RAISE EXCEPTION 'El depósito legal ya existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: modifica_depositolegal_documento; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.modifica_depositolegal_documento (in_id integer,
                                                                          in_documento integer,
                                                                          in_deposito_legal character varying,
                                                                          in_edicion character varying,
                                                                          in_anio integer,
                                                                          in_generador character varying)
RETURNS character varying AS $$
DECLARE
    was general_accc.deposito_legal%ROWTYPE;
BEGIN

    SELECT * INTO was
        FROM general_accc.deposito_legal
        WHERE id = in_id
            AND documento = in_documento;
    IF FOUND THEN
        BEGIN
            UPDATE general_accc.deposito_legal SET
                                               documento = COALESCE (in_documento, was.documento),
                                               deposito_legal = COALESCE (in_deposito_legal, was.deposito_legal),
                                               edicion = COALESCE (in_edicion, was.edicion),
                                               anio = COALESCE (in_anio, was.anio),
                                               generador = COALESCE (in_generador, was.generador)
                WHERE id = in_id;
            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha actualizado. (%) %', SQLSTATE, SQLERRM;
        END;

    END IF;

    RAISE EXCEPTION 'El depósito legal no existe o el documento es erróneo.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: baja_depositolegal_documento; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_depositolegal_documento (in_id integer,
                                                                      in_documento integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    was general_accc.deposito_legal%ROWTYPE;
BEGIN

    SELECT * INTO was
        FROM general_accc.deposito_legal
        WHERE id = in_id
            AND documento = in_documento;
    IF FOUND THEN
        BEGIN
            DELETE FROM general_accc.deposito_legal WHERE id = in_id;
            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha eliminado el registro. (%) %', SQLSTATE, SQLERRM;
        END;

    END IF;
    RAISE EXCEPTION 'El depósito legal no existe o el documento es erróneo.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;






--
-- Nombre: alta_descriptor_documento; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.alta_descriptor_documento (in_descriptor integer,
                                                                   in_documento integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
BEGIN

    SELECT in_descriptor INTO auxInteger
        FROM general_accc.descriptor_documento
        WHERE descriptor = in_descriptor
            AND documento = in_documento;

    IF NOT FOUND THEN
        BEGIN
            INSERT INTO general_accc.descriptor_documento(descriptor, documento)
                VALUES (in_descriptor, in_documento);

            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha creado la relación entre descriptor y documento. %', SQLERRM;
        END;

    END IF;

    RAISE EXCEPTION 'La relación entre el descriptor y documento ya existe.';


EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: modifica_descriptor_documento; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.modifica_descriptor_documento (in_id integer,
                                                                       in_descriptor integer,
                                                                       in_documento integer)
RETURNS character varying AS $$
DECLARE
    auxCharacter character varying;
    was general_accc.descriptor_documento%ROWTYPE;
BEGIN

    SELECT * INTO was
        FROM general_accc.descriptor_documento
        WHERE id = in_id
            AND documento = in_documento;
    IF FOUND THEN
        BEGIN
            -- No tiene sentido cambiar un descriptor de documento
            UPDATE general_accc.descriptor_documento SET
                                                     descriptor = COALESCE (in_descriptor, was.descriptor)
                WHERE id = in_id;

            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha actualizado. (%) %', SQLSTATE, SQLERRM;
        END;

    END IF;

    RAISE EXCEPTION 'La asociación entre descriptor y documento no existe, o es errónea.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: baja_descriptor_documento; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_descriptor_documento (in_id integer,
                                                                   in_descriptor integer,
                                                                   in_documento integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
BEGIN

    SELECT COUNT(descriptor) INTO auxInteger
        FROM general_accc.descriptor_documento
        WHERE documento = in_documento;
    IF auxInteger > 1 THEN
    -- Si solo hay un descriptor asociado al documento no se eliminará
        BEGIN
            DELETE FROM general_accc.descriptor_documento
                WHERE id = in_id
                    AND documento = in_documento
                    AND descriptor = in_descriptor;
            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha eliminado la relación. (%) %', SQLSTATE, SQLERRM;
        END;

    END IF;

    RAISE EXCEPTION 'La asociación entre documento y descriptor no existe, es la única disponible o es errónea.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;

