-- TFG BBDD Carnaval de Cádiz - Paloma Cortés


-- Creación de procedimientos almacenados para altas, baja y modificación de
-- los subtipos de Entidad, más los de la tabla ActividadEmpresa.




--
-- Nombre: alta_empresa; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.alta_empresa (in_nombre character varying,
                                                      in_localidad character varying,
                                                      in_provincia character varying,
                                                      in_cif character varying,
                                                      in_domicilio character varying,
                                                      in_telefono character varying,
                                                      in_contacto character varying,
                                                      in_actividad integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
BEGIN

    IF in_actividad IS NULL THEN
        RAISE EXCEPTION 'La actividad de la empresa es obligatoria';
    END IF;

    BEGIN
        INSERT INTO general_accc.entidad (tipo_entidad) VALUES ('Empresa');

    EXCEPTION WHEN others THEN
        RAISE EXCEPTION 'No se ha creado de entidad. (%) %', SQLSTATE, SQLERRM;
    END;

    BEGIN
        auxInteger = currval('general_accc.seq_entidad'::regclass);
        INSERT INTO general_accc.empresa(entidad, nombre, localidad, provincia,
                                         cif, domicilio, telefono, contacto)
            VALUES (auxInteger, in_nombre, in_localidad, in_provincia, in_cif,
                    in_domicilio, in_telefono, in_contacto);

    EXCEPTION WHEN others THEN
        SELECT setval('general_accc.seq_entidad'::regclass, currval('general_accc.seq_entidad'::regclass) - 1, true)
            INTO auxInteger;
        RAISE EXCEPTION 'No se ha creado de empresa. (%) %', SQLSTATE, SQLERRM;
    END;

    BEGIN
        SELECT general_accc.alta_actividad_empresa(in_actividad, auxInteger)
            INTO auxCharacter;
    EXCEPTION WHEN others THEN
        SELECT setval('general_accc.seq_entidad'::regclass, currval('general_accc.seq_entidad'::regclass) - 1, true)
            INTO auxInteger;
        RAISE EXCEPTION '%', SQLERRM;
    END;

    RETURN 'OK';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: modifica_empresa; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.modifica_empresa (in_entidad integer,
                                                          in_nombre character varying,
                                                          in_localidad character varying,
                                                          in_provincia character varying,
                                                          in_cif character varying,
                                                          in_domicilio character varying,
                                                          in_telefono character varying,
                                                          in_contacto character varying)
RETURNS character varying AS $$
DECLARE
    was general_accc.empresa%ROWTYPE;
BEGIN

    SELECT * INTO was FROM general_accc.empresa WHERE entidad = in_entidad;
    IF FOUND THEN

        BEGIN
            UPDATE general_accc.empresa SET
                                        nombre = COALESCE(in_nombre, was.nombre),
                                        localidad = COALESCE(in_localidad, was.localidad),
                                        provincia = COALESCE(in_provincia, was.provincia),
                                        cif = COALESCE(in_cif, was.cif),
                                        domicilio = COALESCE(in_domicilio, was.domicilio),
                                        telefono = COALESCE(in_telefono, was.telefono),
                                        contacto = COALESCE(in_contacto, was.contacto)
                WHERE entidad = in_entidad;

            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha actualizado. (%) %', SQLSTATE, SQLERRM;
        END;

    END IF;

    RAISE EXCEPTION 'El identificador de la empresa no se ha recibido o no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;




--
-- Nombre: baja_empresa; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_empresa (in_entidad integer)
RETURNS character varying AS $$
DECLARE
    was general_accc.empresa%ROWTYPE;
BEGIN

    SELECT * INTO was FROM general_accc.empresa WHERE entidad = in_entidad;
    IF FOUND THEN
        BEGIN
            -- se eliminará aunque la entidad esté vinculada a nivel de entidad
            DELETE FROM general_accc.empresa WHERE entidad = in_entidad;

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha eliminado la empresa. (%) %', SQLSTATE, SQLERRM;

        END;

        BEGIN
            -- no se eliminará si está vinculada a nivel de entidad, por autorías o creación
            DELETE FROM general_accc.entidad WHERE id = in_entidad;

            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha eliminado la entidad. (%) %', SQLSTATE, SQLERRM;
        END;
    END IF;

    RAISE EXCEPTION 'No existe la empresa con identificador: %.', in_entidad;

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;




--
-- Nombre: alta_actividad_empresa; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.alta_actividad_empresa (in_actividad integer,
                                                                in_empresa integer)
RETURNS character varying AS $$
DECLARE
BEGIN

    INSERT INTO general_accc.actividad_empresa (actividad, empresa)
        VALUES (in_actividad, in_empresa);
    RETURN 'OK';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;




--
-- Nombre: modifica_actividad_empresa; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.modifica_actividad_empresa (in_id integer,
                                                                    in_actividad integer,
                                                                    in_empresa integer)
RETURNS character varying AS $$
DECLARE
    was general_accc.actividad_empresa%ROWTYPE;
BEGIN

    SELECT * INTO was
        FROM general_accc.actividad_empresa
        WHERE id = in_id
            AND empresa = in_empresa;
    IF FOUND THEN
        -- No tiene sentido cambiar una actividad de empresa, sino a las
        -- empresas cambiarles las actividades
        BEGIN
            UPDATE general_accc.actividad_empresa SET
                                                  actividad = in_actividad
                WHERE id = in_id;
            RETURN 'OK';

    	EXCEPTION WHEN others THEN
        	RAISE EXCEPTION 'No se ha actualizado la relación. (%) %', SQLSTATE, SQLERRM;
  	    END;
    END IF;

    RAISE EXCEPTION 'La relación no se ha recibido, la empresa no puede modificarse o no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;




--
-- Nombre: baja_actividad_empresa; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_actividad_empresa (in_id integer,
                                                                in_actividad integer,
                                                                in_empresa integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
BEGIN

    SELECT COUNT(actividad) INTO auxInteger
        FROM general_accc.actividad_empresa
        WHERE empresa = in_empresa;
    IF auxInteger > 1 THEN
    -- Si solo hay una actividad asociada a la empresa no se eliminará

        BEGIN
            DELETE FROM general_accc.actividad_empresa
                WHERE id = in_id
                    AND actividad = in_actividad
                    AND empresa = in_empresa;
            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha eliminado la relación. (%) %', SQLSTATE, SQLERRM;

        END;

    END IF;

    RAISE EXCEPTION 'No existe la relación o es la única actividad asociada a la empresa.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;




--
-- Nombre: alta_persona; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.alta_persona (in_nombre character varying,
                                                      in_apellido1 character varying,
                                                      in_apellido2 character varying,
                                                      in_alias character varying,
                                                      in_dni character varying,
                                                      in_telefono character varying,
                                                      in_domicilio character varying,
                                                      in_localidad character varying,
                                                      in_provincia character varying,
                                                      in_cp character varying,
                                                      in_profesion character varying,
                                                      in_fec_nac date,
                                                      in_loc_nac character varying,
                                                      in_prov_nac character varying,
                                                      in_historia text,
                                                      in_observaciones text)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxNombre character varying;
BEGIN

    IF (in_nombre IS NULL OR char_length(trim(in_nombre)) = 0)
            OR (in_apellido1 IS NULL OR char_length(trim(in_apellido1)) = 0) THEN
        RAISE EXCEPTION 'El nombre y el primer apellido son obligatorios.';
    END IF;

    IF (in_apellido2 IS NULL OR char_length(trim(in_apellido2)) = 0)
        AND (in_dni IS NULL OR char_length(trim(in_dni)) = 0)
        AND ((in_domicilio IS NULL OR char_length(trim(in_domicilio)) = 0)
            OR (in_localidad IS NULL OR char_length(trim(in_localidad)) = 0))
    THEN
        RAISE EXCEPTION 'En defecto del segundo apellido se requiere el domicilio o DNI.';
    END IF;

    IF in_dni IS NOT NULL
            AND char_length(trim(in_dni)) <> 0
            AND char_length(trim(in_dni)) <> 8
            AND char_length(trim(in_dni)) <> 9 THEN
        RAISE EXCEPTION 'El valor de DNI es erróneo.';
    END IF;

    SELECT COUNT(entidad) INTO auxInteger
        FROM general_accc.persona
        WHERE (nombre = COALESCE(in_nombre,'')
               AND apellido1 = COALESCE(in_apellido1,'')
               AND COALESCE(domicilio,'x') = COALESCE(in_domicilio,'') 
               AND COALESCE(localidad,'x') = COALESCE(in_localidad,''))
            OR (nombre = COALESCE(in_nombre,'')
               AND apellido1 = COALESCE(in_apellido1,'')
               AND COALESCE(apellido2,'x') = COALESCE(in_apellido2,'')
               AND COALESCE(alias,'x') = COALESCE(in_alias,''));
    IF auxInteger > 0 THEN
        RAISE EXCEPTION 'Ya existe una persona con esta identificación, puede indicar el alias si necesita repetir nombre y apellidos.';
    END IF;
    
	BEGIN
        -- Transacción necesaria para controlar la consistencia del identificador
        -- de la nueva entidad 'Persona'
        INSERT INTO general_accc.entidad (tipo_entidad) VALUES ('Persona');

    EXCEPTION WHEN others THEN
        SELECT setval('general_accc.seq_entidad'::regclass, currval('general_accc.seq_entidad'::regclass) - 1, true)
            INTO auxInteger;
        RAISE EXCEPTION 'No se ha creado de entidad. (%) %', SQLSTATE, SQLERRM;

    END;

    BEGIN
        SELECT currval('general_accc.seq_entidad'::regclass) INTO auxInteger;
        INSERT INTO general_accc.persona(entidad, nombre, apellido1, apellido2, alias,
                                         dni, telefono, domicilio, localidad, provincia,
                                         cp, profesion, fec_nac, loc_nac, prov_nac,
                                         historia, observaciones)
            VALUES (auxInteger, in_nombre, in_apellido1, in_apellido2, in_alias,
                    in_dni, in_telefono, in_domicilio, in_localidad, in_provincia,
                    in_cp, in_profesion, in_fec_nac, in_loc_nac, in_prov_nac,
                    in_historia, in_observaciones);

        RETURN 'OK';

    EXCEPTION WHEN others THEN
        SELECT setval('general_accc.seq_entidad'::regclass, currval('general_accc.seq_entidad'::regclass) - 1, true)
            INTO auxInteger;
        RAISE EXCEPTION 'No se ha creado de persona. (%) %', SQLSTATE, SQLERRM;
    END;

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;




--
-- Nombre: modifica_persona; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.modifica_persona (in_entidad integer,
                                                          in_nombre character varying,
                                                          in_apellido1 character varying,
                                                          in_apellido2 character varying,
                                                          in_alias character varying,
                                                          in_dni character varying,
                                                          in_telefono character varying,
                                                          in_domicilio character varying,
                                                          in_localidad character varying,
                                                          in_provincia character varying,
                                                          in_cp character varying,
                                                          in_profesion character varying,
                                                          in_fec_nac date,
                                                          in_loc_nac character varying,
                                                          in_prov_nac character varying,
                                                          in_historia text,
                                                          in_observaciones text)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    was general_accc.persona%ROWTYPE;
BEGIN

    SELECT * INTO was FROM general_accc.persona WHERE entidad = in_entidad;
    IF FOUND THEN

        IF (in_nombre IS NOT NULL AND char_length(trim(in_nombre)) = 0)
            OR (in_apellido1 IS NOT NULL AND char_length(trim(in_apellido1)) = 0) THEN
            RAISE EXCEPTION 'El nombre y el primer apellido son obligatorios.';
        END IF;

        IF in_dni IS NOT NULL
                AND char_length(trim(in_dni)) <> 0
                AND char_length(trim(in_dni)) <> 8
                AND char_length(trim(in_dni)) <> 9 THEN
            RAISE EXCEPTION 'El valor de DNI es erróneo.';
        END IF;

        IF ((in_apellido2 IS NOT NULL AND char_length(trim(in_apellido2)) = 0)
                    OR (in_apellido2 IS NULL AND (was.apellido2 IS NULL OR char_length(trim(was.apellido2)) = 0)))

                AND ((in_dni IS NOT NULL AND char_length(trim(in_dni)) = 0)
                        OR (in_dni IS NULL AND (was.dni IS NULL OR char_length(trim(was.dni)) = 0)))

                AND (((in_domicilio IS NOT NULL AND char_length(trim(in_domicilio)) = 0)
                        OR (in_domicilio IS NULL AND (was.domicilio IS NULL OR char_length(trim(was.domicilio)) = 0)))

                        OR  ((in_localidad IS NOT NULL AND char_length(trim(in_localidad)) = 0)
                                OR (in_localidad IS NULL AND (was.localidad IS NULL OR char_length(trim(was.localidad)) = 0)))) THEN

            RAISE EXCEPTION 'En defecto del segundo apellido se requiere el domicilio o DNI.';
        END IF;

        SELECT COUNT(entidad) INTO auxInteger
            FROM general_accc.persona
            WHERE ((nombre = COALESCE(in_nombre,'')
                       AND apellido1 = COALESCE(in_apellido1,'')
                       AND COALESCE(domicilio,'x') = COALESCE(in_domicilio,'') 
                       AND COALESCE(localidad,'x') = COALESCE(in_localidad,''))
                    OR (nombre = COALESCE(in_nombre,'')
                       AND apellido1 = COALESCE(in_apellido1,'')
                       AND COALESCE(apellido2,'x') = COALESCE(in_apellido2,'')
                       AND COALESCE(alias,'x') = COALESCE(in_alias,'')))
                AND entidad <> in_entidad;
        IF auxInteger > 0 THEN
            RAISE EXCEPTION 'Ya existe una persona con esta identificación, puede indicar el alias si necesita repetir nombre y apellidos.';
        END IF;
        BEGIN
            UPDATE general_accc.persona SET
                                        nombre = COALESCE(in_nombre, was.nombre),
                                        apellido1 = COALESCE(in_apellido1, was.apellido1),
                                        apellido2 = COALESCE(in_apellido2, was.apellido2),
                                        alias = COALESCE(in_alias, was.alias),
                                        dni = COALESCE(in_dni, was.dni),
                                        telefono = COALESCE(in_telefono, was.telefono),
                                        domicilio = COALESCE(in_domicilio, was.domicilio),
                                        localidad = COALESCE(in_localidad, was.localidad),
                                        provincia = COALESCE(in_provincia, was.provincia),
                                        cp = COALESCE(in_cp, was.cp),
                                        profesion = COALESCE(in_profesion, was.profesion),
                                        fec_nac = COALESCE(in_fec_nac, was.fec_nac),
                                        loc_nac = COALESCE(in_loc_nac, was.loc_nac),
                                        prov_nac = COALESCE(in_prov_nac, was.prov_nac),
                                        historia = COALESCE(in_historia, was.historia),
                                        observaciones = COALESCE(in_observaciones, was.observaciones)
                WHERE entidad = in_entidad;

            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha actualizado. (%) %', SQLSTATE, SQLERRM;
        END;

    END IF;
    RAISE EXCEPTION 'El identificador de la persona no ha sido recibido o no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;




--
-- Nombre: baja_persona; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_persona (in_entidad integer)
RETURNS character varying AS $$
DECLARE
    was general_accc.persona%ROWTYPE;
BEGIN

    SELECT * INTO was FROM general_accc.persona WHERE entidad = in_entidad;
    IF FOUND THEN

        BEGIN
            -- Se eliminará aunque la entidad esté vinculada a nivel de entidad
            DELETE FROM general_accc.persona WHERE entidad = in_entidad;

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha eliminado la persona. (%) %', SQLSTATE, SQLERRM;
        END;

        BEGIN
            -- No se eliminará si está vinculada a nivel de entidad,
            -- por autorías o creación
            DELETE FROM general_accc.entidad WHERE id = in_entidad;
            RETURN 'OK';
        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha eliminado la entidad. (%) %', SQLSTATE, SQLERRM;

        END;
    END IF;

    RAISE EXCEPTION 'No existe la persona con identificador: %.', in_entidad;

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;




--
-- Nombre: alta_agrupacion; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.alta_agrupacion (in_entidad integer,
                                                         in_localidad character varying,
                                                         in_provincia character varying,
                                                         in_alias character varying,
                                                         in_contacto character varying,
                                                         in_observaciones text)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxEntidad integer;
BEGIN

    BEGIN

        -- Admite que el usuario pueda insertar un identificador de entidad si
        -- ya estaba registrada, esto solo se contempla para las agrupaciones
        IF in_entidad IS NULL THEN
            auxEntidad = nextval('general_accc.seq_entidad'::regclass);
        ELSE
            auxEntidad := in_entidad;
        END IF;

        INSERT INTO general_accc.entidad (id, tipo_entidad) VALUES (auxEntidad, 'Agrupacion');

    EXCEPTION WHEN others THEN
        IF in_entidad IS NULL THEN
            SELECT setval('general_accc.seq_entidad'::regclass, currval('general_accc.seq_entidad'::regclass) - 1, true)
                INTO auxInteger;
        END IF;
        RAISE EXCEPTION 'No se ha creado la entidad. (%) %', SQLSTATE, SQLERRM;
    END;

    BEGIN
        INSERT INTO general_accc.agrupacion(entidad, localidad, provincia, alias,
                                            contacto, observaciones)
            VALUES (auxEntidad, in_localidad, in_provincia, in_alias, in_contacto,
                    in_observaciones);

        RETURN 'OK';

    EXCEPTION WHEN others THEN
        IF in_entidad IS NULL THEN
            SELECT setval('general_accc.seq_entidad'::regclass, currval('general_accc.seq_entidad'::regclass) - 1, true)
                INTO auxInteger;
        END IF;
        RAISE EXCEPTION 'No se ha creado la agrupación. (%) %', SQLSTATE, SQLERRM;
    END;

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;




--
-- Nombre: modifica_agrupacion; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.modifica_agrupacion (in_entidad integer,
                                                             in_localidad character varying,
                                                             in_provincia character varying,
                                                             in_alias character varying,
                                                             in_contacto character varying,
                                                             in_observaciones text)
RETURNS character varying AS $$
DECLARE
    was general_accc.agrupacion%ROWTYPE;
BEGIN

    IF in_entidad IS NULL THEN
        RAISE EXCEPTION 'El identificador de la agrupación es obligatorio para modificarla.';
    END IF;

    SELECT * INTO was FROM general_accc.agrupacion WHERE entidad = in_entidad;
    IF FOUND THEN

        BEGIN
            UPDATE general_accc.agrupacion SET
                                           localidad = COALESCE(in_localidad, was.localidad),
                                           provincia = COALESCE(in_provincia, was.provincia),
                                           alias = COALESCE(in_alias, was.alias),
                                           contacto = COALESCE(in_contacto, was.contacto),
                                           observaciones = COALESCE(in_observaciones, was.observaciones)
                WHERE entidad = in_entidad;

            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha actualizado. (%) %', SQLSTATE, SQLERRM;

        END;

    END IF;

    RAISE EXCEPTION 'El identificador de la agrupación no existe o no se ha indicado.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;




--
-- Nombre: baja_agrupacion; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_agrupacion (in_entidad integer)
RETURNS character varying AS $$
DECLARE
    was general_accc.agrupacion%ROWTYPE;
BEGIN

    SELECT * INTO was FROM general_accc.agrupacion WHERE entidad = in_entidad;
    IF FOUND THEN

        BEGIN
            -- Se eliminará aunque la entidad esté vinculada a nivel de entidad
            DELETE FROM general_accc.agrupacion WHERE entidad = in_entidad;

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha eliminado la agrupación. (%) %', SQLSTATE, SQLERRM;
        END;

        BEGIN
            -- No se eliminará si está vinculada a nivel de entidad,
            -- por autorías o creación
            DELETE FROM general_accc.entidad WHERE id = in_entidad;
            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha eliminado la entidad. (%) %', SQLSTATE, SQLERRM;
        END;

    END IF;
    RAISE EXCEPTION 'No existe la agrupación con identificador: %.', in_entidad;

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;




--
-- Nombre: modifica_id_agrupacion; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.modifica_id_agrupacion (in_id_nuevo integer,
                                                                in_id_viejo integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
BEGIN

    SELECT entidad INTO auxInteger FROM general_accc.agrupacion WHERE entidad = in_id_nuevo;
    IF NOT FOUND AND in_id_nuevo < 100000 THEN
                                   -- Valor inicial de la secuencia de Entidad
    	UPDATE general_accc.entidad SET
                                      id = in_id_nuevo
            WHERE id = in_id_viejo;
        RETURN 'OK';
    END IF;
    RAISE EXCEPTION 'El identificador ya está en uso o no puede utilizarse.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;


