-- TFG BBDD Carnaval de Cádiz - Paloma Cortés


-- Creación de procedimientos almacenados para altas, baja y modificación de
-- Evento y Concurso, más las directamente relacionadas, Premio y participa.


--
-- Nombre: alta_evento; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.alta_evento (in_tipo_evento character varying,
                                                     in_anio integer,
                                                     in_nombre character varying,
                                                     in_localidad character varying,
                                                     in_provincia character varying,
                                                     in_ubicacion character varying,
                                                     in_descripcion text)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
BEGIN

    SELECT id INTO auxInteger
        FROM general_accc.evento
        WHERE anio = in_anio
            AND lower(nombre) = lower(in_nombre)
            AND lower(localidad) = lower(in_localidad)
            AND lower(provincia) = lower(in_provincia);

    IF NOT FOUND AND in_tipo_evento <> 'Concurso' THEN
        
        BEGIN
            INSERT INTO general_accc.evento(tipo_evento, anio, nombre, localidad,
                                        provincia, ubicacion, descripcion)
               VALUES (in_tipo_evento::general_accc.enum_evento, in_anio, 
                       in_nombre, in_localidad, in_provincia, in_ubicacion, in_descripcion);
            RETURN 'OK';
        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha creado el evento. (%) %', SQLSTATE, SQLERRM;
        END;

    END IF;

    RAISE EXCEPTION 'El evento ya existe o se intenta crear un Concurso.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: modifica_Evento; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.modifica_evento (in_id integer,
                                                         in_tipo_evento character varying,
                                                         in_anio integer,
                                                         in_nombre character varying,
                                                         in_localidad character varying,
                                                         in_provincia character varying,
                                                         in_ubicacion character varying,
                                                         in_descripcion text)
RETURNS character varying AS $$
DECLARE
    was general_accc.evento%ROWTYPE;
BEGIN

    SELECT * INTO was
        FROM general_accc.evento
        WHERE id = in_id;
    IF FOUND AND COALESCE(in_tipo_evento,'') <> 'Concurso'
            AND was.tipo_evento <> 'Concurso'::general_accc.enum_evento THEN
        BEGIN
            UPDATE general_accc.evento SET
                                       tipo_evento = COALESCE (in_tipo_evento::general_accc.enum_evento, was.tipo_evento),
                                       anio = COALESCE (in_anio, was.anio),
                                       nombre = COALESCE (in_nombre, was.nombre),
                                       localidad = COALESCE (in_localidad, was.localidad),
                                       provincia = COALESCE (in_provincia, was.provincia),
                                       ubicacion = COALESCE (in_ubicacion, was.ubicacion),
                                       descripcion = COALESCE (in_descripcion, was.descripcion)
                WHERE id = in_id;

            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha actualizado. (%) %', SQLSTATE, SQLERRM;
        END;

    END IF;

    RAISE EXCEPTION 'El identificador del evento no ha sido recibido, refiere a un concurso o no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: baja_evento; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_evento (in_id integer)
RETURNS character varying AS $$
DECLARE
    was general_accc.evento%ROWTYPE;
BEGIN

    SELECT * INTO was FROM general_accc.evento WHERE id = in_id;
    IF NOT FOUND OR was.tipo_evento = 'Concurso'::general_accc.enum_evento THEN
        RAISE EXCEPTION 'El evento no existe o es un concurso.';
    END IF;

    -- ~ general_accc.baja_documento_evento
    -- es innecesario por que existe ON DELETE RESTRICT en la tabla documento_evento
    DELETE FROM general_accc.evento WHERE id = in_id;
    RETURN 'OK';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: A_Concurso; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.alta_concurso (in_anio integer,
                                                       in_nombre character varying,
                                                       in_localidad character varying,
                                                       in_provincia character varying,
                                                       in_ubicacion character varying,
                                                       in_descripcion text,
                                                       in_categoria character varying)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
BEGIN


    SELECT id INTO auxInteger
        FROM general_accc.evento
        WHERE anio = in_anio
            AND lower(nombre) = lower(in_nombre)
            AND lower(localidad) = lower(in_localidad)
            AND lower(provincia) = lower(in_provincia);

    IF NOT FOUND THEN
        
        BEGIN
            INSERT INTO general_accc.evento(tipo_evento, anio, nombre, localidad,
                                        provincia, ubicacion, descripcion)
               VALUES ('Concurso', in_anio, 
                       in_nombre, in_localidad, in_provincia, in_ubicacion, in_descripcion);
        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha creado el evento. (%) %', SQLSTATE, SQLERRM;
        END;

    ELSE 
        RAISE EXCEPTION 'El concurso ya existe.';
    END IF;

    -- para declarar un concurso dos veces con distinta categoría habrá que marcar la diferencia en el nombre
    -- ya que deben existir dos eventos diferentes, resulta necesario tanto para la gestión de los premios
    -- como de los documentos vinculados

    BEGIN
        SELECT currval('general_accc.seq_evento'::regclass) INTO auxInteger;
        INSERT INTO general_accc.concurso(evento, categoria)
            VALUES (auxInteger, in_categoria::general_accc.enum_categoria);

        RETURN 'OK';

    EXCEPTION WHEN others THEN
        RAISE EXCEPTION 'No se ha creado el concurso. (%) %', SQLSTATE, SQLERRM;
    END;

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: M_Concurso; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.modifica_concurso (in_evento integer,
                                                           in_anio integer,
                                                           in_nombre character varying,
                                                           in_localidad character varying,
                                                           in_provincia character varying,
                                                           in_ubicacion character varying,
                                                           in_descripcion text,
                                                           in_categoria character varying)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    wasEvento general_accc.evento%ROWTYPE;
    wasConcurso general_accc.concurso%ROWTYPE;
BEGIN

    SELECT * INTO wasEvento FROM general_accc.evento WHERE id = in_evento;
    SELECT * INTO wasConcurso FROM general_accc.concurso WHERE evento = in_evento;
    IF FOUND THEN

        IF (in_categoria IS NOT NULL AND wasConcurso.categoria <> in_categoria::general_accc.enum_categoria)
            OR (in_anio IS NOT NULL AND in_anio <> wasEvento.anio) THEN
            SELECT count(concurso) INTO auxInteger
                FROM general_accc.participa
                WHERE concurso = in_evento;
            IF auxInteger > 0 THEN
                RAISE EXCEPTION 'No se puede cambiar la categoría ni el año porque tiene agrupaciones inscritas.';
            END IF;
        END IF;

        BEGIN
            UPDATE general_accc.evento SET
                                       anio = COALESCE(in_anio, wasEvento.anio),
                                       nombre = COALESCE(in_nombre, wasEvento.nombre),
                                       localidad = COALESCE(in_localidad, wasEvento.localidad),
                                       provincia = COALESCE(in_provincia, wasEvento.provincia),
                                       ubicacion = COALESCE(in_ubicacion, wasEvento.ubicacion),
                                       descripcion = COALESCE(in_descripcion, wasEvento.descripcion)
                WHERE id = in_evento;
            UPDATE general_accc.concurso SET
                                         categoria = COALESCE(in_categoria::general_accc.enum_categoria, wasConcurso.categoria)
                WHERE evento = in_evento;

            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha actualizado. (%) %', SQLSTATE, SQLERRM;

        END;
    END IF;

    RAISE EXCEPTION 'El identificador del concurso no ha sido recibido, es erróneo o no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: B_Concurso; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_concurso (in_evento integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
BEGIN

    SELECT evento INTO auxInteger FROM general_accc.concurso WHERE evento = in_evento;
    IF FOUND THEN

        BEGIN
            -- se eliminará aunque el evento esté vinculado a nivel de evento
            DELETE FROM general_accc.concurso WHERE evento = in_evento;

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha eliminado el concurso. (%) %', SQLSTATE, SQLERRM;
        END;

        BEGIN
            -- no se eliminará si está vinculada a nivel de entidad, por autorías o creación
            DELETE FROM general_accc.evento WHERE id = in_evento;
            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha eliminado el evento. (%) %', SQLSTATE, SQLERRM;
        END;

    END IF;

    RAISE EXCEPTION 'El identificador del concurso no ha sido recibido o no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: A_PremioConcurso; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.alta_premio_concurso (in_concurso integer,
                                                              in_modalidad integer,
                                                              in_premio integer,
                                                              in_cuantia numeric(10,0),
                                                              in_observaciones text)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
BEGIN

    -- esta verificación refiere a la clave candidata de la relación
    SELECT id INTO auxInteger
        FROM general_accc.premio
        WHERE concurso = in_concurso
            AND modalidad = in_modalidad
            AND premio = in_premio;

    IF NOT FOUND THEN
        INSERT INTO general_accc.premio(concurso, id, modalidad, premio, cuantia, observaciones)
               VALUES (in_concurso, nextval('general_accc.seq_premio'::regclass),
                       in_modalidad, in_premio, in_cuantia, in_observaciones);
        RETURN 'OK';
    END IF;

    RAISE EXCEPTION 'El premio indicado ya existe en este concurso.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;




--
-- Nombre: M_PremioConcurso; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.modifica_premio_concurso (in_id integer,
                                                                  in_concurso integer,
                                                                  in_modalidad integer,
                                                                  in_premio integer,
                                                                  in_cuantia numeric(10,0),
                                                                  in_observaciones text)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    was general_accc.premio%ROWTYPE;
BEGIN

    SELECT * INTO was
        FROM general_accc.premio
        WHERE concurso = in_concurso
            AND id = in_id;
    IF FOUND THEN
        IF in_modalidad <> was.modalidad THEN
            -- Si se va a cambiar la modalidad del premio, al igual que para eliminarlo,
            -- se verifica que el premio no afecta a ninguna relación de participación
            SELECT part.agrupacion_anio INTO auxInteger
                FROM general_accc.participa AS part
                    INNER JOIN general_accc.agrupacion_anio AS agrup
                        ON agrup.id = part.agrupacion_anio
                WHERE agrup.modalidad = was.modalidad;
            IF FOUND THEN
                RAISE EXCEPTION 'El premio está relacionado a una agrupación.';
            END IF;
        END IF;

        BEGIN
            UPDATE general_accc.premio SET
                                       -- Cambiar un premio de concurso no es normal,
                                       -- por la relación de composición, puesto que 
                                       -- los premios se editarían a partir de los concursos
                                       modalidad = COALESCE(in_modalidad, was.modalidad),
                                       premio = COALESCE(in_premio, was.premio),
                                       cuantia = COALESCE(in_cuantia, was.cuantia),
                                       observaciones = COALESCE(in_observaciones, was.observaciones)
                WHERE id = in_id;

            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha actualizado. (%) %', SQLSTATE, SQLERRM;
        END;

    END IF;
    RAISE EXCEPTION 'El identificador del premio no ha sido recibido, es erróneo o no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: B_PremioConcurso; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_premio_concurso (in_concurso integer,
                                                              in_id integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    was general_accc.premio%ROWTYPE;
BEGIN

    SELECT * INTO was
        FROM general_accc.premio
        WHERE concurso = in_concurso
            AND id = in_id;

    IF FOUND THEN
        -- Se verifica que el premio no afecta a ninguna relación de participación
        SELECT agrup.id INTO auxInteger
            FROM general_accc.participa AS part,
                general_accc.agrupacion_anio AS agrup,
                general_accc.premio AS prem
            WHERE prem.id = in_id
                AND prem.concurso = part.concurso
                AND agrup.id = part.agrupacion_anio
                AND agrup.modalidad = prem.modalidad;
        IF NOT FOUND THEN
            BEGIN
                DELETE FROM general_accc.premio WHERE id = in_id;
                RETURN 'OK';

            EXCEPTION WHEN others THEN
                RAISE EXCEPTION 'No se ha eliminado. (%) %', SQLSTATE, SQLERRM;
            END;

        END IF;
        RAISE EXCEPTION 'El premio está relacionado a una agrupación.';

    END IF;
    RAISE EXCEPTION 'El identificador del concurso no ha sido recibido o no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: doc_nuevo_a_evento; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.doc_nuevo_a_evento (in_evento integer,	
                                                            in_id_doc integer,	
                                                            in_3_1_1 character varying,
                                                            in_3_1_2 character varying,
                                                            in_3_1_3 character varying,
                                                            in_tipo_documento character varying,
                                                            in_tipo_soporte character varying,
                                                            in_ubicacion character varying,
                                                            in_deposito character varying,
                                                            in_estante character varying,
                                                            in_url character varying,
                                                            in_3_1_4 character varying,
                                                            in_3_1_5 text,
                                                            in_3_2_1 text,
                                                            in_3_2_2 text,
                                                            in_3_2_3 text,
                                                            in_3_2_4 text,
                                                            in_3_3_1 text,
                                                            in_3_3_2 text,
                                                            in_3_3_3 text,
                                                            in_3_3_4 text,
                                                            in_3_4_1 text,
                                                            in_3_4_2 text,
                                                            in_3_4_3 text,
                                                            in_3_4_4 text,
                                                            in_3_4_5 text,
                                                            in_3_5_1 text,
                                                            in_3_5_2 text,
                                                            in_3_5_3 text,
                                                            in_3_5_4 text,
                                                            in_3_6_1 text,
                                                            in_3_7_1 text,
                                                            in_3_7_2 text,
                                                            in_3_7_3 text,
                                                            in_descriptor integer,
                                                            in_esta_descrito boolean,
                                                            in_esta_digitalizado boolean,
                                                            in_en_deposito boolean,
                                                            in_observaciones_gestion text)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxDoc integer;
    auxCharacter character varying;
BEGIN

    SELECT id INTO auxInteger FROM general_accc.evento WHERE id = in_evento;
    IF FOUND THEN

        IF in_id_doc IS NULL THEN
            SELECT nextval('general_accc.seq_documento'::regclass) INTO auxDoc;
        ELSE
            auxDoc := in_id_doc;
        END IF;

        IF in_3_1_1 IS NULL OR char_length(trim(in_3_1_1)) = 0 THEN
            in_3_1_1 = 'ES.ACCC//' || 'E.' || in_evento || '-' || in_id_doc;
        ELSEIF position('ES.ACCC//' in in_3_1_1) <> 1 THEN
            RAISE EXCEPTION 'El valor 3.1.1 Código de referencia debe empezar por ES.ACCC//.';
        END IF;

        BEGIN
            SELECT general_accc.__alta_documento (auxDoc, in_3_1_1, in_3_1_2,
                                                  in_3_1_3, in_tipo_documento,
                                                  in_tipo_soporte,in_ubicacion, in_deposito,
                                                  in_estante, in_url, in_3_1_4, in_3_1_5,
                                                  in_3_2_1, in_3_2_2, in_3_2_3, in_3_2_4,
                                                  in_3_3_1, in_3_3_2, in_3_3_3, in_3_3_4,
                                                  in_3_4_1, in_3_4_2, in_3_4_3, in_3_4_4,
                                                  in_3_4_5, in_3_5_1, in_3_5_2, in_3_5_3,
                                                  in_3_5_4, in_3_6_1, in_3_7_1, in_3_7_2,
                                                  in_3_7_3, in_descriptor, in_esta_descrito,
                                                  in_esta_digitalizado, in_en_deposito,
                                                  in_observaciones_gestion)
                INTO auxCharacter;

        EXCEPTION WHEN others THEN
            IF in_id_doc IS NULL THEN
                SELECT setval('general_accc.seq_documento'::regclass, currval('general_accc.seq_documento'::regclass) - 1, true)
                    INTO auxInteger;
            END IF;
            RAISE EXCEPTION '%', SQLERRM;
        END;

        BEGIN
            INSERT INTO general_accc.documento_evento(documento, evento)
                VALUES (in_id_doc, in_evento);
            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha creado la relación. (%) %', SQLSTATE, SQLERRM;
        END;

    END IF;
    RAISE EXCEPTION 'El identificador del evento no ha sido recibido o no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: modifica_documento_evento; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.modifica_documento_evento (in_evento integer,		
                                                                   in_documento integer,
                                                                   in_3_1_1 character varying,
                                                                   in_3_1_2 character varying,
                                                                   in_3_1_3 character varying,
                                                                   in_tipo_documento character varying,
                                                                   in_tipo_soporte character varying,
                                                                   in_ubicacion character varying,
                                                                   in_deposito character varying,
                                                                   in_estante character varying,
                                                                   in_url character varying,
                                                                   in_3_1_4 character varying,
                                                                   in_3_1_5 text,
                                                                   in_3_2_1 text,
                                                                   in_3_2_2 text,
                                                                   in_3_2_3 text,
                                                                   in_3_2_4 text,
                                                                   in_3_3_1 text,
                                                                   in_3_3_2 text,
                                                                   in_3_3_3 text,
                                                                   in_3_3_4 text,
                                                                   in_3_4_1 text,
                                                                   in_3_4_2 text,
                                                                   in_3_4_3 text,
                                                                   in_3_4_4 text,
                                                                   in_3_4_5 text,
                                                                   in_3_5_1 text,
                                                                   in_3_5_2 text,
                                                                   in_3_5_3 text,
                                                                   in_3_5_4 text,
                                                                   in_3_6_1 text,
                                                                   in_3_7_1 text,
                                                                   in_3_7_2 text,
                                                                   in_3_7_3 text,
                                                                   in_esta_descrito boolean,
                                                                   in_esta_digitalizado boolean,
                                                                   in_en_deposito boolean,
                                                                   in_observaciones_gestion text)
RETURNS character varying AS $$
DECLARE
    auxCharacter character varying;
    was general_accc.documento_evento%ROWTYPE;
BEGIN

    SELECT * INTO was
        FROM general_accc.documento_evento
        WHERE documento = in_documento
            AND evento = in_evento;
    IF FOUND THEN
        -- Imposibilita que se cambie el identificador del documento en la
        -- modificación del mismo, porque ha verificado que el par de
        -- identificadores existan en la tabla de N:M

        -- Verifica que si cambia el código de referencia del documento aun
        -- respete el formato impuesto
        IF in_3_1_1 IS NOT NULL THEN
            IF position('ES.ACCC//' in in_3_1_1) <> 1  THEN
                RAISE EXCEPTION 'El valor 3.1.1 Código de referencia debe empezar por ES.ACCC//';
            ELSEIF position(in_documento in in_3_1_1) < 9 THEN
                RAISE EXCEPTION 'El valor 3.1.1 Código de referencia debe empezar por ES.ACCC//';
            END IF;
            -- Para este caso no se requiere que contenga el identificador del
            -- evento porque quizás no haya sido introducido el documento
            -- apartir de dicho evento
        END IF;

        BEGIN
            SELECT general_accc.__modifica_documento (in_documento, in_3_1_1,
                                                      in_3_1_2, in_3_1_3,
                                                      in_tipo_documento,
                                                      in_tipo_soporte,
                                                      in_ubicacion, in_deposito,
                                                      in_estante, in_url,
                                                      in_3_1_4, in_3_1_5,
                                                      in_3_2_1, in_3_2_2, in_3_2_3,
                                                      in_3_2_4, in_3_3_1, in_3_3_2,
                                                      in_3_3_3, in_3_3_4, in_3_4_1,
                                                      in_3_4_2, in_3_4_3, in_3_4_4,
                                                      in_3_4_5, in_3_5_1, in_3_5_2,
                                                      in_3_5_3, in_3_5_4, in_3_6_1,
                                                      in_3_7_1, in_3_7_2, in_3_7_3,
                                                      in_esta_descrito,
                                                      in_esta_digitalizado,
                                                      in_en_deposito,
                                                      in_observaciones_gestion)
                INTO auxCharacter;
        EXCEPTION WHEN others THEN
            RAISE EXCEPTION '%', SQLERRM;
        END;
        RETURN 'OK';

    END IF;

    RAISE EXCEPTION 'La relación entre documento y evento no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;




--
-- Nombre: doc_existente_a_evento; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.doc_existente_a_evento (in_documento integer,
                                                                in_evento integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
BEGIN

    INSERT INTO general_accc.documento_evento(documento, evento)
        VALUES (in_documento, in_evento);
    RETURN 'OK';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;



--
-- Nombre: baja_documento_evento; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_documento_evento (in_documento integer,
                                                               in_evento integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
    was general_accc.documento_evento%ROWTYPE;
BEGIN

    SELECT * INTO was
        FROM general_accc.documento_evento
        WHERE documento = in_documento
            AND evento = in_evento;
    IF FOUND THEN

        BEGIN
            DELETE FROM general_accc.documento_evento
                WHERE documento = in_documento
                    AND evento = in_evento;
            BEGIN
                SELECT general_accc.__baja_documento (in_documento)
                    INTO auxCharacter;

            EXCEPTION WHEN others THEN
                -- Estos dos mensajes se reciben cuando la relación del
                -- documento eliminada es la única que había para el mismo
                IF SQLERRM = 'La descripción del documento refiere a un documento electrónico localizado.'
                        OR SQLERRM = 'La descripción del documento refiere a un documento físico localizado.' THEN
                    RAISE EXCEPTION 'No se puede eliminar el evento porque tiene documentos tangibles vinculados. %', SQLERRM;

                ELSEIF SQLERRM <> 'El documento aún refiere a algún elemento del inventario.' THEN
                    RAISE EXCEPTION '%', SQLERRM;
                END IF;
            END;

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha eliminado la relación. (%) %', SQLSTATE, SQLERRM;
        END;

        -- El resto de errores que lanza la función anterior no influyen en la
        -- eliminación de la relación entre el evento y el documento indicados
        RETURN 'OK';

    END IF;

    RAISE EXCEPTION 'La relación entre documento y evento no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;






