-- TFG BBDD Carnaval de Cádiz - Paloma Cortés


-- Creación de procedimientos almacenados para altas, baja y modificación de AgrupacionAnio




--
-- Nombre: __verifica_agrupacionanio_obra; Tipo: FUNCTION; Schema: general_accc
--
-- Función auxiliar para gestionar la corrección de las obras a interpretar
--
CREATE OR REPLACE FUNCTION general_accc.__verifica_agrupacionanio_obra (in_tipo_obra character varying,
                                                                        in_nombre_agrupacion character varying,
                                                                        in_obra integer)
RETURNS character varying AS $$
DECLARE
    aux1 character varying;
BEGIN

    SELECT doc.id INTO aux1
        FROM general_accc.documento_obra AS d_o
            INNER JOIN general_accc.documento AS doc
                ON doc.id = d_o.documento
            INNER JOIN general_accc.obra AS obra
                ON obra.signatura = d_o.obra
        WHERE d_o.obra = in_obra
            AND obra.tipo_obra = in_tipo_obra::general_accc.enum_obra
            AND upper(doc."3_1_2_titulo") LIKE '%' || upper(in_nombre_agrupacion) || '%';
                    
    IF NOT FOUND THEN
        RAISE EXCEPTION 'La obra indicada no tiene ningún documento con el nombre de la agrupación como título.';
    END IF;

    RETURN 'OK';

    -- Verificar que la fecha de la obra coincide con la de la agrupación es
    -- excesivo por las innumerables ocasiones en que las obras se reinterpretan

END;
$$ LANGUAGE plpgsql;




--
-- Nombre: alta_agrupacionanio; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.alta_agrupacionanio (in_entidad integer,
                                                             in_id integer,
                                                             in_localidad character varying,
                                                             in_provincia character varying,
                                                             in_nombre character varying,
                                                             in_anio integer,
                                                             in_categoria character varying,
                                                             in_modalidad integer,
                                                             in_submodalidad character varying,
                                                             in_composicion character varying,
                                                             in_num_componentes integer,
                                                             in_presupuesto numeric(10,0),
                                                             in_lugar_ensayo character varying,
                                                             in_observaciones text,
                                                             in_en_prensa boolean,
                                                             in_en_documento_oficial boolean,
                                                             in_fuentes character varying,
                                                             in_observaciones_gestion text,
                                                             in_interpreta_literaria integer,
                                                             in_interpreta_musical integer,
                                                             in_utiliza_tipo integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxID integer;
    auxCharacter character varying;
BEGIN

    SELECT id INTO auxInteger FROM general_accc.agrupacion_anio WHERE lower(nombre) = lower(in_nombre);
    IF FOUND THEN
        RAISE EXCEPTION 'Ya existe una agrupación de nombre %.', in_nombre;
    END IF;

    -- Admite que el usuario pueda insertar un identificador de agrupación si
    -- ya estaba registrada
    IF in_id IS NULL THEN
        auxID = nextval('general_accc.seq_agrupacionanio'::regclass);
    ELSE
        auxID := in_id;
    END IF;
        
    IF in_en_prensa IS NULL THEN
        in_en_prensa := false;
    END IF;
    IF in_en_documento_oficial IS NULL THEN
        in_en_documento_oficial := false;
    END IF;

    IF in_en_prensa OR in_en_documento_oficial THEN
        IF in_fuentes IS NULL OR char_length(trim(in_fuentes)) = 0 THEN
            RAISE EXCEPTION 'Fuentes es valor obligatorio si se ha indicado que en prensa o documento oficial.';
        END IF;
    END IF;

    -- Verificar que coincide el nombre de la obra con el de la agrupación
    IF in_interpreta_literaria IS NOT NULL THEN
        BEGIN
            SELECT general_accc.__verifica_agrupacionanio_obra('Literaria', in_nombre, in_interpreta_literaria)
                INTO auxCharacter;
        EXCEPTION WHEN others THEN
            RAISE EXCEPTION '%', SQLERRM;
        END;
    END IF;

    IF in_interpreta_musical IS NOT NULL THEN
        BEGIN
            SELECT general_accc.__verifica_agrupacionanio_obra('Musical', in_nombre, in_interpreta_musical)
                INTO auxCharacter;
        EXCEPTION WHEN others THEN
            RAISE EXCEPTION '%', SQLERRM;
        END;
    END IF;

    IF in_utiliza_tipo IS NOT NULL THEN
        BEGIN
            SELECT general_accc.__verifica_agrupacionanio_obra('Grafica', in_nombre, in_utiliza_tipo)
                INTO auxCharacter;
        EXCEPTION WHEN others THEN
            RAISE EXCEPTION '%', SQLERRM;
        END;
    END IF;

    -- creación de la agrupación en caso de que no exista
    SELECT entidad INTO auxInteger FROM general_accc.agrupacion WHERE entidad = in_entidad;
    IF NOT FOUND THEN
        BEGIN
            SELECT general_accc.alta_agrupacion(in_entidad, in_localidad, in_provincia, null, null, null)
                INTO auxCharacter;
        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'La agrupación no ha sido indicada y no ha podido crearse. %', SQLERRM;
        END;
        IF in_entidad IS NULL THEN
            -- Si este identificador fue enviado al procedimiento alta_agrupacion()
            -- no se habrá hecho uso de la secuencia
            SELECT currval('general_accc.seq_entidad'::regclass) INTO in_entidad;
            auxInteger := -5;
        END IF;

    END IF;

    BEGIN
        INSERT INTO general_accc.agrupacion_anio(entidad, id, nombre, anio,
                                                 categoria, modalidad,
                                                 submodalidad, composicion,
                                                 num_componentes, presupuesto,
                                                 lugar_ensayo, observaciones,
                                                 en_prensa, en_documento_oficial,
                                                 fuentes, observaciones_gestion,
                                                 interpreta_literaria,
                                                 interpreta_musical, utiliza_tipo)
            VALUES (in_entidad, auxID, in_nombre, in_anio, 
                    in_categoria::general_accc.enum_categoria, in_modalidad,
                    in_submodalidad, in_composicion::general_accc.enum_composicion, 
                    in_num_componentes, in_presupuesto, in_lugar_ensayo, in_observaciones,
                    in_en_prensa, in_en_documento_oficial, in_fuentes,
                    in_observaciones_gestion, in_interpreta_literaria,
                    in_interpreta_musical, in_utiliza_tipo);
        RETURN 'OK';

    EXCEPTION WHEN others THEN
        IF auxInteger = -5 THEN
            -- Si se hizo uso de la secuencia y el registro no se ha insertado
            -- en agrupacion_anio
            SELECT setval('general_accc.seq_entidad'::regclass, currval('general_accc.seq_entidad'::regclass) - 1, true) INTO auxInteger;
        END IF;
        RAISE EXCEPTION 'No se ha creado la agrupación. %', SQLERRM;
    END;

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: modifica_agrupacionanio; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.modifica_agrupacionanio (in_entidad integer,
                                                                 in_id integer,
                                                                 in_nombre character varying,
                                                                 in_anio integer,
                                                                 in_categoria character varying,
                                                                 in_modalidad integer,
                                                                 in_submodalidad character varying,
                                                                 in_composicion character varying,
                                                                 in_num_componentes integer,
                                                                 in_presupuesto numeric(10,0),
                                                                 in_lugar_ensayo character varying,
                                                                 in_observaciones text,
                                                                 in_en_prensa boolean,
                                                                 in_en_documento_oficial boolean,
                                                                 in_fuentes character varying,
                                                                 in_observaciones_gestion text,
                                                                 in_interpreta_literaria integer,
                                                                 in_interpreta_musical integer,
                                                                 in_utiliza_tipo integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
    auxNombre character varying;
    was general_accc.agrupacion_anio%ROWTYPE;
BEGIN

    SELECT nombre INTO auxCharacter
        FROM general_accc.agrupacion_anio
        WHERE lower(nombre) = lower(in_nombre)
            AND id <> in_id;
    IF FOUND THEN
        RAISE EXCEPTION 'Ya existe una agrupación de nombre %.', in_nombre;
    END IF;

    SELECT * INTO was
        FROM general_accc.agrupacion_anio
        WHERE id = in_id;
    IF FOUND THEN

        IF in_categoria::general_accc.enum_categoria <> was.categoria OR in_anio <> was.anio THEN

            SELECT count(concurso) INTO auxInteger
                FROM general_accc.participa
                WHERE agrupacion_anio = in_id;

            IF FOUND THEN
                RAISE EXCEPTION 'No se puede cambiar de categoría ni de año mientras esté inscrito en un concurso.';
            END IF;
        END IF;

        IF (in_en_prensa OR in_en_documento_oficial)
                AND (in_fuentes IS NULL OR char_length(trim(in_fuentes)) = 0) THEN
            RAISE EXCEPTION 'Fuentes es valor obligatorio si se ha indicado en prensa o documento oficial.';
        END IF;

        IF in_nombre IS NULL THEN
            auxNombre := was.nombre;
        ELSE
            auxNombre := in_nombre;
        END IF;

        -- Verificar que coincide el nombre de la obra con el de la agrupación
        IF in_interpreta_literaria IS NOT NULL THEN
            BEGIN
                SELECT general_accc.__verifica_agrupacionanio_obra('Literaria', auxNombre, in_interpreta_literaria)
                    INTO auxCharacter;
            EXCEPTION WHEN others THEN
                RAISE EXCEPTION '%', SQLERRM;
            END;
        END IF;

        IF in_interpreta_musical IS NOT NULL THEN
            BEGIN
                SELECT general_accc.__verifica_agrupacionanio_obra('Musical', auxNombre, in_interpreta_musical)
                    INTO auxCharacter;
            EXCEPTION WHEN others THEN
                RAISE EXCEPTION '%', SQLERRM;
            END;
        END IF;

        IF in_utiliza_tipo IS NOT NULL THEN
            BEGIN
                SELECT general_accc.__verifica_agrupacionanio_obra('Grafica', auxNombre, in_utiliza_tipo)
                    INTO auxCharacter;
            EXCEPTION WHEN others THEN
                RAISE EXCEPTION '%', SQLERRM;
            END;
        END IF;

        -- Verificar lo mismo en caso de cambio del nombre de la agrupacion
        -- esto puede generar bloqueos en el lado del usuario, impediría cambiar
        -- el nombre de obra y agrupación

        BEGIN
            UPDATE general_accc.agrupacion_anio SET
                                                entidad = COALESCE(in_entidad, was.entidad),
                                                -- admite cambiar una agrupación de entidad
                                                nombre = COALESCE(in_nombre, was.nombre),
                                                anio = COALESCE(in_anio, was.anio),
                                                categoria = COALESCE(in_categoria::general_accc.enum_categoria, was.categoria),
                                                modalidad = COALESCE(in_modalidad, was.modalidad),
                                                submodalidad = COALESCE(in_submodalidad, was.submodalidad),
                                                composicion = COALESCE(in_composicion::general_accc.enum_composicion, was.composicion),
                                                num_componentes = COALESCE(in_num_componentes, was.num_componentes),
                                                presupuesto = COALESCE(in_presupuesto, was.presupuesto),
                                                lugar_ensayo = COALESCE(in_lugar_ensayo, was.lugar_ensayo),
                                                observaciones = COALESCE(in_observaciones, was.observaciones),
                                                en_prensa = COALESCE(in_en_prensa, was.en_prensa),
                                                en_documento_oficial = COALESCE(in_en_documento_oficial, was.en_documento_oficial),
                                                fuentes = COALESCE(in_fuentes, was.fuentes),
                                                observaciones_gestion = COALESCE(in_observaciones_gestion, was.observaciones_gestion),
                                                interpreta_literaria = COALESCE(in_interpreta_literaria, was.interpreta_literaria),
                                                interpreta_musical = COALESCE(in_interpreta_musical, was.interpreta_musical),
                                                utiliza_tipo = COALESCE(in_utiliza_tipo, was.utiliza_tipo)
                WHERE id = in_id;
            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha actualizado la agrupación. (%) %', SQLSTATE, SQLERRM;
        END;

    END IF;
    RAISE EXCEPTION 'El identificador de la agrupación no ha sido recibido o no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: modifica_id_agrupacionanio; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.modifica_id_agrupacionanio (in_id_nuevo integer,
                                                                    in_id_viejo integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
BEGIN

    SELECT id INTO auxInteger FROM general_accc.agrupacion_anio WHERE id = in_id_nuevo;
    IF NOT FOUND AND in_id_nuevo < 100000 THEN
                                   -- Valor inicial de la secuencia de AgrupacionAnio
    	UPDATE general_accc.agrupacion_anio SET
                                      id = in_id_nuevo
            WHERE id = in_id_viejo;
        RETURN 'OK';
    END IF;
    RAISE EXCEPTION 'El identificador ya está en uso o no puede utilizarse.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: baja_agrupacionanio; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_agrupacionanio (in_entidad integer,
                                                             in_id integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
    was general_accc.agrupacion_anio%ROWTYPE;
BEGIN

    SELECT * INTO was
        FROM general_accc.agrupacion_anio
        WHERE entidad = in_entidad
            AND id = in_id;
    IF FOUND THEN

        -- La relación con instrumentos no es innecesario gestionarla,
        -- al desaparecer la agrupación se eliminan las tuplas relacionadas en
        -- cascada en cuanto a las obras interpretadas, la eliminación de la
        -- agrupación no se hará si tiene obras asociadas

        IF was.interpreta_literaria IS NOT NULL OR was.interpreta_musical IS NOT NULL OR was.utiliza_tipo IS NOT NULL THEN
            RAISE EXCEPTION 'La agrupación tiene obras asociadas.';
        END IF;

        BEGIN
            DELETE FROM general_accc.agrupacion_anio
                WHERE entidad = in_entidad
                    AND id = in_id;

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha eliminado. (%) %', SQLSTATE, SQLERRM;
        END;

        BEGIN
            SELECT general_accc.baja_agrupacion (in_entidad)
                INTO auxCharacter;
        EXCEPTION WHEN others THEN
            RETURN 'OK';
        END;

        RETURN 'OK';

    END IF;
    RAISE EXCEPTION 'No existe la agrupación con identificador: %.', in_id;

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: baja_literaria_agrupacionanio; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_literaria_agrupacionanio (in_entidad integer,
                                                                       in_id integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
    was general_accc.agrupacion_anio%ROWTYPE;
BEGIN

    SELECT * INTO was
        FROM general_accc.agrupacion_anio
        WHERE entidad = in_entidad
            AND id = in_id;
    IF FOUND THEN
            UPDATE general_accc.agrupacion_anio SET
                                                interpreta_literaria = null
                WHERE entidad = in_entidad
                    AND id = in_id;

        RETURN 'OK';

    END IF;
    RAISE EXCEPTION 'No existe la agrupación con identificador: %.', in_id;

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: baja_musical_agrupacionanio; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_musical_agrupacionanio (in_entidad integer,
                                                                     in_id integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
    was general_accc.agrupacion_anio%ROWTYPE;
BEGIN

    SELECT * INTO was
        FROM general_accc.agrupacion_anio
        WHERE entidad = in_entidad
            AND id = in_id;
    IF FOUND THEN
            UPDATE general_accc.agrupacion_anio SET
                                                interpreta_musical = null
                WHERE entidad = in_entidad
                    AND id = in_id;

        RETURN 'OK';

    END IF;
    RAISE EXCEPTION 'No existe la agrupación con identificador: %.', in_id;

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: baja_tipo_agrupacionanio; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_tipo_agrupacionanio (in_entidad integer,
                                                                  in_id integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
    was general_accc.agrupacion_anio%ROWTYPE;
BEGIN

    SELECT * INTO was
        FROM general_accc.agrupacion_anio
        WHERE entidad = in_entidad
            AND id = in_id;
    IF FOUND THEN
            UPDATE general_accc.agrupacion_anio SET
                                                utiliza_tipo = null
                WHERE entidad = in_entidad
                    AND id = in_id;

        RETURN 'OK';

    END IF;
    RAISE EXCEPTION 'No existe la agrupación con identificador: %.', in_id;

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;






-- Creación de procedimientos almacenados para altas, baja y modificación
-- de las relaciones con AgrupacionAnio



--
-- Nombre: alta_agrupacion_usa_instrumento; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.alta_agrupacion_usa_instrumento (in_instrumento integer,
                                                                         in_agrupacion_anio integer,
                                                                         in_total integer)
RETURNS character varying AS $$
BEGIN

    INSERT INTO general_accc.usa(instrumento, agrupacion_anio, total)
        VALUES (in_instrumento, in_agrupacion_anio, in_total);
    RETURN 'OK';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION 'No se ha creado la relación. (%) %', SQLSTATE, SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: modifica_agrupacion_usa_instrumento; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.modifica_agrupacion_usa_instrumento (in_id integer,
                                                                             in_instrumento integer,
                                                                             in_agrupacion_anio integer,
                                                                             in_total integer)
RETURNS character varying AS $$
DECLARE
    was general_accc.usa%ROWTYPE;
BEGIN

    SELECT * INTO was
        FROM general_accc.usa
        WHERE id = in_id
            AND agrupacion_anio = in_agrupacion_anio;
        -- No tiene sentido cambiar un instrumento de agrupación, sino a las
        -- agrupaciones cambiarles los instrumentos
    IF FOUND THEN
        BEGIN
            UPDATE general_accc.usa SET
                                    instrumento = COALESCE (in_instrumento, was.instrumento),
                                    total = COALESCE (in_total, was.total)
                WHERE id = in_id;

            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha actualizado la relación. (%) %', SQLSTATE, SQLERRM;
        END;

    END IF;

    RAISE EXCEPTION 'La relación no existe o la agrupación es errónea.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: baja_agrupacion_usa_instrumento; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_agrupacion_usa_instrumento (in_id integer,
                                                                         in_instrumento integer,
                                                                         in_agrupacion_anio integer)
RETURNS character varying AS $$
DECLARE
    was general_accc.usa%ROWTYPE;
BEGIN

    SELECT * INTO was
        FROM general_accc.usa
        WHERE id = in_id
            AND agrupacion_anio = in_agrupacion_anio
            AND instrumento = in_instrumento;
    IF FOUND THEN
        BEGIN
            DELETE FROM general_accc.usa
                WHERE id = in_id;
            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha eliminado la relación. (%) %', SQLSTATE, SQLERRM;

        END;
    END IF;

    RAISE EXCEPTION 'No existe la relación.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: __alta_rol; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.__alta_rol (in_id integer,
                                                    in_persona integer,
                                                    in_agrupacion_anio integer,
                                                    in_tipo_rol character varying)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
BEGIN
    SELECT COUNT (id) INTO auxInteger
        FROM general_accc.rol
        WHERE persona = in_persona
            AND agrupacion_anio = in_agrupacion_anio
            AND tipo_rol = in_tipo_rol::general_accc.enum_rol;

    IF in_tipo_rol = 'Voz' AND auxInteger > 0 THEN
        RAISE EXCEPTION 'La misma persona no puede adoptar más de un registro de voz.';

    ELSEIF in_tipo_rol = 'Orquesta' AND auxInteger > 0 THEN
        RAISE EXCEPTION 'La misma persona no puede utilizar más de un instrumento.';

    ELSEIF in_tipo_rol = 'Cargo' AND auxInteger > 0 THEN
        SELECT setval('general_accc.seq_rol'::regclass, currval('general_accc.seq_rol'::regclass) - 1, true)
            INTO auxInteger;
        RAISE EXCEPTION '__CARGO__';
    END IF;

    INSERT INTO general_accc.rol(id, persona, agrupacion_anio, tipo_rol)
        VALUES (in_id, in_persona, in_agrupacion_anio, in_tipo_rol::general_accc.enum_rol);

    RETURN 'OK';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: alta_cargo_agrupacionanio; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.alta_cargo_agrupacionanio (in_persona integer,
                                                                   in_agrupacion_anio integer,
                                                                   in_tipo_cargo character varying,
                                                                   in_observaciones text)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
BEGIN

    -- Verificación de los cargos existentes para la agrupación:
    -- solo un 'Presidente', un 'Director' y un 'Representante legal'

    IF in_tipo_cargo = 'Presidente' OR in_tipo_cargo = 'Director' OR in_tipo_cargo = 'Representante legal' THEN
        SELECT COUNT(cargo.rol) INTO auxInteger
            FROM general_accc.cargo AS cargo
                INNER JOIN general_accc.rol AS rol
                    ON cargo.rol = rol.id
            WHERE rol.agrupacion_anio = in_agrupacion_anio
                AND cargo.tipo_cargo = in_tipo_cargo::general_accc.enum_cargo;
        IF auxInteger > 0 THEN
            RAISE EXCEPTION 'Esta agrupación ya tiene un %.', in_tipo_cargo;
        END IF;
    END IF;

    SELECT nextval('general_accc.seq_rol'::regclass) INTO auxInteger;
    BEGIN
        SELECT general_accc.__alta_rol(auxInteger, in_persona, in_agrupacion_anio, 'Cargo')
            INTO auxCharacter;
    EXCEPTION WHEN others THEN
        IF SQLERRM = '__CARGO__' THEN
            SELECT id INTO auxInteger
                FROM general_accc.rol
                WHERE rol.agrupacion_anio = in_agrupacion_anio
                    AND rol.persona = in_persona;
        ELSEIF SQLERRM <> 'OK' THEN
            RAISE EXCEPTION '%', SQLERRM;
        END IF;
    END;

    BEGIN
        INSERT INTO general_accc.cargo (rol, tipo_cargo, observaciones)
            VALUES (auxInteger, in_tipo_cargo::general_accc.enum_cargo, in_observaciones);
        RETURN 'OK';

    EXCEPTION WHEN others THEN
        SELECT setval('general_accc.seq_rol'::regclass, currval('general_accc.seq_rol'::regclass) - 1, true) INTO auxInteger;
        RAISE EXCEPTION 'No se ha creado la relación de cargo. (%) %', SQLSTATE, SQLERRM;

    END;

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: modifica_cargo_agrupacionanio; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.modifica_cargo_agrupacionanio (in_id integer,
                                                                       in_rol integer,
                                                                       in_persona integer,
                                                                       in_agrupacion_anio integer,
                                                                       in_tipo_cargo character varying,
                                                                       in_observaciones text)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    was general_accc.cargo%ROWTYPE;
BEGIN

    -- Primero se verifica que el rol es de tipo cargo y que el par persona y
    -- agrupación no haya cambiado, para ello habrá que dar la baja del rol y
    -- el alta al nuevo par
    SELECT cargo.* INTO was
        FROM general_accc.cargo AS cargo
            INNER JOIN general_accc.rol AS rol
                ON cargo.rol = rol.id
        WHERE cargo.id = in_id
            AND cargo.rol = in_rol
            AND rol.persona = in_persona
            AND rol.agrupacion_anio = in_agrupacion_anio
            AND rol.tipo_rol = 'Cargo'::general_accc.enum_rol;
    IF FOUND THEN

        IF in_tipo_cargo IS NOT NULL AND in_tipo_cargo <> was.tipo_cargo::character varying THEN
            -- La consulta solo se hace si cambia el valor de tipo_cargo

            IF in_tipo_cargo = 'Presidente' OR in_tipo_cargo = 'Director' OR in_tipo_cargo = 'Representante legal' THEN
                SELECT COUNT(cargo.rol) INTO auxInteger
                    FROM general_accc.cargo AS cargo
                        INNER JOIN general_accc.rol AS rol
                            ON cargo.rol = rol.id
                    WHERE rol.agrupacion_anio = in_agrupacion_anio
                        AND cargo.tipo_cargo = in_tipo_cargo::general_accc.enum_cargo;
                IF auxInteger > 0 THEN
                    RAISE EXCEPTION 'Esta agrupación ya tiene un %.', in_tipo_cargo;
                END IF;
            END IF;
        END IF;

        BEGIN
            UPDATE general_accc.cargo SET
                                      tipo_cargo = COALESCE (in_tipo_cargo::general_accc.enum_cargo, was.tipo_cargo),
                                      observaciones = COALESCE (in_observaciones, was.observaciones)
                WHERE id = in_id
                    AND rol = in_rol;
            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha actualizado la relación. (%) %', SQLSTATE, SQLERRM;
        END;

    END IF;
    RAISE EXCEPTION 'El rol de tipo cargo no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: baja_cargo_agrupacionanio; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_cargo_agrupacionanio (in_id integer,
                                                                   in_rol integer,
                                                                   in_persona integer,
                                                                   in_agrupacion_anio integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    was general_accc.cargo%ROWTYPE;
BEGIN

    SELECT rol INTO auxInteger
        FROM general_accc.cargo
            WHERE id = in_id
                AND rol = in_rol;
    IF FOUND THEN
        BEGIN
            DELETE FROM general_accc.cargo
                WHERE id = in_id
                    AND rol = in_rol;

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha eliminado la relación de cargo. (%) %', SQLSTATE, SQLERRM;
        END;

        BEGIN
            DELETE FROM general_accc.rol
                WHERE id = in_rol
                    AND persona = in_persona
                    AND agrupacion_anio = in_agrupacion_anio;
            RETURN 'OK';

        EXCEPTION WHEN others THEN
            -- Si no se ha eliminado es porque la persona ostenta otro cargo en
            -- esta agrupación
            RETURN 'OK';
        END;

    END IF;

    RAISE EXCEPTION 'Este cargo entre persona y agrupación no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: alta_orquesta_agrupacionanio; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.alta_orquesta_agrupacionanio (in_persona integer,
                                                                      in_agrupacion_anio integer,
                                                                      in_instrumento integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
BEGIN

    SELECT id INTO auxInteger
        FROM general_accc.rol
        WHERE persona = in_persona
            AND agrupacion_anio = in_agrupacion_anio
            AND tipo_rol = 'Voz'::general_accc.enum_rol;
    IF NOT FOUND THEN

        SELECT nextval('general_accc.seq_rol'::regclass) INTO auxInteger;
        BEGIN
            SELECT general_accc.__alta_rol(auxInteger, in_persona, in_agrupacion_anio, 'Orquesta')
                INTO auxCharacter;
        EXCEPTION WHEN others THEN
            RAISE EXCEPTION '%', SQLERRM;
        END;

        BEGIN
            INSERT INTO general_accc.orquesta (rol, instrumento)
                VALUES (auxInteger, in_instrumento);
            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha creado el rol de orquesta. (%) %', SQLSTATE, SQLERRM;
        END;

    END IF;
    RAISE EXCEPTION 'La persona tiene perfil de voz en la agrupación.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: modifica_orquesta_agrupacionanio; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.modifica_orquesta_agrupacionanio (in_rol integer,
                                                                          in_persona integer,
                                                                          in_agrupacion_anio integer,
                                                                          in_instrumento integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    wasRol general_accc.rol%ROWTYPE;
    wasOrquesta general_accc.orquesta%ROWTYPE;
BEGIN
    SELECT orq.* INTO wasOrquesta
        FROM general_accc.orquesta AS orq
            INNER JOIN general_accc.rol AS rol
                ON orq.rol = rol.id
        WHERE persona = in_persona
            AND agrupacion_anio = in_agrupacion_anio
            AND rol.id = in_rol;
    IF FOUND THEN
        SELECT * INTO wasRol FROM general_accc.rol WHERE id = in_rol;
        BEGIN
            UPDATE general_accc.orquesta SET
                                         instrumento = COALESCE (in_instrumento, wasOrquesta.instrumento)
                WHERE rol = in_rol;
            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha actualizado. (%) %', SQLSTATE, SQLERRM;
        END;

    END IF;
    RAISE EXCEPTION 'El rol de tipo orquesta no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: baja_orquesta_agrupacionanio; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_orquesta_agrupacionanio (in_rol integer,
                                                                      in_persona integer,
                                                                      in_agrupacion_anio integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
BEGIN

    SELECT orq.rol INTO auxInteger
        FROM general_accc.orquesta AS orq
            INNER JOIN general_accc.rol AS rol
                ON orq.rol = rol.id
            WHERE rol.persona = in_persona
                AND rol.agrupacion_anio = in_agrupacion_anio
                AND rol.id = in_rol;
    IF FOUND THEN
        BEGIN
            DELETE FROM general_accc.orquesta WHERE rol = in_rol;

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha eliminado el rol orquesta. (%) %', SQLSTATE, SQLERRM;
        END;

        BEGIN
            DELETE FROM general_accc.rol
                WHERE id = in_rol
                    AND persona = in_persona
                    AND agrupacion_anio = in_agrupacion_anio;
            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha eliminado el rol. (%) %', SQLSTATE, SQLERRM;
        END;

    END IF;

    RAISE EXCEPTION 'Esta relación entre persona y agrupación no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: alta_voz_agrupacionanio; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.alta_voz_agrupacionanio (in_persona integer,
                                                                 in_agrupacion_anio integer,
                                                                 in_tipo_voz character varying)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
BEGIN

    SELECT id INTO auxInteger
        FROM general_accc.rol
        WHERE persona = in_persona
            AND agrupacion_anio = in_agrupacion_anio
            AND tipo_rol = 'Orquesta'::general_accc.enum_rol;
    IF NOT FOUND THEN

        SELECT nextval('general_accc.seq_rol'::regclass) INTO auxInteger;
        BEGIN
            SELECT general_accc.__alta_rol(auxInteger, in_persona, in_agrupacion_anio, 'Voz')
                INTO auxCharacter;
        EXCEPTION WHEN others THEN
            RAISE EXCEPTION '%', SQLERRM;
        END;

        BEGIN
            INSERT INTO general_accc.voz (rol, tipo_voz)
                VALUES (currval('general_accc.seq_rol'::regclass), in_tipo_voz::general_accc.enum_voz);

            RETURN 'OK';
        EXCEPTION WHEN others THEN
            SELECT setval('general_accc.seq_rol'::regclass, currval('general_accc.seq_rol'::regclass) - 1, true) INTO auxInteger;
            RAISE EXCEPTION 'No se ha creado la relación de voz. (%) %', SQLSTATE, SQLERRM;
        END;
    END IF;

    RAISE EXCEPTION 'La persona tiene perfil de orquesta en la agrupación.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: modifica_voz_agrupacionanio; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.modifica_voz_agrupacionanio (in_rol integer,
                                                                     in_persona integer,
                                                                     in_agrupacion_anio integer,
                                                                     in_tipo_voz character varying)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    wasRol general_accc.rol%ROWTYPE;
    wasVoz general_accc.voz%ROWTYPE;
BEGIN
    SELECT voz.* INTO wasVoz
        FROM general_accc.voz AS voz
            INNER JOIN general_accc.rol AS rol
                ON voz.rol = rol.id
        WHERE persona = in_persona
            AND agrupacion_anio = in_agrupacion_anio
            AND rol.id = in_rol;
    IF FOUND THEN
        SELECT * INTO wasRol FROM general_accc.rol WHERE id = in_rol;
        BEGIN
            UPDATE general_accc.voz SET
                                    tipo_voz = COALESCE (in_tipo_voz::general_accc.enum_voz, wasVoz.tipo_voz)
                WHERE rol = in_rol;
            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha actualizado. (%) %', SQLSTATE, SQLERRM;
        END;

    END IF;

    RAISE EXCEPTION 'El rol de voz orquesta no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: baja_voz_agrupacionanio; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_voz_agrupacionanio (in_rol integer,
                                                                 in_persona integer,
                                                                 in_agrupacion_anio integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    was general_accc.voz%ROWTYPE;
BEGIN

    SELECT orq.rol INTO auxInteger
        FROM general_accc.voz AS orq
            INNER JOIN general_accc.rol AS rol
                ON orq.rol = rol.id
            WHERE rol.persona = in_persona
                AND rol.agrupacion_anio = in_agrupacion_anio
                AND rol.id = in_rol;

    IF FOUND THEN
        BEGIN
            DELETE FROM general_accc.voz WHERE rol = in_rol;

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha eliminado el rol de tipo voz. (%) %', SQLSTATE, SQLERRM;
        END;

        BEGIN
            DELETE FROM general_accc.rol
                WHERE id = in_rol
                    AND persona = in_persona
                    AND agrupacion_anio = in_agrupacion_anio;
            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha eliminado el rol. (%) %', SQLSTATE, SQLERRM;
        END;

    END IF;

    RAISE EXCEPTION 'Esta relación entre persona y agrupación no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: alta_participa; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.alta_participa (in_agrupacion_anio integer,
                                                        in_concurso integer,
                                                        in_clasificacion integer,
                                                        in_puntos integer,
                                                        in_fase character varying)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    wasAgrupacion general_accc.agrupacion_anio%ROWTYPE;
    wasEvento general_accc.evento%ROWTYPE;
    wasConcurso general_accc.concurso%ROWTYPE;
BEGIN

    SELECT agrupacion_anio INTO auxInteger
        FROM general_accc.participa
        WHERE agrupacion_anio = in_agrupacion_anio
            AND concurso = in_concurso;
    IF NOT FOUND THEN

        SELECT * INTO wasAgrupacion
            FROM general_accc.agrupacion_anio
            WHERE id = in_agrupacion_anio;

        SELECT * INTO wasEvento
            FROM general_accc.evento
            WHERE id = in_concurso;

        SELECT * INTO wasConcurso
            FROM general_accc.concurso
            WHERE evento = in_concurso;

        IF wasAgrupacion.anio <> wasEvento.anio THEN
            RAISE EXCEPTION 'La agrupación no es del mismo año que el concurso.';
        END IF;
        IF wasAgrupacion.categoria <> wasConcurso.categoria THEN
            RAISE EXCEPTION 'La agrupación no es de la misma categoría que el concurso.';
        END IF;

        -- Controlar si ya hay una agrupación para la misma modalidad con la misma clasificación
        SELECT COUNT(agrup.id) INTO auxInteger 
            FROM general_accc.participa AS participa
            INNER JOIN general_accc.agrupacion_anio AS agrup
                ON participa.agrupacion_anio = agrup.id
            INNER JOIN general_accc.premio AS premio
            	ON participa.concurso = premio.concurso
                    AND premio.modalidad = agrup.modalidad
                    AND premio.premio = participa.clasificacion
                WHERE participa.concurso = in_concurso
                AND participa.clasificacion = in_clasificacion
                AND premio.modalidad = (SELECT modalidad
                                           FROM general_accc.agrupacion_anio
                                           WHERE id = in_agrupacion_anio);
        IF auxInteger > 0 THEN
            RAISE EXCEPTION 'Existe una agrupación con este premio en este concurso en la modalidad.';
        END IF;

        INSERT INTO general_accc.participa(agrupacion_anio, concurso, clasificacion,
                                           puntos, fase)
            VALUES (in_agrupacion_anio, in_concurso, in_clasificacion, in_puntos, in_fase);
        RETURN 'OK';

    END IF;

    RAISE EXCEPTION 'La relación entre agrupación y concurso ya existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: modifica_participa; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.modifica_participa (in_agrupacion_anio integer,
                                                            in_concurso integer,
                                                            in_clasificacion integer,
                                                            in_puntos integer,
                                                            in_fase character varying)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    was general_accc.participa%ROWTYPE;
BEGIN

    SELECT * INTO was
        FROM general_accc.participa
        WHERE agrupacion_anio = in_agrupacion_anio
            AND concurso = in_concurso;
    IF FOUND THEN

        -- No se controla que cambie la categoría, para ello hay que dar de baja
        -- la agrupación en el concurso y dar alta en la nueva categoría

        -- Controlar si ya hay una agrupación para la misma modalidad con la
        -- misma clasificación
        SELECT COUNT(agrup.id) INTO auxInteger
            FROM general_accc.premio AS premio
                INNER JOIN general_accc.participa AS participa
                    ON premio.concurso = participa.concurso
                INNER JOIN general_accc.agrupacion_anio AS agrup
                    ON agrup.id = participa.agrupacion_anio
            WHERE
                participa.clasificacion = in_clasificacion
                AND premio.concurso = in_concurso
                AND agrup.modalidad = premio.modalidad
                AND agrup.id <> in_agrupacion_anio;

        IF auxInteger > 0 THEN
            RAISE EXCEPTION 'Existe una agrupación con este premio en este concurso con esta modalidad.';
        END IF;

        BEGIN
            UPDATE general_accc.participa SET
                                          clasificacion = COALESCE (in_clasificacion, was.clasificacion),
                                          puntos = COALESCE (in_puntos, was.puntos),
                                          fase = COALESCE (in_fase, was.fase)
                WHERE agrupacion_anio = in_agrupacion_anio
                    AND concurso = in_concurso;

            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha actualizado la relación. (%) %', SQLSTATE, SQLERRM;
        END;

    END IF;

    RAISE EXCEPTION 'La relación entre agrupación y concurso no existe, o es errónea.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: baja_participa; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_participa (in_agrupacion_anio integer,
                                                        in_concurso integer)
RETURNS character varying AS $$
DECLARE
    was general_accc.participa%ROWTYPE;
BEGIN

    SELECT * INTO was
        FROM general_accc.participa
        WHERE agrupacion_anio = in_agrupacion_anio
            AND concurso = in_concurso;
    IF FOUND THEN
        BEGIN
            DELETE FROM general_accc.participa
                WHERE agrupacion_anio = in_agrupacion_anio
                    AND concurso = in_concurso;
            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha eliminado. (%) %', SQLSTATE, SQLERRM;
        END;

    END IF;

    RAISE EXCEPTION 'La relación entre agrupación y concurso no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;


