-- TFG BBDD Carnaval de Cádiz - Paloma Cortés


-- Creación de procedimientos almacenados para altas, baja y modificación




--
-- Nombre: alta_letra; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.alta_letra (in_signatura_literaria integer,
                                                    in_signatura_letra integer,
                                                    in_autoria_letra integer,
                                                    in_genero character varying,
                                                    in_transcripcion text,
                                                    in_tema integer,
                                                    in_id_doc integer,
                                                    in_3_1_1 character varying,
                                                    in_3_1_2 character varying,
                                                    in_3_1_3 character varying,
                                                    in_tipo_documento character varying,
                                                    in_tipo_soporte character varying,
                                                    in_ubicacion character varying,
                                                    in_deposito character varying,
                                                    in_estante character varying,
                                                    in_url character varying,
                                                    in_3_1_4 character varying,
                                                    in_3_1_5 text,
                                                    in_3_2_1 text,
                                                    in_3_2_2 text,
                                                    in_3_2_3 text,
                                                    in_3_2_4 text,
                                                    in_3_3_1 text,
                                                    in_3_3_2 text,
                                                    in_3_3_3 text,
                                                    in_3_3_4 text,
                                                    in_3_4_1 text,
                                                    in_3_4_2 text,
                                                    in_3_4_3 text,
                                                    in_3_4_4 text,
                                                    in_3_4_5 text,
                                                    in_3_5_1 text,
                                                    in_3_5_2 text,
                                                    in_3_5_3 text,
                                                    in_3_5_4 text,
                                                    in_3_6_1 text,
                                                    in_3_7_1 text,
                                                    in_3_7_2 text,
                                                    in_3_7_3 text,
                                                    in_descriptor integer,
                                                    in_esta_descrito boolean,
                                                    in_esta_digitalizado boolean,
                                                    in_en_deposito boolean,
                                                    in_observaciones_gestion text)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
    auxDoc integer;
BEGIN

    IF in_signatura_literaria IS NULL OR in_tema IS NULL THEN
        -- El tema se debe controlar porque no se ha podido generar la restricción en el DDL
        RAISE EXCEPTION 'El identificador de la obra y el tema son obligatorios.';
    END IF;

    -- Si el género es presentación o popurrit debe hacer solo uno por agrupación
    IF in_genero = 'Popurrí' OR  in_genero = 'Presentación'  THEN
        SELECT COUNT (id) INTO auxInteger
            FROM general_accc.letra
            WHERE obra_literaria = in_signatura_literaria
                AND genero = in_genero::general_accc.enum_genero_letra;

        IF auxInteger > 0 THEN
            RAISE EXCEPTION 'La letra no puede tener más de un %, pero puede adjuntarle documentos.', in_genero;

        END IF;
    END IF;

    IF in_signatura_letra IS NULL THEN
        -- Se genera la signatura secuencial dentro de la obra literaria
        SELECT MAX(signatura) + 1 INTO in_signatura_letra
            FROM general_accc.letra
            WHERE obra_literaria = in_signatura_literaria;
        IF in_signatura_letra IS NULL THEN
            in_signatura_letra := 1;
        END IF;
    END IF;
        -- De darse la signatura y haber duplicidad el error lo dará la inserción,
        -- lo mismo si el autor no se da o no existe, o si el género es erróneo

    IF in_id_doc IS NULL THEN
        SELECT nextval('general_accc.seq_documento'::regclass) INTO auxDoc;
    ELSE
        auxDoc := in_id_doc;
    END IF;

    -- Este identificador es interno, lo que se admite es la signatura
    SELECT nextval('general_accc.seq_letra'::regclass) INTO auxInteger;
    BEGIN
        SELECT general_accc.__doc_nuevo_a_componente (in_signatura_literaria,
                                                      in_signatura_letra, auxDoc,
                                                      in_3_1_1, in_3_1_2, in_3_1_3,
                                                      in_tipo_documento, in_tipo_soporte,
                                                      in_ubicacion, in_deposito,
                                                      in_estante, in_url,
                                                      in_3_1_4, in_3_1_5, in_3_2_1,
                                                      in_3_2_2, in_3_2_3, in_3_2_4,
                                                      in_3_3_1, in_3_3_2, in_3_3_3,
                                                      in_3_3_4, in_3_4_1, in_3_4_2,
                                                      in_3_4_3, in_3_4_4, in_3_4_5,
                                                      in_3_5_1, in_3_5_2, in_3_5_3,
                                                      in_3_5_4, in_3_6_1, in_3_7_1,
                                                      in_3_7_2, in_3_7_3, in_descriptor,
                                                      in_esta_descrito, in_esta_digitalizado,
                                                      in_en_deposito, in_observaciones_gestion)
            INTO auxCharacter;
    EXCEPTION WHEN others THEN
        IF in_id_doc IS NULL THEN
            SELECT setval('general_accc.seq_documento'::regclass, currval('general_accc.seq_documento'::regclass) - 1, true)
                INTO auxInteger;
        END IF;
        RAISE EXCEPTION '%', SQLERRM;
    END;

    BEGIN
        INSERT INTO general_accc.letra (id, obra_literaria, signatura, autoria_letra, genero, transcripcion)
            VALUES (auxInteger, in_signatura_literaria, in_signatura_letra, in_autoria_letra, in_genero::general_accc.enum_genero_letra, in_transcripcion);

    EXCEPTION WHEN others THEN
        RAISE EXCEPTION 'No se ha creado el registro en letra. %', SQLERRM;
    END;

    BEGIN
        SELECT general_accc.__alta_documento_letra(auxDoc, auxInteger)
            INTO auxCharacter;
    EXCEPTION WHEN others THEN
        RAISE EXCEPTION '%', SQLERRM;
    END;

    BEGIN
        SELECT general_accc.alta_tema_letra(in_tema, auxInteger)
            INTO auxCharacter;
    EXCEPTION WHEN others THEN
        RAISE EXCEPTION '%', SQLERRM;
    END;

    RETURN 'OK';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION 'No se ha creado el registro de letra para la obra. %', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: modifica_letra; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.modifica_letra (in_id_letra integer,
                                                        in_signatura_literaria integer,
                                                        in_signatura_letra integer,
                                                        in_autoria_letra integer,
                                                        in_genero character varying,
                                                        in_transcripcion text,
                                                        in_id_doc integer,
                                                        in_3_1_1 character varying,
                                                        in_3_1_2 character varying,
                                                        in_3_1_3 character varying,
                                                        in_tipo_documento character varying,
                                                        in_tipo_soporte character varying,
                                                        in_ubicacion character varying,
                                                        in_deposito character varying,
                                                        in_estante character varying,
                                                        in_url character varying,
                                                        in_3_1_4 character varying,
                                                        in_3_1_5 text,
                                                        in_3_2_1 text,
                                                        in_3_2_2 text,
                                                        in_3_2_3 text,
                                                        in_3_2_4 text,
                                                        in_3_3_1 text,
                                                        in_3_3_2 text,
                                                        in_3_3_3 text,
                                                        in_3_3_4 text,
                                                        in_3_4_1 text,
                                                        in_3_4_2 text,
                                                        in_3_4_3 text,
                                                        in_3_4_4 text,
                                                        in_3_4_5 text,
                                                        in_3_5_1 text,
                                                        in_3_5_2 text,
                                                        in_3_5_3 text,
                                                        in_3_5_4 text,
                                                        in_3_6_1 text,
                                                        in_3_7_1 text,
                                                        in_3_7_2 text,
                                                        in_3_7_3 text,
                                                        in_esta_descrito boolean,
                                                        in_esta_digitalizado boolean,
                                                        in_en_deposito boolean,
                                                        in_observaciones_gestion text)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
    was general_accc.letra%ROWTYPE;
BEGIN

    -- La condición se basa en la premisa de que siempre existe mínimo un documento por componente
    -- el cual será indicado, sea el primero u otro a modificar, por contener los datos de la obra;
    -- aunque los campos del mismo no vayan a actualizarse
    SELECT documento INTO auxInteger
        FROM general_accc.documento_letra
        WHERE documento = in_id_doc
            AND letra = in_id_letra;
    IF FOUND THEN
        SELECT * INTO was
            FROM general_accc.letra
            WHERE id = in_id_letra
                AND obra_literaria = in_signatura_literaria;

        -- Si el género es presentación o popurrí debe haber solo uno por agrupación
        IF in_genero IS NOT NULL
                AND (in_genero = 'Popurrí' OR in_genero = 'Presentación')  THEN
            SELECT COUNT (id) INTO auxInteger
                FROM general_accc.letra
                WHERE id <> in_id_letra
                    AND obra_literaria = in_signatura_literaria
                    AND genero = in_genero::general_accc.enum_genero;

            IF auxInteger > 0 THEN
                RETURN 'La obra literaria no puede tener más de un %. Solo puede adjuntarle documentos.', in_genero;

            END IF;
        END IF;

        -- Valida la composición de in_3_1_1 con las signaturas, requieren la signatura de la obra
        IF in_3_1_1 IS NOT NULL OR in_signatura_letra IS NOT NULL THEN
            BEGIN
                SELECT general_accc.__verifica_codigo_referencia(in_3_1_1, in_signatura_literaria,
                                                                 in_signatura_letra, auxInteger)
                    INTO auxCharacter;
            EXCEPTION WHEN others THEN
                RAISE EXCEPTION '%', SQLERRM;
            END;
        END IF;

        BEGIN
            -- Si ha cambiado la signatura y se repite para la obra la siguiente
            -- instrucción generará el error
            UPDATE general_accc.letra SET
                                      signatura = COALESCE(in_signatura_letra, was.signatura),
                                      autoria_letra = COALESCE(in_autoria_letra, was.autoria_letra),
                                      genero = COALESCE(in_genero::general_accc.enum_genero_letra, was.genero),
                                      transcripcion = COALESCE(in_transcripcion, was.transcripcion)
                WHERE id = in_id_letra;
        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha actualizado el registro en letra. %', SQLERRM;
        END;

        -- Quizás mande todos a null, excepto el identificador de documento
        BEGIN
            SELECT general_accc.__modifica_documento (in_id_doc, in_3_1_1,
                                                      in_3_1_2, in_3_1_3,
                                                      in_tipo_documento,
                                                      in_tipo_soporte,
                                                      in_ubicacion, in_deposito,
                                                      in_estante, in_url,
                                                      in_3_1_4, in_3_1_5,
                                                      in_3_2_1, in_3_2_2, in_3_2_3,
                                                      in_3_2_4, in_3_3_1, in_3_3_2,
                                                      in_3_3_3, in_3_3_4, in_3_4_1,
                                                      in_3_4_2, in_3_4_3, in_3_4_4,
                                                      in_3_4_5, in_3_5_1, in_3_5_2,
                                                      in_3_5_3, in_3_5_4, in_3_6_1,
                                                      in_3_7_1, in_3_7_2, in_3_7_3,
                                                      in_esta_descrito,
                                                      in_esta_digitalizado,
                                                      in_en_deposito,
                                                      in_observaciones_gestion)
                INTO auxCharacter;
        EXCEPTION WHEN others THEN
            RAISE EXCEPTION '%', SQLERRM;
        END;

        RETURN 'OK';

    END IF;
    RAISE EXCEPTION 'El identificador de la letra no ha sido recibido, es erróneo o no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;





--
-- Nombre: baja_letra; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_letra (in_id_letra integer,
                                                    in_signatura_literaria integer,
                                                    in_signatura_letra integer,
                                                    in_id_doc integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
BEGIN

    SELECT documento INTO auxInteger
        FROM general_accc.documento_letra AS d_l
        INNER JOIN general_accc.letra AS l
            ON d_l.letra = l.id
        WHERE d_l.documento = in_id_doc
            AND d_l.letra = in_id_letra
            AND l.obra_literaria = in_signatura_literaria
            AND l.signatura = in_signatura_letra;
    IF FOUND THEN

        SELECT COUNT(documento) INTO auxInteger
            FROM general_accc.documento_letra
            WHERE letra = in_id_letra;
        IF auxInteger > 1 THEN
            RAISE EXCEPTION 'La letra indicada tiene varios documentos vinculados.';
        END IF;

        BEGIN
            SELECT FROM general_accc.baja_documento_letra (in_id_doc, in_id_letra)
                INTO auxCharacter;
        EXCEPTION WHEN others THEN
            IF SQLERRM <> 'La relación entre documento y letra no existe, o es el mínimo asociado a la misma.' THEN
                RAISE EXCEPTION 'No se ha eliminado la relación entre documento y letra. (%) %', SQLSTATE, SQLERRM;
            END IF;
            BEGIN
                DELETE FROM general_accc.documento_letra WHERE letra = in_id_letra;
            EXCEPTION WHEN others THEN
                RAISE EXCEPTION 'No se ha eliminado la relación entre documento y letra. (%) %', SQLSTATE, SQLERRM;
            END;
        END;

        BEGIN
            SELECT general_accc.__baja_documento_componente (in_id_doc, in_id_letra)
                INTO auxCharacter;
        EXCEPTION WHEN others THEN
            RAISE EXCEPTION '%', SQLERRM;
        END;
        
        -- Llegado a este punto se ha eliminado el documento o el mismo está
        -- vinculado a otro elemento del inventario y se puede eliminar la letra
        BEGIN
            DELETE FROM general_accc.letra WHERE id = in_id_letra;
            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha eliminado la letra. (%) %', SQLSTATE, SQLERRM;
        END;
    END IF;
    RAISE EXCEPTION 'La asociación entre documento y letra no existe, o es errónea.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: __alta_documento_letra; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.__alta_documento_letra (in_documento integer,
                                                                in_letra integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
    aux1 character varying;
    aux2 character varying;
BEGIN

    SELECT documento INTO auxInteger
        FROM general_accc.documento_letra
        WHERE documento = in_documento AND letra = in_letra;

    IF NOT FOUND THEN

        SELECT tipo_documento, "3_1_4_nivel_descripcion" INTO aux1, aux2
            FROM general_accc.documento
            WHERE id = in_documento;

        IF aux2 = 'Unidad documental simple'
                AND (aux1 = 'Texto' OR aux1 = 'Audio' OR aux1 = 'Video') THEN
            BEGIN
                INSERT INTO general_accc.documento_letra(documento, letra)
                    VALUES (in_documento, in_letra);
                RETURN 'OK';

            EXCEPTION WHEN others THEN
                RAISE EXCEPTION 'No se ha creado la relación entre documento y letra. (%) %', SQLSTATE, SQLERRM;
            END;
        END IF;
        RAISE EXCEPTION 'Solo las unidades documentales simples de tipo texto, audio y video, se pueden vincular a una letra.';

    END IF;
    RAISE EXCEPTION 'La relación entre la letra y documento ya existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;





--
-- Nombre: modifica_documento_letra; Tipo: FUNCTION; Schema: general_accc
--
-- Esta gestión no procede, para cambiar un documento de una letra, debe estar
-- asociado a otra y dar baja en la pretendida




--
-- Nombre: doc_existente_a_letra; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.doc_existente_a_letra (in_documento integer,
                                                               in_letra integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
BEGIN

    SELECT id INTO auxInteger
        FROM general_accc.documento
        WHERE id = in_documento;
    IF FOUND THEN
        BEGIN
            SELECT general_accc.__alta_documento_letra (in_documento, in_letra)
                INTO auxCharacter;
        EXCEPTION WHEN others THEN
            RAISE EXCEPTION '%', SQLERRM;
        END;
        RETURN 'OK';

    END IF;
    RAISE EXCEPTION 'El documento no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;




--
-- Nombre: baja_documento_letra; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_documento_letra (in_documento integer,
                                                              in_letra integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
BEGIN

    SELECT COUNT(documento) INTO auxInteger
        FROM general_accc.documento_letra
        WHERE letra = in_letra;

    IF auxInteger < 2 THEN
        RAISE EXCEPTION 'La relación entre documento y letra no existe, o es el mínimo asociado a la misma.';
    ELSE
        BEGIN
            DELETE FROM general_accc.documento_letra
                WHERE documento = in_documento
                    AND letra = in_letra;
        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha eliminado la relación entre letra y documento. %', SQLERRM;
        END;

        BEGIN
            SELECT general_accc.__baja_documento_componente (in_documento, in_letra)
                INTO auxCharacter;
        EXCEPTION WHEN others THEN
            RAISE EXCEPTION '%', SQLERRM;
        END;
        RETURN 'OK';

    END IF;

    RAISE EXCEPTION 'La asociación entre documento y letra no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;



--
-- Nombre: doc_nuevo_a_letra; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.doc_nuevo_a_letra (in_id_letra integer,
                                                           in_signatura_literaria integer,
                                                           in_signatura_letra integer,
                                                           in_id_doc integer,
                                                           in_3_1_1 character varying,
                                                           in_3_1_2 character varying,
                                                           in_3_1_3 character varying,
                                                           in_tipo_documento character varying,
                                                           in_tipo_soporte character varying,
                                                           in_ubicacion character varying,
                                                           in_deposito character varying,
                                                           in_estante character varying,
                                                           in_url character varying,
                                                           in_3_1_4 character varying,
                                                           in_3_1_5 text,
                                                           in_3_2_1 text,
                                                           in_3_2_2 text,
                                                           in_3_2_3 text,
                                                           in_3_2_4 text,
                                                           in_3_3_1 text,
                                                           in_3_3_2 text,
                                                           in_3_3_3 text,
                                                           in_3_3_4 text,
                                                           in_3_4_1 text,
                                                           in_3_4_2 text,
                                                           in_3_4_3 text,
                                                           in_3_4_4 text,
                                                           in_3_4_5 text,
                                                           in_3_5_1 text,
                                                           in_3_5_2 text,
                                                           in_3_5_3 text,
                                                           in_3_5_4 text,
                                                           in_3_6_1 text,
                                                           in_3_7_1 text,
                                                           in_3_7_2 text,
                                                           in_3_7_3 text,
                                                           in_descriptor integer,
                                                           in_esta_descrito boolean,
                                                           in_esta_digitalizado boolean,
                                                           in_en_deposito boolean,
                                                           in_observaciones_gestion text)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxDoc integer;
    auxCharacter character varying;
BEGIN

    IF in_id_letra IS NULL OR in_signatura_literaria IS NULL OR in_signatura_letra IS NULL THEN
        -- El necesario para identificar el componente y las necesarias para
        -- construir el código de referencia
        RAISE EXCEPTION 'No se ha recibido el identificador o las signaturas de la letra.';
    END IF;

    IF in_id_doc IS NULL THEN
        SELECT nextval('general_accc.seq_documento'::regclass) INTO auxDoc;
    ELSE
        auxDoc := in_id_doc;
    END IF;

    BEGIN
        SELECT general_accc.__doc_nuevo_a_componente (in_signatura_literaria,
                                                      in_signatura_letra, auxDoc,
                                                      in_3_1_1, in_3_1_2, in_3_1_3,
                                                      in_tipo_documento, in_tipo_soporte,
                                                      in_ubicacion, in_deposito,
                                                      in_estante, in_url,
                                                      in_3_1_4, in_3_1_5, in_3_2_1,
                                                      in_3_2_2, in_3_2_3, in_3_2_4,
                                                      in_3_3_1, in_3_3_2, in_3_3_3,
                                                      in_3_3_4, in_3_4_1, in_3_4_2,
                                                      in_3_4_3, in_3_4_4, in_3_4_5,
                                                      in_3_5_1, in_3_5_2, in_3_5_3,
                                                      in_3_5_4, in_3_6_1, in_3_7_1,
                                                      in_3_7_2, in_3_7_3, in_descriptor,
                                                      in_esta_descrito, in_esta_digitalizado,
                                                      in_en_deposito, in_observaciones_gestion)
            INTO auxCharacter;
    EXCEPTION WHEN others THEN
        IF in_id_doc IS NULL THEN
            SELECT setval('general_accc.seq_documento'::regclass, currval('general_accc.seq_documento'::regclass) - 1, true)
                INTO auxInteger;
        END IF;
        RAISE EXCEPTION '%', SQLERRM;
    END;

    BEGIN
        SELECT general_accc.__alta_documento_letra(auxDoc, in_id_letra)
            INTO auxCharacter;
    EXCEPTION WHEN others THEN
        IF in_id_doc IS NULL THEN
            SELECT setval('general_accc.seq_documento'::regclass, currval('general_accc.seq_documento'::regclass) - 1, true)
                INTO auxInteger;
        END IF;
        RAISE EXCEPTION '%', SQLERRM;
    END;

    RETURN 'OK';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: alta_tema_letra; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.alta_tema_letra (in_tema integer,
                                                         in_letra integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
BEGIN

    SELECT tema INTO auxInteger FROM general_accc.tema_letra
            WHERE tema = in_tema AND letra = in_letra;
    IF NOT FOUND THEN
        INSERT INTO general_accc.tema_letra(tema, letra)
            VALUES (in_tema, in_letra);

        RETURN 'OK';

    END IF;

    RAISE EXCEPTION 'La relación entre el tema y letra ya existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;




--
-- Nombre: modifica_tema_letra; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.modifica_tema_letra (in_id integer,
                                                             in_tema integer,
                                                             in_letra integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
    was general_accc.tema_letra%ROWTYPE;
BEGIN

    SELECT * INTO was
        FROM general_accc.tema_letra
        WHERE id = in_id
            AND letra = in_letra;
    IF FOUND THEN
        BEGIN
            -- No tiene sentido cambiar un tema de letra, sino a las letras
            -- cambiarles los temas
            UPDATE general_accc.tema_letra SET
                                           tema = COALESCE (in_tema, was.tema)
                WHERE id = in_id;
            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha actualizado la relación. (%) %', SQLSTATE, SQLERRM;
        END;

    END IF;
    RAISE EXCEPTION 'La asociación entre tema y letra no existe, o es errónea.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: baja_tema_letra; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_tema_letra (in_id integer,
                                                         in_tema integer,
                                                         in_letra integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
BEGIN

    SELECT COUNT(id) INTO auxInteger
        FROM general_accc.tema_letra
        WHERE letra = in_letra;
    IF auxInteger > 1 THEN
        BEGIN
            DELETE FROM general_accc.tema_letra
                WHERE id = in_id
                AND tema = in_tema
                AND letra = in_letra;
            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha eliminado. (%) %', SQLSTATE, SQLERRM;
        END;

    END IF;

    RAISE EXCEPTION 'Se ha indicado el único tema relacionado a la letra, o la relación no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;

