-- TFG BBDD Carnaval de Cádiz - Paloma Cortés


-- Creación de procedimientos almacenados para altas, baja y modificación




--
--
-- Nombre: alta_musica; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.alta_musica (in_signatura_musical integer,
                                                     in_signatura_musica integer,
                                                     in_autoria_musica integer,
                                                     in_genero character varying,
                                                     in_id_doc integer,
                                                     in_3_1_1 character varying,
                                                     in_3_1_2 character varying,
                                                     in_3_1_3 character varying,
                                                     in_tipo_documento character varying,
                                                     in_tipo_soporte character varying,
                                                     in_ubicacion character varying,
                                                     in_deposito character varying,
                                                     in_estante character varying,
                                                     in_url character varying,
                                                     in_3_1_4 character varying,
                                                     in_3_1_5 text,
                                                     in_3_2_1 text,
                                                     in_3_2_2 text,
                                                     in_3_2_3 text,
                                                     in_3_2_4 text,
                                                     in_3_3_1 text,
                                                     in_3_3_2 text,
                                                     in_3_3_3 text,
                                                     in_3_3_4 text,
                                                     in_3_4_1 text,
                                                     in_3_4_2 text,
                                                     in_3_4_3 text,
                                                     in_3_4_4 text,
                                                     in_3_4_5 text,
                                                     in_3_5_1 text,
                                                     in_3_5_2 text,
                                                     in_3_5_3 text,
                                                     in_3_5_4 text,
                                                     in_3_6_1 text,
                                                     in_3_7_1 text,
                                                     in_3_7_2 text,
                                                     in_3_7_3 text,
                                                     in_descriptor integer,
                                                     in_esta_descrito boolean,
                                                     in_esta_digitalizado boolean,
                                                     in_en_deposito boolean,
                                                     in_observaciones_gestion text)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxDoc integer;
    auxCharacter character varying;
BEGIN

    IF in_signatura_musical IS NULL THEN
        -- Se debe controlar porque no se ha podido generar la restricción en el DDL
        RAISE EXCEPTION 'El identificador de la obra es obligatorio.';
    END IF;

    IF in_signatura_musica IS NULL THEN
        -- Se genera la signatura secuencial dentro de la obra literaria
        SELECT MAX(signatura) + 1 INTO in_signatura_musica
            FROM general_accc.musica
            WHERE obra_musical = in_signatura_musical;
        IF in_signatura_musica IS NULL THEN
            in_signatura_musica := 1;
        END IF;
    END IF;
        -- De darse la signatura y haber duplicidad el error lo dará la inserción
        -- similar si el autor no se da o no existe y si el género es erróneo

    IF in_id_doc IS NULL THEN
        SELECT nextval('general_accc.seq_documento'::regclass) INTO auxDoc;
    ELSE
        auxDoc := in_id_doc;
    END IF;

    -- Este identificador es interno, lo que se admite es la signatura
    SELECT nextval('general_accc.seq_musica'::regclass) INTO auxInteger;
    BEGIN
        SELECT general_accc.__doc_nuevo_a_componente (in_signatura_musical,
                                                      in_signatura_musica, auxDoc,
                                                      in_3_1_1, in_3_1_2, in_3_1_3,
                                                      in_tipo_documento, in_tipo_soporte,
                                                      in_ubicacion, in_deposito,
                                                      in_estante, in_url,
                                                      in_3_1_4, in_3_1_5, in_3_2_1,
                                                      in_3_2_2, in_3_2_3, in_3_2_4,
                                                      in_3_3_1, in_3_3_2, in_3_3_3,
                                                      in_3_3_4, in_3_4_1, in_3_4_2,
                                                      in_3_4_3, in_3_4_4, in_3_4_5,
                                                      in_3_5_1, in_3_5_2, in_3_5_3,
                                                      in_3_5_4, in_3_6_1, in_3_7_1,
                                                      in_3_7_2, in_3_7_3, in_descriptor,
                                                      in_esta_descrito, in_esta_digitalizado,
                                                      in_en_deposito, in_observaciones_gestion)
            INTO auxCharacter;
    EXCEPTION WHEN others THEN
        IF in_id_doc IS NULL THEN
            SELECT setval('general_accc.seq_documento'::regclass, currval('general_accc.seq_documento'::regclass) - 1, true)
                INTO auxInteger;
        END IF;
        RAISE EXCEPTION '%', SQLERRM;
    END;

    BEGIN
        INSERT INTO general_accc.musica (id, obra_musical, signatura, autoria_musica, genero)
            VALUES (auxInteger, in_signatura_musical, in_signatura_musica, in_autoria_musica, in_genero::general_accc.enum_genero_musica);

    EXCEPTION WHEN others THEN
        RAISE EXCEPTION 'No se ha creado el registro en musica. %', SQLERRM;
    END;

    BEGIN
        SELECT general_accc.__alta_documento_musica (auxDoc, auxInteger)
            INTO auxCharacter;
    EXCEPTION WHEN others THEN
        RAISE EXCEPTION '%', SQLERRM;
    END;

    RETURN 'OK';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION 'No se ha creado el registro de musica para la obra. %', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: modifica_musica; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.modifica_musica (in_id_musica integer,
                                                         in_signatura_musical integer,
                                                         in_signatura_musica integer,
                                                         in_autoria_musica integer,
                                                         in_genero character varying,
                                                         in_id_doc integer,
                                                         in_3_1_1 character varying,
                                                         in_3_1_2 character varying,
                                                         in_3_1_3 character varying,
                                                         in_tipo_documento character varying,
                                                         in_tipo_soporte character varying,
                                                         in_ubicacion character varying,
                                                         in_deposito character varying,
                                                         in_estante character varying,
                                                         in_url character varying,
                                                         in_3_1_4 character varying,
                                                         in_3_1_5 text,
                                                         in_3_2_1 text,
                                                         in_3_2_2 text,
                                                         in_3_2_3 text,
                                                         in_3_2_4 text,
                                                         in_3_3_1 text,
                                                         in_3_3_2 text,
                                                         in_3_3_3 text,
                                                         in_3_3_4 text,
                                                         in_3_4_1 text,
                                                         in_3_4_2 text,
                                                         in_3_4_3 text,
                                                         in_3_4_4 text,
                                                         in_3_4_5 text,
                                                         in_3_5_1 text,
                                                         in_3_5_2 text,
                                                         in_3_5_3 text,
                                                         in_3_5_4 text,
                                                         in_3_6_1 text,
                                                         in_3_7_1 text,
                                                         in_3_7_2 text,
                                                         in_3_7_3 text,
                                                         in_esta_descrito boolean,
                                                         in_esta_digitalizado boolean,
                                                         in_en_deposito boolean,
                                                         in_observaciones_gestion text)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
    was general_accc.musica%ROWTYPE;
BEGIN

    -- La condición se basa en la premisa de que siempre existe mínimo un documento por componente
    -- el cual será indicado, sea el primero u otro a modificar, por contener los datos de la obra;
    -- aunque los campos del mismo no vayan a actualizarse
    SELECT documento INTO auxInteger
        FROM general_accc.documento_musica
        WHERE documento = in_id_doc
            AND musica = in_id_musica;
    IF FOUND THEN
        SELECT * INTO was
            FROM general_accc.musica
            WHERE id = in_id_musica
                AND obra_musical = in_signatura_musical;

        -- Valida la composición de in_3_1_1 con las signaturas, requieren la signatura de la obra
        IF in_3_1_1 IS NOT NULL OR in_signatura_musica IS NOT NULL THEN
            BEGIN
                SELECT general_accc.__verifica_codigo_referencia(in_3_1_1, in_signatura_musical,
                                                                 in_signatura_musica, auxInteger)
                    INTO auxCharacter;
            EXCEPTION WHEN others THEN
                RAISE EXCEPTION '%', SQLERRM;
            END;
        END IF;

        BEGIN
            -- Si ha cambiado la signatura y se repite para la obra la siguiente
            -- instrucción generará el error
            UPDATE general_accc.musica SET
                                       signatura = COALESCE(in_signatura_musica, was.signatura),
                                       autoria_musica = COALESCE(in_autoria_musica, was.autoria_musica),
                                       genero = COALESCE(in_genero::general_accc.enum_genero_musica, was.genero)
                WHERE id = in_id_musica;
        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha actualizado el registro en musica. %', SQLERRM;
        END;

        -- Quizás mande todos a null, excepto el identificador de documento
        BEGIN
            SELECT general_accc.__modifica_documento (in_id_doc, in_3_1_1,
                                                      in_3_1_2, in_3_1_3,
                                                      in_tipo_documento,
                                                      in_tipo_soporte,
                                                      in_ubicacion, in_deposito,
                                                      in_estante, in_url,
                                                      in_3_1_4, in_3_1_5,
                                                      in_3_2_1, in_3_2_2, in_3_2_3,
                                                      in_3_2_4, in_3_3_1, in_3_3_2,
                                                      in_3_3_3, in_3_3_4, in_3_4_1,
                                                      in_3_4_2, in_3_4_3, in_3_4_4,
                                                      in_3_4_5, in_3_5_1, in_3_5_2,
                                                      in_3_5_3, in_3_5_4, in_3_6_1,
                                                      in_3_7_1, in_3_7_2, in_3_7_3,
                                                      in_esta_descrito,
                                                      in_esta_digitalizado,
                                                      in_en_deposito,
                                                      in_observaciones_gestion)
                INTO auxCharacter;
        EXCEPTION WHEN others THEN
            RAISE EXCEPTION '%', SQLERRM;
        END;

        RETURN 'OK';

    END IF;
    RAISE EXCEPTION 'El identificador de la musica no ha sido recibido, es erróneo o no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: baja_musica; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_musica (in_id_musica integer,
                                                     in_signatura_musical integer,
                                                     in_signatura_musica integer,
                                                     in_id_doc integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
BEGIN

    SELECT documento INTO auxInteger
        FROM general_accc.documento_musica AS d_m
        INNER JOIN general_accc.musica AS mus
            ON d_m.musica = mus.id
        WHERE d_m.documento = in_id_doc
            AND d_m.musica = in_id_musica
            AND mus.obra_musical = in_signatura_musical
            AND mus.signatura = in_signatura_musica;

    IF FOUND THEN

        SELECT COUNT(documento) INTO auxInteger
            FROM general_accc.documento_musica
            WHERE musica = in_id_musica;
        IF auxInteger > 1 THEN
            RAISE EXCEPTION 'La música indicada tiene varios documentos vinculados.';
        END IF;

        BEGIN
            SELECT FROM general_accc.baja_documento_musica (in_id_doc, in_id_musica)
                INTO auxCharacter;
        EXCEPTION WHEN others THEN
            IF SQLERRM <> 'La relación entre documento y música no existe, o es el mínimo asociado a la misma.' THEN
                RAISE EXCEPTION 'No se ha eliminado la relación entre documento y música. (%) %', SQLSTATE, SQLERRM;
            END IF;
            BEGIN
                DELETE FROM general_accc.documento_musica WHERE musica = in_id_musica;
            EXCEPTION WHEN others THEN
                RAISE EXCEPTION 'No se ha eliminado la relación entre documento y música. (%) %', SQLSTATE, SQLERRM;
            END;
        END;

        BEGIN
            SELECT general_accc.__baja_documento_componente (in_id_doc, in_id_musica)
                INTO auxCharacter;
        EXCEPTION WHEN others THEN
            RAISE EXCEPTION '%', SQLERRM;
        END;

        -- Llegado a este punto se ha eliminado el documento o el mismo está vinculado
        -- a otro elemento del inventario y se puede eliminar la composición musical
        BEGIN
            DELETE FROM general_accc.musica WHERE id = in_id_musica;
            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha eliminado la música. (%) %', SQLSTATE, SQLERRM;
        END;

    END IF;
    RAISE EXCEPTION 'La asociación entre documento y música no existe, o es errónea.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: __alta_documento_musica; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.__alta_documento_musica (in_documento integer,
                                                                 in_musica integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
    aux1 character varying;
    aux2 character varying;
BEGIN

    SELECT documento INTO auxInteger
        FROM general_accc.documento_musica
        WHERE documento = in_documento AND musica = in_musica;
    IF NOT FOUND THEN

        SELECT tipo_documento, "3_1_4_nivel_descripcion" INTO aux1, aux2
            FROM general_accc.documento
            WHERE id = in_documento;

        IF aux2 = 'Unidad documental simple'
                AND (aux1 = 'Texto' OR aux1 = 'Imagen' OR aux1 = 'Audio' OR aux1 = 'Video') THEN
            BEGIN
                INSERT INTO general_accc.documento_musica (documento, musica)
                    VALUES (in_documento, in_musica);
                RETURN 'OK';

            EXCEPTION WHEN others THEN
                RAISE EXCEPTION 'No se ha creado la relación entre documento y música. (%) %', SQLSTATE, SQLERRM;
            END;
        END IF;
        RAISE EXCEPTION 'Solo las unidades documentales simples de tipo texto, imagen, audio y video, se pueden vincular a una música.';

    END IF;
    RAISE EXCEPTION 'La relación entre la música y documento ya existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: modifica_documento_musica; Tipo: FUNCTION; Schema: general_accc
--
-- Esta gestión no procede, para cambiar un documento de una componente, debe
-- estar asociado a otra y dar baja en la pretendida.
--




--
-- Nombre: doc_existente_a_musica; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.doc_existente_a_musica (in_documento integer,
                                                                in_musica integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
BEGIN

    SELECT id INTO auxInteger
        FROM general_accc.documento
        WHERE id = in_documento;
    IF FOUND THEN
        BEGIN
            SELECT general_accc.__alta_documento_musica (in_documento, in_musica)
                INTO auxCharacter;
        EXCEPTION WHEN others THEN
            RAISE EXCEPTION '%', SQLERRM;
        END;
        RETURN 'OK';

    END IF;
    RAISE EXCEPTION 'El documento no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;



--
-- Nombre: baja_documento_musica; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_documento_musica (in_documento integer,
                                                               in_musica integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
BEGIN

    SELECT COUNT(documento) INTO auxInteger
        FROM general_accc.documento_musica
        WHERE musica = in_musica;

    IF auxInteger < 2 THEN
        RAISE EXCEPTION 'La relación entre documento y música no existe, o es el mínimo asociado a la misma.';
    ELSE
        BEGIN
            DELETE FROM general_accc.documento_musica
                WHERE documento = in_documento
                    AND musica = in_musica;

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha eliminado la relación entre música y documento. %', SQLERRM;
        END;

        BEGIN
            SELECT general_accc.__baja_documento_componente (in_documento, in_musica)
                INTO auxCharacter;
        EXCEPTION WHEN others THEN
            RAISE EXCEPTION '%', SQLERRM;
        END;
        RETURN 'OK';

    END IF;

    RAISE EXCEPTION 'La asociación entre documento y música no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;


--
-- Nombre: doc_nuevo_a_musica; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.doc_nuevo_a_musica (in_id_musica integer,
                                                            in_signatura_musical integer,
                                                            in_signatura_musica integer,
                                                            in_id_doc integer,
                                                            in_3_1_1 character varying,
                                                            in_3_1_2 character varying,
                                                            in_3_1_3 character varying,
                                                            in_tipo_documento character varying,
                                                            in_tipo_soporte character varying,
                                                            in_ubicacion character varying,
                                                            in_deposito character varying,
                                                            in_estante character varying,
                                                            in_url character varying,
                                                            in_3_1_4 character varying,
                                                            in_3_1_5 text,
                                                            in_3_2_1 text,
                                                            in_3_2_2 text,
                                                            in_3_2_3 text,
                                                            in_3_2_4 text,
                                                            in_3_3_1 text,
                                                            in_3_3_2 text,
                                                            in_3_3_3 text,
                                                            in_3_3_4 text,
                                                            in_3_4_1 text,
                                                            in_3_4_2 text,
                                                            in_3_4_3 text,
                                                            in_3_4_4 text,
                                                            in_3_4_5 text,
                                                            in_3_5_1 text,
                                                            in_3_5_2 text,
                                                            in_3_5_3 text,
                                                            in_3_5_4 text,
                                                            in_3_6_1 text,
                                                            in_3_7_1 text,
                                                            in_3_7_2 text,
                                                            in_3_7_3 text,
                                                            in_descriptor integer,
                                                            in_esta_descrito boolean,
                                                            in_esta_digitalizado boolean,
                                                            in_en_deposito boolean,
                                                            in_observaciones_gestion text)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxDoc integer;
    auxCharacter character varying;
BEGIN

    IF in_id_musica IS NULL OR in_signatura_musical IS NULL OR in_signatura_musica IS NULL THEN
        -- El necesario para identificar el componente y las necesarias para
        -- construir el código de referencia
        RAISE EXCEPTION 'No se ha recibido el identificador o las signaturas de la música.';
    END IF;

    IF in_id_doc IS NULL THEN
        SELECT nextval('general_accc.seq_documento'::regclass) INTO auxDoc;
    ELSE
        auxDoc := in_id_doc;
    END IF;

    BEGIN
        SELECT general_accc.__doc_nuevo_a_componente (in_signatura_musical,
                                                      in_signatura_musica, auxDoc,
                                                      in_3_1_1, in_3_1_2, in_3_1_3,
                                                      in_tipo_documento, in_tipo_soporte,
                                                      in_ubicacion, in_deposito,
                                                      in_estante, in_url,
                                                      in_3_1_4, in_3_1_5, in_3_2_1,
                                                      in_3_2_2, in_3_2_3, in_3_2_4,
                                                      in_3_3_1, in_3_3_2, in_3_3_3,
                                                      in_3_3_4, in_3_4_1, in_3_4_2,
                                                      in_3_4_3, in_3_4_4, in_3_4_5,
                                                      in_3_5_1, in_3_5_2, in_3_5_3,
                                                      in_3_5_4, in_3_6_1, in_3_7_1,
                                                      in_3_7_2, in_3_7_3, in_descriptor,
                                                      in_esta_descrito, in_esta_digitalizado,
                                                      in_en_deposito, in_observaciones_gestion)
            INTO auxCharacter;
    EXCEPTION WHEN others THEN
        IF in_id_doc IS NULL THEN
            SELECT setval('general_accc.seq_documento'::regclass, currval('general_accc.seq_documento'::regclass) - 1, true)
                INTO auxInteger;
        END IF;
        RAISE EXCEPTION '%', SQLERRM;
    END;

    BEGIN
        SELECT general_accc.__alta_documento_musica(auxDoc, in_id_musica)
            INTO auxCharacter;
    EXCEPTION WHEN others THEN
        IF in_id_doc IS NULL THEN
            SELECT setval('general_accc.seq_documento'::regclass, currval('general_accc.seq_documento'::regclass) - 1, true)
                INTO auxInteger;
        END IF;
        RAISE EXCEPTION '%', SQLERRM;
    END;

    RETURN 'OK';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;



