-- TFG BBDD Carnaval de Cádiz - Paloma Cortés


-- Creación de procedimientos almacenados para altas, baja y modificación de Disenio




--
-- Nombre: alta_disenio; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.alta_disenio (in_signatura_grafica integer,
                                                      in_signatura_disenio integer,
                                                      in_autoria_disenio integer,
                                                      in_tipo_disenio character varying,
                                                      in_tema integer,
                                                      in_id_doc integer,
                                                      in_3_1_1 character varying,
                                                      in_3_1_2 character varying,
                                                      in_3_1_3 character varying,
                                                      in_tipo_documento character varying,
                                                      in_tipo_soporte character varying,
                                                      in_ubicacion character varying,
                                                      in_deposito character varying,
                                                      in_estante character varying,
                                                      in_url character varying,
                                                      in_3_1_4 character varying,
                                                      in_3_1_5 text,
                                                      in_3_2_1 text,
                                                      in_3_2_2 text,
                                                      in_3_2_3 text,
                                                      in_3_2_4 text,
                                                      in_3_3_1 text,
                                                      in_3_3_2 text,
                                                      in_3_3_3 text,
                                                      in_3_3_4 text,
                                                      in_3_4_1 text,
                                                      in_3_4_2 text,
                                                      in_3_4_3 text,
                                                      in_3_4_4 text,
                                                      in_3_4_5 text,
                                                      in_3_5_1 text,
                                                      in_3_5_2 text,
                                                      in_3_5_3 text,
                                                      in_3_5_4 text,
                                                      in_3_6_1 text,
                                                      in_3_7_1 text,
                                                      in_3_7_2 text,
                                                      in_3_7_3 text,
                                                      in_descriptor integer,
                                                      in_esta_descrito boolean,
                                                      in_esta_digitalizado boolean,
                                                      in_en_deposito boolean,
                                                      in_observaciones_gestion text)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxDoc integer;
    auxCharacter character varying;
BEGIN

    IF in_signatura_grafica IS NULL THEN
        -- Se debe controlar porque no se ha podido generar la restricción en el DDL
        RAISE EXCEPTION 'El identificador de la obra es obligatorio.';
    END IF;

    IF in_signatura_disenio IS NULL THEN
        -- Se genera la signatura secuencial dentro de la obra literaria
        SELECT MAX(signatura) + 1 INTO in_signatura_disenio
            FROM general_accc.disenio
            WHERE obra_grafica = in_signatura_grafica;
        IF in_signatura_disenio IS NULL THEN
            in_signatura_disenio := 1;
        END IF;
    END IF;
        -- De darse la signatura y haber duplicidad el error lo dará la inserción
        -- similar si el autor no se da o no existe y si el género es erróneo

    IF in_id_doc IS NULL THEN
        SELECT nextval('general_accc.seq_documento'::regclass) INTO auxDoc;
    ELSE
        auxDoc := in_id_doc;
    END IF;

    -- Este identificador es interno, lo que se admite es la signatura
    SELECT nextval('general_accc.seq_disenio'::regclass) INTO auxInteger;

    BEGIN
        SELECT general_accc.__doc_nuevo_a_componente (in_signatura_grafica,
                                                      in_signatura_disenio, auxDoc,
                                                      in_3_1_1, in_3_1_2, in_3_1_3,
                                                      in_tipo_documento, in_tipo_soporte,
                                                      in_ubicacion, in_deposito,
                                                      in_estante, in_url,
                                                      in_3_1_4, in_3_1_5, in_3_2_1,
                                                      in_3_2_2, in_3_2_3, in_3_2_4,
                                                      in_3_3_1, in_3_3_2, in_3_3_3,
                                                      in_3_3_4, in_3_4_1, in_3_4_2,
                                                      in_3_4_3, in_3_4_4, in_3_4_5,
                                                      in_3_5_1, in_3_5_2, in_3_5_3,
                                                      in_3_5_4, in_3_6_1, in_3_7_1,
                                                      in_3_7_2, in_3_7_3, in_descriptor,
                                                      in_esta_descrito, in_esta_digitalizado,
                                                      in_en_deposito, in_observaciones_gestion)
            INTO auxCharacter;
    EXCEPTION WHEN others THEN
        RAISE EXCEPTION '%', SQLERRM;
    END;

    BEGIN
        INSERT INTO general_accc.disenio (id, obra_grafica, signatura, autoria_disenio, tipo_disenio)
            VALUES (auxInteger, in_signatura_grafica, in_signatura_disenio, in_autoria_disenio, in_tipo_disenio::general_accc.enum_disenio);

    EXCEPTION WHEN others THEN
        RAISE EXCEPTION 'No se ha creado el registro en disenio. %', SQLERRM;
    END;

    BEGIN
        SELECT general_accc.alta_tema_disenio(in_tema, auxInteger)
            INTO auxCharacter;
    EXCEPTION WHEN others THEN
        RAISE EXCEPTION '%', SQLERRM;
    END;

    BEGIN
        SELECT general_accc.__alta_documento_disenio (auxDoc, auxInteger)
            INTO auxCharacter;
    EXCEPTION WHEN others THEN
        RAISE EXCEPTION '%', SQLERRM;
    END;

    RETURN 'OK';

EXCEPTION WHEN others THEN
    IF in_id_doc IS NULL THEN
        SELECT setval('general_accc.seq_documento'::regclass, currval('general_accc.seq_documento'::regclass) - 1, true)
            INTO auxInteger;
    END IF;
    RAISE EXCEPTION 'No se ha creado el registro de disenio para la obra. %', SQLERRM;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: modifica_disenio; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.modifica_disenio (in_id_disenio integer,
                                                          in_signatura_grafica integer,
                                                          in_signatura_disenio integer,
                                                          in_autoria_disenio integer,
                                                          in_tipo_disenio character varying,
                                                          in_id_doc integer,
                                                          in_3_1_1 character varying,
                                                          in_3_1_2 character varying,
                                                          in_3_1_3 character varying,
                                                          in_tipo_documento character varying,
                                                          in_tipo_soporte character varying,
                                                          in_ubicacion character varying,
                                                          in_deposito character varying,
                                                          in_estante character varying,
                                                          in_url character varying,
                                                          in_3_1_4 character varying,
                                                          in_3_1_5 text,
                                                          in_3_2_1 text,
                                                          in_3_2_2 text,
                                                          in_3_2_3 text,
                                                          in_3_2_4 text,
                                                          in_3_3_1 text,
                                                          in_3_3_2 text,
                                                          in_3_3_3 text,
                                                          in_3_3_4 text,
                                                          in_3_4_1 text,
                                                          in_3_4_2 text,
                                                          in_3_4_3 text,
                                                          in_3_4_4 text,
                                                          in_3_4_5 text,
                                                          in_3_5_1 text,
                                                          in_3_5_2 text,
                                                          in_3_5_3 text,
                                                          in_3_5_4 text,
                                                          in_3_6_1 text,
                                                          in_3_7_1 text,
                                                          in_3_7_2 text,
                                                          in_3_7_3 text,
                                                          in_esta_descrito boolean,
                                                          in_esta_digitalizado boolean,
                                                          in_en_deposito boolean,
                                                          in_observaciones_gestion text)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
    was general_accc.disenio%ROWTYPE;
BEGIN

    -- La condición se basa en la premisa de que siempre existe mínimo un documento por componente
    -- el cual será indicado, sea el primero u otro a modificar, por contener los datos de la obra;
    -- aunque los campos del mismo no vayan a actualizarse
    SELECT documento INTO auxInteger
        FROM general_accc.documento_disenio
        WHERE documento = in_id_doc
            AND disenio = in_id_disenio;

    IF FOUND THEN
        SELECT * INTO was
            FROM general_accc.disenio
            WHERE id = in_id_disenio
                AND obra_grafica = in_signatura_grafica;

        -- Valida la composición de in_3_1_1 con las signaturas, requieren la signatura de la obra
        IF in_3_1_1 IS NOT NULL OR in_signatura_disenio IS NOT NULL THEN
            BEGIN
                SELECT general_accc.__verifica_codigo_referencia(in_3_1_1, in_signatura_grafica,
                                                                 in_signatura_disenio, auxInteger)
                    INTO auxCharacter;
            EXCEPTION WHEN others THEN
                RAISE EXCEPTION '%', SQLERRM;
            END;
        END IF;

        BEGIN
            -- Si ha cambiado la signatura y se repite para la obra la siguiente
            -- instrucción generará el error
            UPDATE general_accc.disenio SET
                                        signatura = COALESCE(in_signatura_disenio, was.signatura),
                                        autoria_disenio = COALESCE(in_autoria_disenio, was.autoria_disenio),
                                        tipo_disenio = COALESCE(in_tipo_disenio::general_accc.enum_disenio, was.tipo_disenio)
                WHERE id = in_id_disenio;
        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha actualizado el registro en disenio. %', SQLERRM;
        END;

        -- Quizás mande todos a null, excepto el identificador de documento
        BEGIN
            SELECT general_accc.__modifica_documento (in_id_doc, in_3_1_1,
                                                      in_3_1_2, in_3_1_3,
                                                      in_tipo_documento,
                                                      in_tipo_soporte,
                                                      in_ubicacion, in_deposito,
                                                      in_estante, in_url,
                                                      in_3_1_4, in_3_1_5,
                                                      in_3_2_1, in_3_2_2, in_3_2_3,
                                                      in_3_2_4, in_3_3_1, in_3_3_2,
                                                      in_3_3_3, in_3_3_4, in_3_4_1,
                                                      in_3_4_2, in_3_4_3, in_3_4_4,
                                                      in_3_4_5, in_3_5_1, in_3_5_2,
                                                      in_3_5_3, in_3_5_4, in_3_6_1,
                                                      in_3_7_1, in_3_7_2, in_3_7_3,
                                                      in_esta_descrito,
                                                      in_esta_digitalizado,
                                                      in_en_deposito,
                                                      in_observaciones_gestion)
                INTO auxCharacter;
        EXCEPTION WHEN others THEN
            RAISE EXCEPTION '%', SQLERRM;
        END;

        RETURN 'OK';

    END IF;
    RAISE EXCEPTION 'El identificador del diseño no ha sido recibido, es erróneo o no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: baja_disenio; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_disenio (in_id_disenio integer,
                                                      in_signatura_grafica integer,
                                                      in_signatura_disenio integer,
                                                      in_id_doc integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
BEGIN

    SELECT documento INTO auxInteger
        FROM general_accc.documento_disenio AS d_d
        INNER JOIN general_accc.disenio AS d
            ON d_d.disenio = d.id
        WHERE d_d.documento = in_id_doc
            AND d_d.disenio = in_id_disenio
            AND d.obra_grafica = in_signatura_grafica
            AND d.signatura = in_signatura_disenio;

    IF FOUND THEN

        SELECT COUNT(documento) INTO auxInteger
            FROM general_accc.documento_disenio
            WHERE disenio = in_id_disenio;
        IF auxInteger > 1 THEN
            RAISE EXCEPTION 'El diseño indicado tiene varios documentos vinculados.';
        END IF;

        BEGIN
            SELECT FROM general_accc.baja_documento_disenio (in_id_doc, in_id_disenio)
                INTO auxCharacter;
        EXCEPTION WHEN others THEN
            IF SQLERRM <> 'La relación entre documento y diseño no existe, o es el mínimo asociado al mismo.' THEN
                RAISE EXCEPTION 'No se ha eliminado la relación entre documento y diseño. (%) %', SQLSTATE, SQLERRM;
            END IF;
            BEGIN
                DELETE FROM general_accc.documento_disenio WHERE disenio = in_id_disenio;
            EXCEPTION WHEN others THEN
                RAISE EXCEPTION 'No se ha eliminado la relación entre documento y diseño. (%) %', SQLSTATE, SQLERRM;
            END;
        END;

        BEGIN
            SELECT general_accc.__baja_documento_componente (in_id_doc, in_id_disenio)
                INTO auxCharacter;
        EXCEPTION WHEN others THEN
            RAISE EXCEPTION '%', SQLERRM;
        END;
        
        -- Llegado a este punto se ha eliminado el documento o el mismo está vinculado
        -- a otro elemento del inventario y se puede eliminar la letra
        BEGIN
            DELETE FROM general_accc.disenio WHERE id = in_id_disenio;
            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha eliminado la disenio. (%) %', SQLSTATE, SQLERRM;
        END;

    END IF;
    RAISE EXCEPTION 'La asociación entre documento y disenio no existe, o es errónea.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: __alta_documento_disenio; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.__alta_documento_disenio (in_documento integer,
                                                                  in_disenio integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
    aux1 character varying;
    aux2 character varying;
BEGIN

    SELECT documento INTO auxInteger
        FROM general_accc.documento_disenio
        WHERE documento = in_documento AND disenio = in_disenio;
    IF NOT FOUND THEN

        SELECT tipo_documento, "3_1_4_nivel_descripcion" INTO aux1, aux2
            FROM general_accc.documento
            WHERE id = in_documento;

        IF aux2 = 'Unidad documental simple'
                AND (aux1 = 'Texto' OR aux1 = 'Imagen' OR aux1 = 'Video') THEN
            BEGIN
                INSERT INTO general_accc.documento_disenio (documento, disenio)
                    VALUES (in_documento, in_disenio);
                RETURN 'OK';

            EXCEPTION WHEN others THEN
                RAISE EXCEPTION 'No se ha creado la relación entre documento y diseño. (%) %', SQLSTATE, SQLERRM;
            END;
        END IF;
        RAISE EXCEPTION 'Solo las unidades documentales simples de tipo texto, imagen o video se pueden vincular a un diseño.';

    END IF;
    RAISE EXCEPTION 'La relación entre la diseño y documento ya existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: modifica_documento_disenio; Tipo: FUNCTION; Schema: general_accc
--
-- Esta gestión no procede, para cambiar un documento de una diseño, debe estar
-- asociado a otro y dar baja en el pretendido.
--




--
-- Nombre: doc_existente_a_disenio; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.doc_existente_a_disenio (in_documento integer,
                                                                 in_disenio integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
BEGIN

    SELECT id INTO auxInteger
        FROM general_accc.documento
        WHERE id = in_documento;
    IF FOUND THEN
        BEGIN
            SELECT general_accc.__alta_documento_disenio (in_documento, in_disenio)
                INTO auxCharacter;
        EXCEPTION WHEN others THEN
            RAISE EXCEPTION '%', SQLERRM;
        END;
        RETURN 'OK';

    END IF;
    RAISE EXCEPTION 'El documento no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;




--
-- Nombre: baja_documento_disenio; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_documento_disenio (in_documento integer,
                                                                in_disenio integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
BEGIN

    SELECT COUNT(documento) INTO auxInteger
        FROM general_accc.documento_disenio
        WHERE disenio = in_disenio;

    IF auxInteger < 2 THEN
        RAISE EXCEPTION 'La relación entre documento y diseño no existe, o es el mínimo asociado al mismo.';
    ELSE
        BEGIN
            DELETE FROM general_accc.documento_disenio
                WHERE documento = in_documento
                    AND disenio = in_disenio;

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha eliminado la relación entre diseño y documento. %', SQLERRM;
        END;

        BEGIN
            SELECT general_accc.__baja_documento_componente (in_documento, in_disenio)
                INTO auxCharacter;
        EXCEPTION WHEN others THEN
            RAISE EXCEPTION '%', SQLERRM;
        END;
        RETURN 'OK';

    END IF;

    RAISE EXCEPTION 'La asociación entre documento y diseño no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;




--
-- Nombre: doc_nuevo_a_disenio; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.doc_nuevo_a_disenio (in_id_disenio integer,
                                                             in_signatura_grafica integer,
                                                             in_signatura_disenio integer,
                                                             in_id_doc integer,
                                                             in_3_1_1 character varying,
                                                             in_3_1_2 character varying,
                                                             in_3_1_3 character varying,
                                                             in_tipo_documento character varying,
                                                             in_tipo_soporte character varying,
                                                             in_ubicacion character varying,
                                                             in_deposito character varying,
                                                             in_estante character varying,
                                                             in_url character varying,
                                                             in_3_1_4 character varying,
                                                             in_3_1_5 text,
                                                             in_3_2_1 text,
                                                             in_3_2_2 text,
                                                             in_3_2_3 text,
                                                             in_3_2_4 text,
                                                             in_3_3_1 text,
                                                             in_3_3_2 text,
                                                             in_3_3_3 text,
                                                             in_3_3_4 text,
                                                             in_3_4_1 text,
                                                             in_3_4_2 text,
                                                             in_3_4_3 text,
                                                             in_3_4_4 text,
                                                             in_3_4_5 text,
                                                             in_3_5_1 text,
                                                             in_3_5_2 text,
                                                             in_3_5_3 text,
                                                             in_3_5_4 text,
                                                             in_3_6_1 text,
                                                             in_3_7_1 text,
                                                             in_3_7_2 text,
                                                             in_3_7_3 text,
                                                             in_descriptor integer,
                                                             in_esta_descrito boolean,
                                                             in_esta_digitalizado boolean,
                                                             in_en_deposito boolean,
                                                             in_observaciones_gestion text)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxDoc integer;
    auxCharacter character varying;
BEGIN

    IF in_id_disenio IS NULL OR in_signatura_grafica IS NULL OR in_signatura_disenio IS NULL THEN
        -- El necesario para identificar el componente y las necesarias para
        -- construir el código de referencia
        RAISE EXCEPTION 'No se ha recibido el identificador o las signaturas del diseño.';
    END IF;

    IF in_id_doc IS NULL THEN
        SELECT nextval('general_accc.seq_documento'::regclass) INTO auxDoc;
    ELSE
        auxDoc := in_id_doc;
    END IF;

    BEGIN
        SELECT general_accc.__doc_nuevo_a_componente (in_signatura_grafica,
                                                      in_signatura_disenio, auxDoc,
                                                      in_3_1_1, in_3_1_2, in_3_1_3,
                                                      in_tipo_documento, in_tipo_soporte,
                                                      in_ubicacion, in_deposito,
                                                      in_estante, in_url,
                                                      in_3_1_4, in_3_1_5, in_3_2_1,
                                                      in_3_2_2, in_3_2_3, in_3_2_4,
                                                      in_3_3_1, in_3_3_2, in_3_3_3,
                                                      in_3_3_4, in_3_4_1, in_3_4_2,
                                                      in_3_4_3, in_3_4_4, in_3_4_5,
                                                      in_3_5_1, in_3_5_2, in_3_5_3,
                                                      in_3_5_4, in_3_6_1, in_3_7_1,
                                                      in_3_7_2, in_3_7_3, in_descriptor,
                                                      in_esta_descrito, in_esta_digitalizado,
                                                      in_en_deposito, in_observaciones_gestion)
                INTO auxCharacter;
    EXCEPTION WHEN others THEN
        IF in_id_doc IS NULL THEN
            SELECT setval('general_accc.seq_documento'::regclass, currval('general_accc.seq_documento'::regclass) - 1, true)
                INTO auxInteger;
        END IF;
        RAISE EXCEPTION '%', SQLERRM;
    END;

    BEGIN
        SELECT general_accc.__alta_documento_disenio(auxDoc, in_id_disenio)
            INTO auxCharacter;
    EXCEPTION WHEN others THEN
        IF in_id_doc IS NULL THEN
            SELECT setval('general_accc.seq_documento'::regclass, currval('general_accc.seq_documento'::regclass) - 1, true)
                INTO auxInteger;
        END IF;
        RAISE EXCEPTION '%', SQLERRM;
    END;

    RETURN 'OK';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;




--
-- Nombre: alta_tema_disenio; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.alta_tema_disenio (in_tema integer,
                                                           in_disenio integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
BEGIN

    SELECT tema INTO auxInteger FROM general_accc.tema_disenio
            WHERE tema = in_tema AND disenio = in_disenio;

    IF NOT FOUND THEN

        INSERT INTO general_accc.tema_disenio(tema, disenio)
            VALUES (in_tema, in_disenio);

        RETURN 'OK';

    END IF;

    RAISE EXCEPTION 'La relación entre el tema y diseño ya existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;



--
-- Nombre: modifica_tema_disenio; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.modifica_tema_disenio (in_id integer,
                                                               in_tema integer,
                                                               in_disenio integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
    was general_accc.tema_disenio%ROWTYPE;
BEGIN

    SELECT * INTO was
        FROM general_accc.tema_disenio
        WHERE id = in_id
            AND disenio = in_disenio;
    IF FOUND THEN
        BEGIN
            UPDATE general_accc.tema_disenio SET
                                             tema = COALESCE (in_tema, was.tema)
                WHERE id = in_id;

            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha actualizado la relación. (%) %', SQLSTATE, SQLERRM;
        END;

    END IF;

    RAISE EXCEPTION 'La asociación entre tema y diseño no existe, o es errónea.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;




--
-- Nombre: baja_tema_disenio; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_tema_disenio (in_id integer,
                                                           in_tema integer,
                                                           in_disenio integer)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
BEGIN

    SELECT COUNT(tema) INTO auxInteger
        FROM general_accc.tema_disenio
        WHERE disenio = in_disenio;
    IF auxInteger > 1 THEN
        BEGIN
            DELETE FROM general_accc.tema_disenio
                WHERE id = in_id
                AND tema = in_tema
                AND disenio = in_disenio;
            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha eliminado. (%) %', SQLSTATE, SQLERRM;
        END;

    END IF;

    RAISE EXCEPTION 'Se ha indicado el único tema relacionado al diseño, o la relación no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;





--
-- Nombre: alta_crea_disenio; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.alta_crea_disenio (in_disenio integer,
                                                           in_entidad integer,
                                                           in_coste numeric (10,0),
                                                           in_observaciones text)
RETURNS character varying AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
BEGIN

    SELECT entidad INTO auxInteger
        FROM general_accc.crea
        WHERE disenio = in_disenio
            AND entidad = in_entidad;
    IF NOT FOUND THEN

        INSERT INTO general_accc.crea(disenio, entidad, coste, observaciones)
            VALUES (in_disenio, in_entidad, in_coste, in_observaciones);

        RETURN 'OK';

    END IF;

    RAISE EXCEPTION 'La relación entre el diseño y la entidad ya existe.';


EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;



--
-- Nombre: modifica_crea_disenio; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.modifica_crea_disenio (in_id integer,
                                                               in_disenio integer,
                                                               in_entidad integer,
                                                               in_coste numeric (10,0),
                                                               in_observaciones text)
RETURNS character varying AS $$
DECLARE
    was general_accc.crea%ROWTYPE;
BEGIN

    SELECT * INTO was
        FROM general_accc.crea
        WHERE id = in_id;
    IF FOUND THEN
        BEGIN
            UPDATE general_accc.crea SET
                                     disenio = COALESCE (in_disenio, was.disenio),
                                     entidad = COALESCE (in_entidad, was.entidad),
                                     coste = COALESCE (in_coste, was.coste),
                                     observaciones = COALESCE (in_observaciones, was.observaciones)
                WHERE id = in_id;

            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha actualizado. (%) %', SQLSTATE, SQLERRM;
        END;

    END IF;

    RAISE EXCEPTION 'La asociación entre empresa y diseño no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;



--
-- Nombre: baja_crea_disenio; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.baja_crea_disenio (in_id integer,
                                                           in_disenio integer,
                                                           in_entidad integer)
RETURNS character varying AS $$
DECLARE
    was general_accc.crea%ROWTYPE;
BEGIN

    SELECT * INTO was
        FROM general_accc.crea
        WHERE disenio = in_disenio
            AND entidad = in_entidad;
            
    IF FOUND THEN
        BEGIN
            DELETE FROM general_accc.crea WHERE id = in_id;
            RETURN 'OK';

        EXCEPTION WHEN others THEN
            RAISE EXCEPTION 'No se ha eliminado. (%) %', SQLSTATE, SQLERRM;
        END;

    END IF;

    RAISE EXCEPTION 'La asociación entre entidad y diseño no existe.';

EXCEPTION WHEN others THEN
    RAISE EXCEPTION '%', SQLERRM;

END;
$$ LANGUAGE plpgsql;




