-- TFG BBDD Carnaval de Cádiz - Paloma Cortés


-- Creación de las vistas necesarias para componer el esquema externo


-- Lista de Agrupacion junto con AgrupacionAnio, con todos los atributos.

CREATE OR REPLACE VIEW general_accc."Agrupacion_AgrupacionAnio_Instrumentos" AS
    SELECT agrup.entidad, agrup.provincia, agrup.localidad, agrup.alias, agrup.contacto,
        agrup.observaciones, agrup_a.anio, agrup_a.nombre, agrup_a.categoria, modalidad.modalidad,
        agrup_a.submodalidad, agrup_a.composicion, agrup_a.num_componentes AS componentes,
        agrup_a.presupuesto, agrup_a.lugar_ensayo,
        string_agg(inst.instrumento || ': ' || COALESCE(usa.total, 0), ', ') AS total_instrumento

    FROM general_accc.agrupacion_anio AS agrup_a
        RIGHT OUTER JOIN general_accc.agrupacion AS agrup
            ON agrup_a.entidad = agrup.entidad
        INNER JOIN general_accc.modalidad AS modalidad
            ON agrup_a.modalidad = modalidad.id
        RIGHT OUTER JOIN general_accc.usa AS usa
            ON usa.agrupacion_anio = agrup_a.id
        INNER JOIN general_accc.instrumento AS inst
            ON usa.instrumento = inst.id
    GROUP BY agrup.entidad, agrup.provincia, agrup.localidad, agrup.alias, agrup.contacto,
        agrup.observaciones, agrup_a.anio, agrup_a.nombre, agrup_a.categoria, modalidad.modalidad,
        agrup_a.submodalidad, agrup_a.composicion, agrup_a.num_componentes,
        agrup_a.presupuesto, agrup_a.lugar_ensayo, agrup_a.observaciones
    ORDER BY agrup_a.anio;




-- Lista de AgrupacionAnio junto con Persona por rol, con todos los atributos
-- implicados excepto los de carácter personal.

CREATE OR REPLACE VIEW general_accc."AgrupacionAnio_Roles" AS
    SELECT agrupacion_anio.anio, agrupacion_anio.categoria,
            modalidad.modalidad, agrupacion.alias AS agrupacion,
            agrupacion_anio.nombre AS agrupacion_anio, roles.rol, persona.nombre,
            persona.apellido1, persona.apellido2, persona.alias
    FROM general_accc.agrupacion_anio AS agrupacion_anio
        INNER JOIN general_accc.modalidad AS modalidad
            ON modalidad.id = agrupacion_anio.modalidad
        INNER JOIN general_accc.agrupacion AS agrupacion
            ON agrupacion.entidad = agrupacion_anio.entidad
        INNER JOIN general_accc.rol AS rol
            ON rol.agrupacion_anio = agrupacion_anio.id
        INNER JOIN general_accc.persona AS persona
            ON rol.persona = persona.entidad
        INNER JOIN (SELECT rol as id, 'Cargo ' || tipo_cargo::character varying as rol FROM general_accc.cargo
                        UNION
                    SELECT rol as id, 'Voz ' || tipo_voz::character varying as rol FROM general_accc.voz
                        UNION
                    SELECT rol as id, 'Instrumento ' || instrumento.instrumento as rol FROM general_accc.orquesta AS orquesta
                        INNER JOIN general_accc.instrumento AS instrumento
                            ON orquesta.instrumento = instrumento.id) AS roles
            ON roles.id = rol.id;




-- Lista de Obra de tipo Literaria junto con sus componentes, más las instancias
-- de Documento asociadas. Esta incluye la autoría de cada componente y los temas.

CREATE OR REPLACE VIEW general_accc."AgrupacionAnio_Letras_Tema" AS
    SELECT agrup.anio AS "Año", modalidad.modalidad, agrup.nombre, 
        doc_obra."3_1_1_codigo_referencia" AS SignaturaObra, doc_obra."3_1_2_titulo" AS TituloObra,
        doc_letra."3_1_1_codigo_referencia" AS SignaturaLetra, doc_letra."3_1_2_titulo" AS TituloLetra,
        entidad.autor, letra.genero, tema_letra.temas, letra.transcripcion, doc_letra.*,
        deposito_legal.deposito_legal, deposito_legal.edicion, deposito_legal.anio, deposito_legal.generador

    FROM general_accc.agrupacion_anio AS agrup
        INNER JOIN general_accc.modalidad AS modalidad
            ON agrup.modalidad = modalidad.id
        INNER JOIN general_accc.literaria AS literaria
            ON agrup.interpreta_literaria = literaria.obra
        INNER JOIN general_accc.documento_obra AS d_o
            ON d_o.obra = literaria.obra
        INNER JOIN general_accc.documento AS doc_obra
            ON d_o.documento = doc_obra.id
        INNER JOIN general_accc.letra AS letra
            ON letra.obra_literaria = literaria.obra
        INNER JOIN (SELECT letra, string_agg(t.tema, ', ') AS temas
                        FROM general_accc.tema AS t
                        INNER JOIN general_accc.tema_letra AS t_l
                            ON t.id = t_l.tema
                        GROUP BY t_l.letra) AS tema_letra
            ON letra.id = tema_letra.letra
        INNER JOIN general_accc.documento_letra AS d_l
            ON d_l.letra = letra.id
        INNER JOIN general_accc.documento AS doc_letra
            ON d_l.documento = doc_letra.id
        INNER JOIN (SELECT entidad, nombre || ' ' || apellido1 || ' ' || COALESCE(apellido2, '') AS Autor
                        FROM general_accc.persona
                    UNION
                    SELECT entidad, 'La agrupacion ' || ' (' || COALESCE(alias, '') || ')' AS Autor
                        FROM general_accc.agrupacion) AS entidad
            ON entidad.entidad = letra.autoria_letra
        LEFT OUTER JOIN general_accc.deposito_legal AS deposito_legal
            ON deposito_legal.documento = doc_obra.id;




-- Lista de Obra de tipo Musical junto con sus componentes y las instancias de
-- Documento asociadas.

CREATE OR REPLACE VIEW general_accc."AgrupacionAnio_Musica" AS
    SELECT agrup.anio AS "Año", modalidad.modalidad, agrup.nombre, 
        doc_obra."3_1_1_codigo_referencia" AS SignaturaObra, doc_obra."3_1_2_titulo" AS TituloObra,
        doc_musica."3_1_1_codigo_referencia" AS SignaturaMusica, doc_musica."3_1_2_titulo" AS TituloMusica,
        entidad.autor, musica.genero, doc_musica.*, 
        deposito_legal.deposito_legal, deposito_legal.edicion, deposito_legal.anio, deposito_legal.generador

    FROM general_accc.agrupacion_anio AS agrup
        INNER JOIN general_accc.modalidad AS modalidad
            ON agrup.modalidad = modalidad.id
        INNER JOIN general_accc.musical AS musical
            ON agrup.interpreta_musical = musical.obra
        INNER JOIN general_accc.documento_obra AS d_o
            ON d_o.obra = musical.obra
        INNER JOIN general_accc.documento AS doc_obra
            ON d_o.documento = doc_obra.id
        INNER JOIN general_accc.musica AS musica
            ON musica.obra_musical = musical.obra
        INNER JOIN general_accc.documento_musica AS d_l
            ON d_l.musica = musica.id
        INNER JOIN general_accc.documento AS doc_musica
            ON d_l.documento = doc_musica.id
        INNER JOIN (SELECT entidad, nombre || ' ' || apellido1 || ' ' || COALESCE(apellido2, '') AS Autor
                        FROM general_accc.persona
                    UNION
                    SELECT entidad, 'La agrupacion (' || COALESCE(alias, '') || ')' AS Autor
                        FROM general_accc.agrupacion) AS entidad
            ON entidad.entidad = musica.autoria_musica
        LEFT OUTER JOIN general_accc.deposito_legal AS deposito_legal
            ON deposito_legal.documento = doc_obra.id;




-- Lista de Obra de tipo Grafica con los componentes y las instancias de
-- Documento asociadas. Además de la referencia a la autoría de cada componente
-- y la información contenida en la relación crea.

CREATE OR REPLACE VIEW general_accc."AgrupacionAnio_Tipo_Tema" AS
    SELECT agrup.anio AS "Año", modalidad.modalidad, agrup.nombre, 
        doc_obra."3_1_1_codigo_referencia" AS SignaturaObra, doc_obra."3_1_2_titulo" AS TituloObra,
        doc_disenio."3_1_1_codigo_referencia" AS SignaturaDisenio, doc_disenio."3_1_2_titulo" AS TituloDisenio,
        autor.autor, creadores.creadores,
        disenio.tipo_disenio, tema_disenio.temas, doc_disenio.*, 
        deposito_legal.deposito_legal, deposito_legal.edicion, deposito_legal.anio, deposito_legal.generador

    FROM general_accc.grafica AS grafica
        INNER JOIN general_accc.documento_obra AS d_o
            ON d_o.obra = grafica.obra
        INNER JOIN general_accc.documento AS doc_obra
            ON d_o.documento = doc_obra.id
        INNER JOIN general_accc.disenio AS disenio
            ON disenio.obra_grafica = grafica.obra
        INNER JOIN (SELECT disenio, string_agg(t.tema, ', ') AS temas
                        FROM general_accc.tema AS t
                        INNER JOIN general_accc.tema_disenio AS t_d
                            ON t.id = t_d.tema
                        GROUP BY t_d.disenio) AS tema_disenio
            ON disenio.id = tema_disenio.disenio
        INNER JOIN general_accc.documento_disenio AS d_d
            ON d_d.disenio = disenio.id
        INNER JOIN general_accc.documento AS doc_disenio
            ON d_d.documento = doc_disenio.id
        INNER JOIN (SELECT entidad, nombre || ' ' || apellido1 || ' ' || COALESCE(apellido2, '') AS Autor
                        FROM general_accc.persona
                    UNION
                    SELECT entidad, 'La agrupacion ' || ' (' || COALESCE(alias, '') || ')' AS Autor
                        FROM general_accc.agrupacion
                    UNION
                    SELECT entidad, nombre || ', ' || localidad || ' (' || provincia || ')' AS Autor
                        FROM general_accc.empresa) AS autor
            ON autor.entidad = disenio.autoria_disenio
        LEFT OUTER JOIN (SELECT crea.disenio, string_agg(creador.creador, ', ') AS creadores
                             FROM (SELECT entidad, nombre || ' ' || apellido1 || ' ' || COALESCE(apellido2, '') AS Creador
                                       FROM general_accc.persona
                                   UNION
                                   SELECT entidad, 'La agrupacion (' || COALESCE(alias, '') || ')' AS Creador
                                       FROM general_accc.agrupacion
                                   UNION
                                   SELECT entidad, nombre || ', ' || localidad || ' (' || provincia || ')' AS Creado
                                       FROM general_accc.empresa) AS creador
                             INNER JOIN general_accc.crea AS crea
                                 ON creador.entidad = crea.entidad
                         GROUP BY crea.disenio) AS creadores
            ON creadores.disenio = disenio.id
        LEFT OUTER JOIN general_accc.agrupacion_anio AS agrup
            ON agrup.utiliza_tipo = grafica.obra
        LEFT OUTER JOIN general_accc.modalidad AS modalidad
            ON agrup.modalidad = modalidad.id
        LEFT OUTER JOIN general_accc.deposito_legal AS deposito_legal
            ON deposito_legal.documento = doc_obra.id;




-- Lista de Documento de tipo multimedia y su AgrupacionAnio, incluida la
-- autoría y la dirección del grupo.

CREATE OR REPLACE VIEW general_accc."AgrupacionAnio_Multimedia" AS
    SELECT agrup.entidad, agrup.provincia, agrup.localidad, agrup.alias, agrup.contacto,
        agrup.observaciones, agrup_a.anio, agrup_a.nombre, agrup_a.categoria, modalidad.modalidad,
        doc.*
    FROM general_accc.agrupacion_anio AS agrup_a
        INNER JOIN general_accc.agrupacion AS agrup
            ON agrup_a.entidad = agrup.entidad
        INNER JOIN general_accc.modalidad AS modalidad
            ON agrup_a.modalidad = modalidad.id
        INNER JOIN general_accc.obra AS obra
            ON agrup_a.interpreta_literaria = obra.signatura
                OR agrup_a.interpreta_musical = obra.signatura
                OR agrup_a.utiliza_tipo = obra.signatura
        INNER JOIN (SELECT documento AS id_doc, obra AS id_obra FROM general_accc.documento_obra
                    UNION
                    SELECT documento AS id_doc, letra.obra_literaria AS id_obra
                        FROM general_accc.documento_letra AS d_l
                        INNER JOIN general_accc.letra AS letra
                            ON letra.id = d_l.letra
                    UNION
                    SELECT documento AS id_doc, musica.obra_musical AS id_obra
                        FROM general_accc.documento_musica AS d_m
                        INNER JOIN general_accc.musica AS musica
                            ON musica.id = d_m.musica
                    UNION
                    SELECT documento AS id_doc, disenio.obra_grafica AS id_obra
                        FROM general_accc.documento_disenio AS d_d
                        INNER JOIN general_accc.disenio AS disenio
                            ON disenio.id = d_d.disenio
                    ) AS docs
            ON docs.id_obra = obra.signatura
        INNER JOIN general_accc.documento AS doc
            ON docs.id_doc = doc.id
    WHERE doc.tipo_documento = 'Video'::general_accc.enum_tipodocumento
        OR doc.tipo_documento = 'Audio'::general_accc.enum_tipodocumento
    ORDER BY agrup_a.anio;




-- Lista de Evento con Documento.

CREATE OR REPLACE VIEW general_accc."Eventos_Documento" AS
    SELECT ev.tipo_evento, ev.anio, ev.nombre, ev.localidad, ev.provincia, 
        ev.ubicacion AS ubicacion_evento, ev.descripcion, documento.*
    FROM general_accc.evento AS ev
        LEFT OUTER JOIN (SELECT evento, doc.* FROM general_accc.documento_evento AS d_e
                             INNER JOIN general_accc.documento AS doc
                                 ON d_e.documento = doc.id
                             LEFT OUTER JOIN general_accc.deposito_legal AS dep
                                 ON dep.documento = doc.id) AS documento
            ON documento.evento = ev.id;

            


-- Lista de Concurso con AgrupacionAnio por participa, incluido Premio.

CREATE OR REPLACE VIEW general_accc."AgrupacionAnio_Concursos_Premios" AS
    SELECT agrup.anio, evento.nombre AS Concurso, evento.localidad,
        concurso.categoria, modalidad.modalidad, agrup.nombre,
        participa.clasificacion, participa.puntos, participa.fase,
        premio.cuantia, premio.observaciones

    FROM general_accc.agrupacion_anio AS agrup
        INNER JOIN general_accc.modalidad AS modalidad
            ON agrup.modalidad = modalidad.id
        INNER JOIN general_accc.participa AS participa
            ON agrup.id = participa.agrupacion_anio
        INNER JOIN general_accc.concurso AS concurso
            ON participa.concurso = concurso.evento
        INNER JOIN general_accc.evento AS evento
            ON concurso.evento = evento.id
        LEFT OUTER JOIN general_accc.premio AS premio
            ON concurso.evento = premio.concurso AND participa.clasificacion = premio.premio AND agrup.modalidad = premio.modalidad
    ORDER BY agrup.anio, evento.nombre, evento.localidad, concurso.categoria, modalidad.modalidad, participa.clasificacion;




-- Lista de Empresa con Actividad, más las creaciones mediante crea.

CREATE OR REPLACE VIEW general_accc."Empresa_Creaciones" AS

    SELECT empresa.nombre, empresa.localidad, empresa.provincia, empresa.cif, empresa.domicilio, 
        empresa.telefono, empresa.contacto, actividades.actividades, crea_disenio.id_obra AS obra, 
        crea_disenio.tipo_disenio AS crea_disenio, crea_disenio."3_1_1_codigo_referencia",
        crea_disenio."3_1_2_titulo - Disenio", crea_disenio."3_1_2_titulo - Obra" 

    FROM general_accc.empresa AS empresa
        INNER JOIN (SELECT a_e.empresa, string_agg(actividad.actividad, ', ') AS actividades
                        FROM general_accc.actividad_empresa AS a_e
                        INNER JOIN general_accc.actividad
                            ON actividad.id = a_e.actividad
                           GROUP BY a_e.empresa) AS actividades
            ON empresa.entidad = actividades.empresa
        LEFT OUTER JOIN (SELECT disenio.tipo_disenio,
                                 disenio.obra_grafica AS id_obra, 
                                 crea.entidad AS entidad,
                                 doc."3_1_1_codigo_referencia",
                                 doc."3_1_2_titulo" AS "3_1_2_titulo - Disenio",
                                 docObra."3_1_2_titulo" AS "3_1_2_titulo - Obra"
                             FROM general_accc.documento_disenio AS d_d
                             INNER JOIN general_accc.disenio AS disenio
                                 ON disenio.id = d_d.disenio
                             INNER JOIN general_accc.documento AS doc
                                 ON d_d.documento = doc.id
                             INNER JOIN general_accc.crea AS crea
                                 ON disenio.id = crea.disenio
                             INNER JOIN general_accc.documento_obra AS d_o
                                 ON disenio.obra_grafica = d_o.obra
                             INNER JOIN general_accc.documento AS docObra
                                 ON d_o.documento = docObra.id) AS crea_disenio
            ON empresa.entidad = crea_disenio.entidad;



