-- TFG BBDD Carnaval de Cádiz - Paloma Cortés


-- Creación de tablas y estruturas para el data warehouse



-- Creación del espacio físico

CREATE TABLESPACE tb_dwh LOCATION '/var/lib/postgresql/tablespaces/tb_dwh';


set default_tablespace = 'tb_dwh';

set default_with_oids = false;



-- Creación de secuencias

CREATE SEQUENCE general_accc.seq_dwh_premios_concurso start 1;
CREATE SEQUENCE general_accc.seq_dwh_funciones_entidad start 1;
CREATE SEQUENCE general_accc.seq_dwh_temas_recurrentes start 1;
CREATE SEQUENCE general_accc.seq_dwh_soporte_documental start 1;



--
-- Nombre: dwh_premios_concurso; Tipo: TABLE; Schema: general_accc; Tablespace: tb_dwh
--
CREATE TABLE general_accc.dwh_premios_concurso (
    id integer NOT NULL DEFAULT nextval('general_accc.seq_dwh_premios_concurso'::regclass),
    id_concurso integer NOT NULL,
    concurso character varying(50) NOT NULL,
    localidad character varying(50) NOT NULL,
    categoria character varying(20) NOT NULL,
    anio integer NOT NULL,
    modalidad character varying(20) NOT NULL,
    premio integer NOT NULL,
    id_agrupacion integer NOT NULL,
    agrupacion character varying(100) NOT NULL,
    autor_letra character varying(200),
    autor_musica character varying(200),
    localidad_agrup character varying(50) NOT NULL,

    CONSTRAINT pk_dwh_premiosconcurso PRIMARY KEY (id) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT uq_dwh_premiosconcurso UNIQUE (anio, concurso, localidad, categoria, modalidad, premio) USING INDEX TABLESPACE tb_idx
);

-- Lista de premios por concurso, agrupación y modalidad.
-- <anio, concurso, localidad, modalidad, premio, agrupacion, autores, localidad>

CREATE OR REPLACE VIEW general_accc."DWH_PremiosConcurso" AS
    SELECT anio, dwh.concurso || ' (' || localidad || ')' AS concurso, categoria, modalidad,
           premio,  dwh.agrupacion || ' (' || localidad_agrup || ')' AS agrupacion, autor_letra,
           autor_musica
        FROM general_accc.dwh_premios_concurso AS dwh
        ORDER BY anio, concurso, categoria, modalidad, premio;
        




--
-- Nombre: dwh_entidad; Tipo: TABLE; Schema: general_accc; Tablespace: tb_dwh
--
CREATE TABLE general_accc.dwh_entidad (
    id integer NOT NULL,
    tipo_entidad general_accc.enum_entidad NOT NULL,
    nombre character varying(200) NOT NULL,
    localidad character varying(50),
    provincia character varying(50),

    CONSTRAINT pk_dwh_entidad PRIMARY KEY (id) USING INDEX TABLESPACE tb_idx
);


--
-- Nombre: dwh_funciones_entidad; Tipo: TABLE; Schema: general_accc; Tablespace: tb_dwh
--
CREATE TABLE general_accc.dwh_funciones_entidad (
    id integer NOT NULL DEFAULT nextval('general_accc.seq_dwh_funciones_entidad'::regclass),
    entidad integer NOT NULL,
    funcion character varying(100) NOT NULL,
    primero integer DEFAULT 0 NOT NULL,
    segundo integer DEFAULT 0 NOT NULL,
    tercero integer DEFAULT 0 NOT NULL,
    cuarto integer DEFAULT 0 NOT NULL,
    veces integer DEFAULT 0 NOT NULL,
    anio_inicio integer,
    anio_fin integer,

    CONSTRAINT pk_dwh_funcionesentidad PRIMARY KEY (id) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT uq_dwh_funcionesentidad UNIQUE (entidad, funcion) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT fk_dwh_funcionesentidad_entidad FOREIGN KEY (entidad)
      REFERENCES general_accc.dwh_entidad (id) MATCH SIMPLE
      ON UPDATE RESTRICT ON DELETE RESTRICT
);


-- Lista de dedicación oficial por entidad, sin datos de carácter personal.
-- <entidad, funcion, primeros, segundos, terceros, cuartos>

CREATE OR REPLACE VIEW general_accc."DWH_PremiosEntidad" AS
    SELECT dwh.localidad, dwh.nombre, dwh.tipo_entidad, f_e.funcion, 
           f_e.primero, f_e.segundo, f_e.tercero, f_e.cuarto
        FROM general_accc.dwh_funciones_entidad AS f_e
        INNER JOIN general_accc.dwh_entidad AS dwh
            ON f_e.entidad = dwh.id
        ORDER BY dwh.localidad, dwh.id, f_e.funcion;


-- Lista de premios por entidad considerado la función.
-- <entidad, funcion, veces, anioInicio, anioFin>

CREATE OR REPLACE VIEW general_accc."DWH_DedicacionEntidad" AS
SELECT dwh.localidad, dwh.nombre, dwh.tipo_entidad, f_e.funcion,
       f_e.veces, f_e.anio_inicio, f_e.anio_fin
	FROM general_accc.dwh_funciones_entidad AS f_e
    INNER JOIN general_accc.dwh_entidad AS dwh
        ON f_e.entidad = dwh.id
        ORDER BY dwh.localidad, dwh.id, f_e.funcion;


        


--
-- Nombre: dwh_temas_recurrentes; Tipo: TABLE; Schema: general_accc; Tablespace: tb_dwh
--
CREATE TABLE general_accc.dwh_temas_recurrentes (
    id integer NOT NULL DEFAULT nextval('general_accc.seq_dwh_temas_recurrentes'::regclass),
    id_tema integer NOT NULL,
    tema character varying NOT NULL,
    tipo_obra general_accc.enum_obra NOT NULL,
    anio integer NOT NULL,
    modalidad character varying NOT NULL,
    categoria general_accc.enum_categoria NOT NULL,
    total integer NOT NULL,

    CONSTRAINT pk_dwh_temasrecurrentes PRIMARY KEY (id) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT uq_dwh_temasrecurrentes UNIQUE (tema, tipo_obra, anio, modalidad, categoria) USING INDEX TABLESPACE tb_idx
);


-- Lista de temas recurrentes.
-- <tema, tipoObra, anio, modalidad, categoría, total>

CREATE OR REPLACE VIEW general_accc."DWH_TemasRecurrentes_AnioModalidadCategoria" AS
    SELECT tema, tipo_obra, anio, modalidad, categoria, total
        FROM general_accc.dwh_temas_recurrentes;


CREATE OR REPLACE VIEW general_accc."DWH_TemasRecurrentes_Anio" AS
    SELECT tema, tipo_obra, anio, SUM(dwh.total) AS total
        FROM general_accc.dwh_temas_recurrentes AS dwh
        GROUP BY tema, tipo_obra, anio;


CREATE OR REPLACE VIEW general_accc."DWH_TemasRecurrentes" AS

    SELECT tema, SUM(dwh.total) AS total
        FROM general_accc.dwh_temas_recurrentes AS dwh
        GROUP BY tema;





--
-- Nombre: dwh_soporte_documental; Tipo: TABLE; Schema: general_accc; Tablespace: tb_dwh
--
CREATE TABLE general_accc.dwh_soporte_documental (
    id integer NOT NULL DEFAULT nextval('general_accc.seq_dwh_soporte_documental'::regclass),
    id_agrupacion integer NOT NULL,
    agrupacion character varying NOT NULL,
    tipo_obra general_accc.enum_obra NOT NULL,
    tipo_documento general_accc.enum_tipodocumento NOT NULL,
    tipo_soporte general_accc.enum_tiposoporte NOT NULL,
    total integer NOT NULL,

    CONSTRAINT pk_dwh_soporte_documental PRIMARY KEY (id) USING INDEX TABLESPACE tb_idx,

    CONSTRAINT uq_dwh_soporte_documental UNIQUE (id_agrupacion, tipo_obra, tipo_documento, tipo_soporte) USING INDEX TABLESPACE tb_idx
);


-- Lista del estado de la documentación respecto a AgrupacionAnio y Obra.
-- <agrupacionAnio, tipoObra, tipoDocumento, tipoSoporte, total>

CREATE OR REPLACE VIEW general_accc."DWH_SoporteDocumental" AS
    SELECT agrup.anio, dwh.agrupacion, tipo_obra, tipo_documento, tipo_soporte, total
        FROM general_accc.dwh_soporte_documental AS dwh
        INNER JOIN general_accc.agrupacion_anio AS agrup
            ON id_agrupacion = agrup.id
        ORDER BY agrup.anio, dwh.agrupacion, tipo_obra, tipo_documento, tipo_soporte;





--
-- Nombre: dwh_error_log; Tipo: TABLE; Schema: general_accc; Tablespace: tb_dwh
--
CREATE TABLE general_accc.dwh_error_log (
    id serial NOT NULL,
    tabla character varying NOT NULL,
    disparador character varying NOT NULL,
    evento character varying NOT NULL,
    mensaje character varying NOT NULL,

    CONSTRAINT pk_dwh_soporte_documental_agrupaciones PRIMARY KEY (id) USING INDEX TABLESPACE tb_idx
);

