-- TFG BBDD Carnaval de Cádiz - Paloma Cortés


-- Creación de triggers para el data warehouse




--
-- Nombre: dwh_sp_agrupacionanio_up; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_agrupacionanio_in()
RETURNS trigger AS $$
DECLARE
    auxRecord record;
BEGIN

    -- Resulta más eficiente la llamada a estos cuando cambian los valores de
    -- interes, la asociación a obras.
    PERFORM general_accc.dwh_sp_temasrecurrentes_agrupacionanio_in (NEW.interpreta_literaria,
                                                                    NEW.utiliza_tipo,
                                                                    NEW.anio,
                                                                    NEW.modalidad,
                                                                    NEW.categoria);

    PERFORM general_accc.dwh_sp_soportedocumental_agrupacionanio_in (NEW.id,
                                                                     NEW.nombre,
                                                                     NEW.interpreta_literaria,
                                                                     NEW.interpreta_musical,
                                                                     NEW.utiliza_tipo);
    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



-- No es necesario en caso de DELETE en agrupacion_anio porque antes de eliminar
-- las agrupaciones deben habérsele retirado las obras.


--
-- Nombre: dwh_sp_agrupacionanio_up; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_agrupacionanio_up()
RETURNS trigger AS $$
BEGIN

    PERFORM general_accc.dwh_sp_premiosconcurso_agrupacionanio_up (NEW.id,
                                                                   OLD.nombre,
                                                                   NEW.nombre);

    -- Se ubica esta en segundo lugar porque la anterior aún lanzaría un segundo
    -- recorrido para el incremento de premios a las entidades, cosa que ya gestiona
    -- la siguiente.
    PERFORM general_accc.dwh_sp_funcionesentidad_agrupacionanio_up (NEW.id,
                                                                    NEW.anio,
                                                                    OLD.anio,
                                                                    NEW.interpreta_literaria,
                                                                    OLD.interpreta_literaria,
                                                                    NEW.interpreta_musical,
                                                                    OLD.interpreta_musical,
                                                                    NEW.utiliza_tipo,
                                                                    OLD.utiliza_tipo);

    PERFORM general_accc.dwh_sp_temasrecurrentes_agrupacionanio_up (NEW.interpreta_literaria,
                                                                    OLD.interpreta_literaria,
                                                                    NEW.utiliza_tipo,
                                                                    OLD.utiliza_tipo,
                                                                    NEW.anio,
                                                                    OLD.anio,
                                                                    NEW.modalidad,
                                                                    OLD.modalidad,
                                                                    NEW.categoria,
                                                                    OLD.categoria);

    PERFORM general_accc.dwh_sp_soportedocumental_agrupacionanio_up (NEW.id,
                                                                     OLD.id,
                                                                     NEW.nombre,
                                                                     OLD.nombre,
                                                                     NEW.interpreta_literaria,
                                                                     OLD.interpreta_literaria,
                                                                     NEW.interpreta_musical,
                                                                     OLD.interpreta_musical,
                                                                     NEW.utiliza_tipo,
                                                                     OLD.utiliza_tipo);
    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: dwh_sp_agrupacionanio_del; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_agrupacionanio_del()
RETURNS trigger AS $$
DECLARE
    auxRecord record;
BEGIN

    SELECT general_accc.dwh_sp_soportedocumental_agrupacionanio_del (OLD.id)
        INTO auxRecord;
    RETURN OLD;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: dwh_tr_agrupacionanio; Tipo: TRIGGER; Schema: general_accc
--
-- Utilizado para premiosconcurso y soportedocumental temasrecurrentes
--
CREATE TRIGGER dwh_tr_agrupacionanio_in
  after INSERT
  ON general_accc.agrupacion_anio
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_agrupacionanio_in();

CREATE TRIGGER dwh_tr_agrupacionanio_up
  after UPDATE
  ON general_accc.agrupacion_anio
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_agrupacionanio_up();

CREATE TRIGGER dwh_tr_agrupacionanio_del
  after DELETE
  ON general_accc.agrupacion_anio
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_agrupacionanio_del();




--
-- Nombre: dwh_sp_agrupacionanio_del; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_letra_del()
RETURNS trigger AS $$
BEGIN

    PERFORM general_accc.dwh_sp_funcionesentidad_letra_del (OLD.obra_literaria,
                                                            OLD.autoria_letra,
                                                            OLD.genero);

    PERFORM general_accc.dwh_sp_temasrecurrentes_letra_del (OLD.id,
                                                            OLD.obra_literaria);

    RETURN OLD;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_tr_temaletra; Tipo: TRIGGER; Schema: general_accc
--
-- Utilizado por temasrecurrentes y funcionesentidad.
-- Necesita ejecutarse antes de la eliminación del registro porque debe acceder
-- a los registros vinculados a la letra en la tabla tema_letra.
--
CREATE TRIGGER dwh_tr_letra_del
  BEFORE DELETE
  ON general_accc.letra
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_letra_del();





--
-- Nombre: dwh_sp_agrupacionanio_del; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_disenio_del()
RETURNS trigger AS $$
BEGIN

    PERFORM general_accc.dwh_sp_funcionesentidad_disenio_del (OLD.obra_grafica,
                                                              OLD.autoria_disenio,
                                                              OLD.tipo_disenio);
                                                              
    PERFORM general_accc.dwh_sp_temasrecurrentes_disenio_del (OLD.id,
                                                               OLD.obra_grafica);
    RETURN OLD;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_tr_temasrecurrentes_disenio; Tipo: TRIGGER; Schema: general_accc
--
-- Utilizado por temasrecurrentes y funcionesentidad.
-- Necesita ejecutarse antes de la eliminación del registro porque debe acceder
-- a los registros vinculados al diseño en la tabla tema_disenio.
--
CREATE TRIGGER dwh_tr_disenio_del
  BEFORE DELETE
  ON general_accc.disenio
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_disenio_del();






