-- TFG BBDD Carnaval de Cádiz - Paloma Cortés


-- Creación de funciones para el data warehouse




-------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------
--              Creación de procedimientos y disparadores para: dwh_entidad              --
-------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------

--
-- Nombre: dwh_sp_entidad_persona_in; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_entidad_persona_in()
RETURNS trigger AS $$
DECLARE
    auxCharacter character varying;
BEGIN

    auxCharacter := NEW.nombre || ' ' || NEW.apellido1 || ' ' || COALESCE(NEW.apellido2, '');

    IF NEW.alias IS NOT NULL THEN
        auxCharacter := auxCharacter || ' (' || NEW.alias || ')';
    END IF;

    INSERT INTO general_accc.dwh_entidad (id, tipo_entidad, nombre, localidad, provincia)
        VALUES (NEW.entidad, 'Persona', auxCharacter, NEW.localidad, NEW.provincia);
    RETURN NEW;


EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_entidad_persona_in; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_entidad_persona_up()
RETURNS trigger AS $$
DECLARE
    auxCharacter character varying;
BEGIN

    IF OLD.nombre <> NEW.nombre 
            OR OLD.apellido1 <> NEW.apellido1
            OR COALESCE(OLD.apellido2,'') <> COALESCE(NEW.apellido2,'')
            OR COALESCE(OLD.alias,'') <> COALESCE(NEW.alias,'') THEN
            -- El apellido2 y el alias pueden ser nulos
            
        auxCharacter := NEW.nombre || ' ' || NEW.apellido1 || ' ' || COALESCE(NEW.apellido2,'');
        
        IF NEW.alias IS NOT NULL THEN
            auxCharacter := auxCharacter || ' (' || NEW.alias || ')';
        END IF;
        
    END IF;

    IF auxCharacter IS NOT NULL
            OR COALESCE(OLD.localidad,'') <> COALESCE(NEW.localidad,'')
            OR COALESCE(OLD.provincia,'') <> COALESCE(NEW.provincia,'') THEN
        -- Para las personas la localidad y la provincia admiten nulos

        UPDATE general_accc.dwh_entidad SET
                                        nombre = COALESCE(auxCharacter,
                                                          (SELECT nombre
                                                               FROM general_accc.dwh_entidad
                                                               WHERE id = NEW.entidad)),
                                        localidad = NEW.localidad,
                                        provincia = NEW.provincia
            WHERE id = NEW.entidad;
    END IF;

    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_entidad_agrupacion; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_entidad_agrupacion_in()
RETURNS trigger AS $$
BEGIN

    INSERT INTO general_accc.dwh_entidad (id, tipo_entidad, nombre, localidad, provincia)
        VALUES (NEW.entidad, 'Agrupacion', COALESCE(NEW.alias, '(agrupación)'), NEW.localidad, NEW.provincia);
    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_entidad_agrupacion; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_entidad_agrupacion_up()
RETURNS trigger AS $$
BEGIN

    IF COALESCE(OLD.alias,'') <> COALESCE(NEW.alias,'')
            OR OLD.localidad <> NEW.localidad
            OR OLD.provincia <> NEW.provincia THEN

        UPDATE general_accc.dwh_entidad SET nombre = COALESCE(NEW.alias, '(agrupación)'),
                                            localidad = NEW.localidad,
                                            provincia = NEW.provincia
            WHERE id = NEW.entidad;
    END IF;
    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_entidad_empresa; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_entidad_empresa_in()
RETURNS trigger AS $$
BEGIN

    INSERT INTO general_accc.dwh_entidad (id, tipo_entidad, nombre, localidad, provincia)
        VALUES (NEW.entidad, 'Empresa', NEW.nombre, NEW.localidad, NEW.provincia);
    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_entidad_empresa; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_entidad_empresa_up()
RETURNS trigger AS $$
BEGIN

    IF OLD.nombre <> NEW.nombre
            OR OLD.localidad <> NEW.localidad
            OR OLD.provincia <> NEW.provincia THEN

        UPDATE general_accc.dwh_entidad SET nombre = NEW.nombre,
                                            localidad = NEW.localidad,
                                            provincia = NEW.provincia
            WHERE id = NEW.entidad;

    END IF;

    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_entidad_persona_del; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_entidad_entidad_del()
RETURNS trigger AS $$
BEGIN

    DELETE FROM general_accc.dwh_entidad
        WHERE id = OLD.id;
    RETURN OLD;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;









-------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------
--        Creación de procedimientos y disparadores para: dwh_funciones_entidad          --
-------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------


--
-- Nombre: __dwh_alta_funcionesentidad; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.__dwh_alta_funcionesentidad (in_entidad integer,
                                                                     in_funcion character varying,
                                                                     in_anio integer)
RETURNS void AS $$
DECLARE
    auxInteger integer;
    auxAnio integer;
BEGIN

    SELECT veces, anio_fin INTO auxInteger, auxAnio
        FROM general_accc.dwh_funciones_entidad
        WHERE entidad = in_entidad
            AND funcion = in_funcion;
    IF FOUND THEN

        IF auxAnio > in_anio THEN
            in_anio := auxAnio;
            -- De esta forma no se reduce la trayectoria de la entidad
        END IF;
    
        UPDATE general_accc.dwh_funciones_entidad SET
                                                  veces = auxInteger + 1,
                                                  anio_fin = in_anio
                WHERE entidad = in_entidad
                    AND funcion = in_funcion;
    ELSE
        -- Verificar que la entidad existe en la tabla dwh_entidad. Si la 
        --BBDD existía antes que esta funcionalidad, quizás no se haya replicado 
        INSERT INTO general_accc.dwh_funciones_entidad (entidad, funcion, veces, anio_inicio, anio_fin)
            VALUES (in_entidad, in_funcion, 1, in_anio, in_anio);

    END IF;

    RETURN;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES ('dwh_funciones_entidad', '__dwh_alta_funcionesentidad', 'function', SQLERRM);
    RETURN;
END;
$$ LANGUAGE plpgsql;


--
-- Nombre: __dwh_baja_funcionesentidad; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.__dwh_baja_funcionesentidad (in_entidad integer,
                                                                     in_funcion character varying)
RETURNS void AS $$
DECLARE
    auxInteger integer;
BEGIN

    SELECT veces INTO auxInteger
        FROM general_accc.dwh_funciones_entidad
        WHERE entidad = in_entidad
            AND funcion = in_funcion;
    IF FOUND AND auxInteger > 1 THEN
        UPDATE general_accc.dwh_funciones_entidad SET
                                                  veces = auxInteger - 1
                WHERE entidad = in_entidad
                    AND funcion = in_funcion;
    ELSE
        DELETE FROM general_accc.dwh_funciones_entidad
            WHERE entidad = in_entidad
                AND funcion = in_funcion;
    END IF;

    RETURN;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES ('dwh_funciones_entidad', '__dwh_baja_funcionesentidad', 'function', SQLERRM);
    RETURN;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_funcionesentidad_agrupacionanio_up; Tipo: FUNCTION; Schema: general_accc
--
-- Esta solo es necesaria para gestionar los premios de las funciones de tipo
-- autoria y los años de ser diferentes.
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_funcionesentidad_agrupacionanio_up (new_id integer,
                                                                                   new_anio integer,
                                                                                   old_anio integer,
                                                                                   new_interpreta_literaria integer,
                                                                                   old_interpreta_literaria integer,
                                                                                   new_interpreta_musical integer,
                                                                                   old_interpreta_musical integer,
                                                                                   new_utiliza_tipo integer,
                                                                                   old_utiliza_tipo integer)
RETURNS void AS $$
DECLARE
    auxInteger integer;
    auxAnioInicio integer;
    auxAnioFin integer;
    auxRecord record;
BEGIN

    -- Gestión de autorías literarias
    
    IF COALESCE(new_interpreta_literaria, 0) <> COALESCE(old_interpreta_literaria, 0) THEN

        IF old_interpreta_literaria IS NOT NULL THEN    
            -- Recorrer las entidades con autoría en la obra retirada
            FOR auxRecord IN (SELECT autoria_letra, genero
                                  FROM general_accc.letra                                 
                                  WHERE obra_literaria = old_interpreta_literaria)
            LOOP
                PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (-1,
                                                                             new_id,
                                                                             auxRecord.autoria_letra,
                                                                             'Letra ' || auxRecord.genero::character varying);
            END LOOP;
        END IF;
        
        IF new_interpreta_literaria IS NOT NULL THEN    
            -- Recorrer las entidades con autoría en la obra asignada
            FOR auxRecord IN (SELECT autoria_letra, genero
                                  FROM general_accc.letra                                 
                                  WHERE obra_literaria = new_interpreta_literaria)
            LOOP
                PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (1,
                                                                             new_id,
                                                                             auxRecord.autoria_letra,
                                                                             'Letra ' || auxRecord.genero::character varying);
                SELECT anio_inicio, anio_fin
                        INTO auxAnioInicio, auxAnioFin
                    FROM general_accc.dwh_funciones_entidad
                    WHERE entidad = new_id
                        AND funcion = 'Letra ' || auxRecord.genero::character varying;

                IF COALESCE(auxAnioInicio, 10000) > new_anio THEN
                    UPDATE general_accc.dwh_funciones_entidad SET
                                                              anio_inicio = new_anio
                        WHERE entidad = new_id
                            AND funcion = 'Letra ' || auxRecord.genero::character varying;
                END IF;

                IF COALESCE(auxAnioFin, 0) < new_anio THEN
                    UPDATE general_accc.dwh_funciones_entidad SET
                                                              anio_fin = new_anio
                        WHERE entidad = new_id
                            AND funcion = 'Letra ' || auxRecord.genero::character varying;
                END IF;

            END LOOP;
        END IF;
    END IF;


    -- Gestión de autorias musicales
    IF COALESCE(new_interpreta_musical, 0) <> COALESCE(old_interpreta_musical, 0) THEN

        IF old_interpreta_musical IS NOT NULL THEN    
            -- Recorrer las entidades con autoría en la obra retirada
            FOR auxRecord IN (SELECT autoria_musica, genero
                                  FROM general_accc.musica                                 
                                  WHERE obra_musical = old_interpreta_musical)
            LOOP
                PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (-1,
                                                                             new_id,
                                                                             auxRecord.autoria_musica,
                                                                             'Letra ' || auxRecord.genero::character varying);
            END LOOP;
        END IF;
        
        IF new_interpreta_musical IS NOT NULL THEN    
            -- Recorrer las entidades con autoría en la obra asignada
            FOR auxRecord IN (SELECT autoria_musica, genero
                                  FROM general_accc.musica                                 
                                  WHERE obra_musical = new_interpreta_musical)
            LOOP
                PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (1,
                                                                             new_id,
                                                                             auxRecord.autoria_musica,
                                                                             'Música ' || auxRecord.genero::character varying);
                SELECT anio_inicio, anio_fin
                        INTO auxAnioInicio, auxAnioFin
                    FROM general_accc.dwh_funciones_entidad
                    WHERE entidad = new_id
                        AND funcion = 'Música ' || auxRecord.genero::character varying;

                IF COALESCE(auxAnioInicio, 10000) > new_anio THEN
                    UPDATE general_accc.dwh_funciones_entidad SET
                                                              anio_inicio = new_anio
                        WHERE entidad = new_id
                            AND funcion = 'Música ' || auxRecord.genero::character varying;
                END IF;

                IF COALESCE(auxAnioFin, 0) < new_anio THEN
                    UPDATE general_accc.dwh_funciones_entidad SET
                                                              anio_fin = new_anio
                        WHERE entidad = new_id
                            AND funcion = 'Música ' || auxRecord.genero::character varying;
                END IF;

            END LOOP;
        END IF;
    END IF;

    -- Gestión de autorías gráficas
        IF COALESCE(new_utiliza_tipo, 0) <> COALESCE(old_utiliza_tipo, 0) THEN

        IF old_utiliza_tipo IS NOT NULL THEN    
            -- Recorrer las entidades con autoría en la obra retirada
            FOR auxRecord IN (SELECT autoria_disenio, tipo_disenio
                                  FROM general_accc.disenio                                 
                                  WHERE obra_grafica = old_utiliza_tipo)
            LOOP
                PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (-1,
                                                                             new_id,
                                                                             auxRecord.autoria_disenio,
                                                                             'Diseño ' || auxRecord.tipo_disenio::character varying);
            END LOOP;
        END IF;
        IF new_utiliza_tipo IS NOT NULL THEN    
            -- Recorrer las entidades con autoría en la obra retirada
            FOR auxRecord IN (SELECT autoria_disenio, tipo_disenio
                                  FROM general_accc.disenio
                                  WHERE obra_grafica = new_utiliza_tipo)
            LOOP
                PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (1,
                                                                             new_id,
                                                                             auxRecord.autoria_disenio,
                                                                             'Diseño ' || auxRecord.tipo_disenio::character varying);
                SELECT anio_inicio, anio_fin
                        INTO auxAnioInicio, auxAnioFin
                    FROM general_accc.dwh_funciones_entidad
                    WHERE entidad = new_id
                        AND funcion = 'Diseño ' || auxRecord.genero::character varying;
                        
                IF COALESCE(auxAnioInicio, 10000) > new_anio THEN
                    UPDATE general_accc.dwh_funciones_entidad SET
                                                              anio_inicio = new_anio
                        WHERE entidad = new_id
                            AND funcion = 'Diseño ' || auxRecord.genero::character varying;
                END IF;

                IF COALESCE(auxAnioFin, 0) < new_anio THEN
                    UPDATE general_accc.dwh_funciones_entidad SET
                                                              anio_fin = new_anio
                        WHERE entidad = new_id
                            AND funcion = 'Diseño ' || auxRecord.genero::character varying;
                END IF;

            END LOOP;
        END IF;
    END IF;
    
    RETURN;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES ('agrupacion_anio', 'dwh_sp_soportedocumental_agrupacionanio_up', 'UPDATE', SQLERRM);
    RETURN;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: __dwh_cambiapremiados_funcionesentidad; Tipo: FUNCTION; Schema: general_accc
--
-- Control del caso en que se relacionan entidades a agrupaciones ya premiadas.
--
CREATE OR REPLACE FUNCTION general_accc.__dwh_cambiapremiados_funcionesentidad (in_valor integer,
                                                                                in_agrupacion integer,
                                                                                in_entidad integer,
                                                                                in_funcion character varying)
RETURNS void AS $$
DECLARE
    auxInteger integer;
    auxRecord record;
BEGIN

    IF in_agrupacion IS NULL THEN
        -- En el caso de las autorias y creaciones de obras quizás no haya
        -- agrupación vinculada.
        -- Se establecen los premios al editar los campos de interpretación
        -- de las agrupaciones.
        RETURN;
    END IF;

    FOR auxRecord IN (SELECT clasificacion
                          FROM general_accc.participa
                          WHERE agrupacion_anio = in_agrupacion
                              AND clasificacion < 4)
    LOOP
        IF auxRecord.clasificacion = 1 THEN
            UPDATE general_accc.dwh_funciones_entidad SET
                                                      primero = in_valor + (SELECT primero
                                                                                FROM general_accc.dwh_funciones_entidad
                                                                                WHERE entidad = in_entidad
                                                                                    AND funcion = in_funcion)
                WHERE entidad = in_entidad
                    AND funcion = in_funcion;

        ELSEIF auxRecord.clasificacion = 2 THEN
            UPDATE general_accc.dwh_funciones_entidad SET
                                                      segundo = in_valor + (SELECT segundo
                                                                                FROM general_accc.dwh_funciones_entidad
                                                                                WHERE entidad = in_entidad
                                                                                    AND funcion = in_funcion)
                WHERE entidad = in_entidad
                    AND funcion = in_funcion;

        ELSEIF auxRecord.clasificacion = 3 THEN
            UPDATE general_accc.dwh_funciones_entidad SET
                                                      tercero = in_valor + (SELECT tercero
                                                                                FROM general_accc.dwh_funciones_entidad
                                                                                WHERE entidad = in_entidad
                                                                                    AND funcion = in_funcion)
                WHERE entidad = in_entidad
                    AND funcion = in_funcion;

        ELSEIF auxRecord.clasificacion = 4 THEN
            UPDATE general_accc.dwh_funciones_entidad SET
                                                      cuarto = in_valor + (SELECT cuarto
                                                                               FROM general_accc.dwh_funciones_entidad
                                                                               WHERE entidad = in_entidad
                                                                                   AND funcion = in_funcion)
                WHERE entidad = in_entidad
                    AND funcion = in_funcion;

        END IF;
    END LOOP;
    RETURN;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES ('dwh_funciones_entidad', '__dwh_cambiapremiados_funcionesentidad', 'function', SQLERRM);
    RETURN;
END;
$$ LANGUAGE plpgsql;





                                                                    
--
-- Nombre: dwh_sp_funcionesentidad_cargo_in; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_funcionesentidad_cargo_in()
RETURNS trigger AS $$
DECLARE
    auxAgrupacion integer;
    auxEntidad integer;
    auxAnio integer;
BEGIN

    SELECT agrupacion_anio, persona, anio
            INTO auxAgrupacion, auxEntidad, auxAnio
        FROM general_accc.rol AS rol
            INNER JOIN general_accc.agrupacion_anio AS agrup
                ON agrup.id = rol.agrupacion_anio
        WHERE rol.id = NEW.rol;

    IF FOUND THEN
        PERFORM general_accc.__dwh_alta_funcionesentidad (auxEntidad, NEW.tipo_cargo::character varying, auxAnio);

        PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (1,
                                                                     auxAgrupacion,
                                                                     auxEntidad,
                                                                     NEW.tipo_cargo::character varying);
    END IF;

    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_funcionesentidad_cargo_up; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_funcionesentidad_cargo_up()
RETURNS trigger AS $$
DECLARE
    auxEntidad integer;
    auxAgrupacion integer;
    auxAnio integer;
BEGIN

    IF OLD.tipo_cargo <> NEW.tipo_cargo THEN
        SELECT persona, agrupacion_anio, anio
                INTO auxEntidad, auxAgrupacion, auxAnio
            FROM general_accc.rol AS rol
                INNER JOIN general_accc.agrupacion_anio AS agrup
                    ON agrup.id = rol.agrupacion_anio
            WHERE rol.id = NEW.rol;

        IF FOUND THEN

            PERFORM general_accc.__dwh_baja_funcionesentidad (auxEntidad, OLD.tipo_cargo::character varying);
            
            PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (-1,
                                                                         auxAgrupacion,
                                                                         auxEntidad,
                                                                         OLD.tipo_cargo::character varying);


            PERFORM general_accc.__dwh_alta_funcionesentidad (auxEntidad, NEW.tipo_cargo::character varying, auxAnio);
            
            PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (1,
                                                                         auxAgrupacion,
                                                                         auxEntidad,
                                                                         NEW.tipo_cargo::character varying);
        END IF;
    END IF;

    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_funcionesentidad_cargo_del; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_funcionesentidad_cargo_del()
RETURNS trigger AS $$
DECLARE
    auxEntidad integer;
    auxAgrupacion integer;
BEGIN

    SELECT persona, agrupacion_anio
            INTO auxEntidad, auxAgrupacion
        FROM general_accc.rol AS rol
            INNER JOIN general_accc.agrupacion_anio AS agrup
                ON agrup.id = rol.agrupacion_anio
        WHERE rol.id = OLD.rol;

    IF FOUND THEN
        PERFORM general_accc.__dwh_baja_funcionesentidad (auxEntidad, OLD.tipo_cargo::character varying);
        
        PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (-1,
                                                                     auxAgrupacion,
                                                                     auxEntidad,
                                                                     OLD.tipo_cargo::character varying);
    END IF;

    RETURN OLD;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: dwh_sp_funcionesentidad_orquesta_in; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_funcionesentidad_orquesta_in()
RETURNS trigger AS $$
DECLARE
    auxAgrupacion integer;
    auxEntidad integer;
    auxAnio integer;
BEGIN

    SELECT agrupacion_anio, persona, anio
            INTO auxAgrupacion, auxEntidad, auxAnio
        FROM general_accc.rol AS rol
            INNER JOIN general_accc.agrupacion_anio AS agrup
                ON agrup.id = rol.agrupacion_anio
        WHERE rol.id = NEW.rol;

    IF FOUND THEN
        PERFORM general_accc.__dwh_alta_funcionesentidad (auxEntidad,
                                                          (SELECT instrumento
                                                               FROM general_accc.instrumento
                                                               WHERE id = NEW.instrumento),
                                                          auxAnio);
                                                          
        PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (1, auxAgrupacion, auxEntidad,
                                                                     (SELECT instrumento
                                                                          FROM general_accc.instrumento
                                                                          WHERE id = NEW.instrumento));
    END IF;

    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_funcionesentidad_orquesta_up; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_funcionesentidad_orquesta_up()
RETURNS trigger AS $$
DECLARE
    auxEntidad integer;
    auxAgrupacion integer;
    auxAnio integer;
BEGIN

    IF OLD.instrumento <> NEW.instrumento THEN  
        SELECT persona, agrupacion_anio, anio
                INTO auxEntidad, auxAgrupacion, auxAnio
            FROM general_accc.rol AS rol
            INNER JOIN general_accc.agrupacion_anio AS agrup
                ON agrup.id = rol.agrupacion_anio
            WHERE rol.id = NEW.rol;
            -- Se utiliza NEW porque este identificador no admite ser modificado
            -- por el diseño físico

        IF FOUND THEN
            PERFORM general_accc.__dwh_baja_funcionesentidad (auxEntidad,
                                                              (SELECT instrumento
                                                                   FROM general_accc.instrumento
                                                                   WHERE id = OLD.instrumento));
            PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (-1, 
                                                                         auxAgrupacion,
                                                                         auxEntidad,
                                                                         (SELECT instrumento
                                                                              FROM general_accc.instrumento
                                                                              WHERE id = OLD.instrumento));
            PERFORM general_accc.__dwh_alta_funcionesentidad (auxEntidad,
                                                              (SELECT instrumento
                                                                    FROM general_accc.instrumento
                                                                    WHERE id = NEW.instrumento),
                                                              auxAnio);
            PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (1, 
                                                                         auxAgrupacion,
                                                                         auxEntidad,
                                                                         (SELECT instrumento
                                                                              FROM general_accc.instrumento
                                                                       WHERE id = NEW.instrumento));
        END IF;
    END IF;

    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_funcionesentidad_orquesta_del; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_funcionesentidad_orquesta_del()
RETURNS trigger AS $$
DECLARE
    auxEntidad integer;
    auxAgrupacion integer;
    auxAnio integer;
BEGIN

    SELECT persona, anio, agrup.id
            INTO auxEntidad, auxAnio, auxAgrupacion
        FROM general_accc.rol AS rol
        INNER JOIN general_accc.agrupacion_anio AS agrup
            ON agrup.id = rol.agrupacion_anio
        WHERE rol.id = OLD.rol;

    IF FOUND THEN
        PERFORM general_accc.__dwh_baja_funcionesentidad (auxEntidad,
                                                          (SELECT instrumento
                                                               FROM general_accc.instrumento
                                                               WHERE id = OLD.instrumento));
        PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (-1,
                                                                     auxAgrupacion,
                                                                     auxEntidad,
                                                                     (SELECT instrumento
                                                                          FROM general_accc.instrumento
                                                                          WHERE id = OLD.instrumento));
    END IF;

    RETURN OLD;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_funcionesentidad_voz_in; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_funcionesentidad_voz_in()
RETURNS trigger AS $$
DECLARE
    auxAgrupacion integer;
    auxEntidad integer;
    auxAnio integer;
BEGIN

    SELECT agrupacion_anio, persona, anio
            INTO auxAgrupacion, auxEntidad, auxAnio
        FROM general_accc.rol AS rol
            INNER JOIN general_accc.agrupacion_anio AS agrup
                ON agrup.id = rol.agrupacion_anio
        WHERE rol.id = NEW.rol;

    IF FOUND THEN
        PERFORM general_accc.__dwh_alta_funcionesentidad (auxEntidad, NEW.tipo_voz::character varying, auxAnio);
        
        PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (1,
                                                                     auxAgrupacion,
                                                                     auxEntidad,
                                                                     NEW.tipo_voz::character varying);
    END IF;

    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_funcionesentidad_voz_up; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_funcionesentidad_voz_up()
RETURNS trigger AS $$
DECLARE
    auxEntidad integer;
    auxAgrupacion integer;
    auxAnio integer;
BEGIN

    SELECT persona, agrupacion_anio, anio
            INTO auxEntidad, auxAgrupacion, auxAnio
        FROM general_accc.rol AS rol
            INNER JOIN general_accc.agrupacion_anio AS agrup
                ON agrup.id = rol.agrupacion_anio
        WHERE rol.id = NEW.rol;

    IF FOUND AND OLD.tipo_voz <> NEW.tipo_voz THEN
        PERFORM general_accc.__dwh_baja_funcionesentidad (auxEntidad, OLD.tipo_voz::character varying);
        PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (-1,
                                                                     auxAgrupacion,
                                                                     auxEntidad,
                                                                     OLD.tipo_voz::character varying);

        PERFORM general_accc.__dwh_alta_funcionesentidad (auxEntidad, NEW.tipo_voz::character varying, auxAnio);
        PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (1,
                                                                     auxAgrupacion,
                                                                     auxEntidad,
                                                                     NEW.tipo_voz::character varying);
    END IF;

    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_funcionesentidad_voz_del; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_funcionesentidad_voz_del()
RETURNS trigger AS $$
DECLARE
    auxEntidad integer;
    auxAgrupacion integer;
    auxAnio integer;
BEGIN

    SELECT agrup.id, persona INTO auxAgrupacion, auxEntidad
        FROM general_accc.rol AS rol
            INNER JOIN general_accc.agrupacion_anio AS agrup
                ON agrup.id = rol.agrupacion_anio
        WHERE rol.id = OLD.rol;

    IF FOUND THEN
        PERFORM general_accc.__dwh_baja_funcionesentidad (auxEntidad, OLD.tipo_voz::character varying);
        PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (-1, auxAgrupacion, auxEntidad, OLD.tipo_voz::character varying);
    END IF;

    RETURN OLD;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_funcionesentidad_letra_in; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_funcionesentidad_letra_in()
RETURNS trigger AS $$
DECLARE
    auxEntidad integer;
    auxAnio integer;
    auxAgrupacion integer;
BEGIN

    IF NEW.autoria_letra IS NOT NULL THEN

        SELECT anio, id
                INTO auxAnio, auxAgrupacion
            FROM general_accc.agrupacion_anio
            WHERE interpreta_literaria = NEW.obra_literaria;

        -- Quizás se den letras nunca interpretadas por agrupaciones
        PERFORM general_accc.__dwh_alta_funcionesentidad (NEW.autoria_letra,
                                                          'Letra ' || NEW.genero::character varying,
                                                          auxAnio);
                                                          
        IF auxAgrupacion IS NOT NULL THEN
        -- Si no hay agrupación interprete no hay premio
            PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (1,
                                                                         auxAgrupacion,
                                                                         NEW.autoria_letra,
                                                                         'Letra ' || NEW.genero::character varying);
        END IF;
    END IF;

    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_funcionesentidad_letra_up; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_funcionesentidad_letra_up()
RETURNS trigger AS $$
DECLARE
    auxEntidad integer;
    auxAnio integer;
    auxAgrupacion integer;
BEGIN

    IF OLD.autoria_letra <> NEW.autoria_letra THEN
    
        PERFORM general_accc.__dwh_baja_funcionesentidad (OLD.autoria_letra,
                                                          'Letra ' || OLD.genero::character varying);

        SELECT anio, id
                INTO auxAnio, auxAgrupacion
            FROM general_accc.agrupacion_anio
            WHERE interpreta_literaria = NEW.obra_literaria;

        IF auxAgrupacion IS NOT NULL THEN
        -- Si no hay agrupación interprete no hay premio
            PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (-1,
                                                                         auxAgrupacion,
                                                                         OLD.autoria_letra,
                                                                         'Letra ' || OLD.genero::character varying);
        END IF;
        
        PERFORM general_accc.__dwh_alta_funcionesentidad (NEW.autoria_letra,
                                                          'Letra ' || NEW.genero::character varying,
                                                          auxAnio);
        IF auxAgrupacion IS NOT NULL THEN
        -- Si no hay agrupación interprete no hay premio
            PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (1,
                                                                         auxAgrupacion,
                                                                         NEW.autoria_letra,
                                                                         'Letra ' || NEW.genero::character varying);
        END IF;
    END IF;
    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_funcionesentidad_letra_del; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_funcionesentidad_letra_del(old_obra_literaria integer,
                                                                          old_autoria_letra integer,
                                                                          old_genero general_accc.enum_genero_letra)
RETURNS void AS $$
DECLARE
    auxEntidad integer;
BEGIN

    PERFORM general_accc.__dwh_baja_funcionesentidad (old_autoria_letra,
                                                      'Letra ' || old_genero::character varying);

    PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (-1,
                                                                 (SELECT id
                                                                      FROM general_accc.agrupacion_anio
                                                                      WHERE interpreta_literaria = old_obra_literaria),
                                                                 old_autoria_letra,
                                                                 'Letra ' || old_genero::character varying);
    RETURN;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES ('letra', 'dwh_tr_letra_del', 'DELETE', SQLERRM);
    RETURN;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: dwh_sp_funcionesentidad_musica_in; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_funcionesentidad_musica_in()
RETURNS trigger AS $$
DECLARE
    auxEntidad integer;
    auxAnio integer;
    auxAgrupacion integer;
BEGIN

    IF NEW.autoria_musica IS NOT NULL THEN
        SELECT anio, id
                INTO auxAnio, auxAgrupacion
            FROM general_accc.agrupacion_anio
            WHERE interpreta_musical = NEW.obra_musical;

        -- Quizás se den composiciones musicales nunca interpretadas por agrupaciones
        PERFORM general_accc.__dwh_alta_funcionesentidad (NEW.autoria_musica,
                                                          'Música ' || NEW.genero::character varying,
                                                          auxAnio);
                                                               
        IF auxAgrupacion IS NOT NULL THEN
        -- Si no hay agrupación interprete no hay premio
            PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (1,
                                                                         auxAgrupacion,
                                                                         NEW.autoria_musica,
                                                                         'Música ' || NEW.genero::character varying);
        END IF;
    END IF;

    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_funcionesentidad_musica_up; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_funcionesentidad_musica_up()
RETURNS trigger AS $$
DECLARE
    auxEntidad integer;
    auxAnio integer;
    auxAgrupacion integer;
BEGIN

    IF OLD.autoria_musica <> NEW.autoria_musica THEN
        PERFORM general_accc.__dwh_baja_funcionesentidad (OLD.autoria_musica,
                                                          'Música ' || OLD.genero::character varying);

        SELECT anio, id
                INTO auxAnio, auxAgrupacion
            FROM general_accc.agrupacion_anio
            WHERE interpreta_musical = NEW.obra_musical;

        IF auxAgrupacion IS NOT NULL THEN
        -- Si no hay agrupación interprete no hay premio
            PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (-1,
                                                                         auxAgrupacion,
                                                                         OLD.autoria_musica,
                                                                         'Música ' || OLD.genero::character varying);
        END IF;
        
        PERFORM general_accc.__dwh_alta_funcionesentidad (NEW.autoria_musica,
                                                          'Música ' || NEW.genero::character varying,
                                                          auxAnio);
                                                          
        IF auxAgrupacion IS NOT NULL THEN
        -- Si no hay agrupación interprete no hay premio
            PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (1,
                                                                         auxAgrupacion,
                                                                         NEW.autoria_musica,
                                                                         'Música ' || NEW.genero::character varying);
        END IF;
    END IF;

    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_funcionesentidad_musica_del; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_funcionesentidad_musica_del()
RETURNS trigger AS $$
DECLARE
    auxEntidad integer;
BEGIN

    PERFORM general_accc.__dwh_baja_funcionesentidad (OLD.autoria_musica,
                                                      'Música ' || OLD.genero::character varying);
                                                      
    PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (-1,
                                                                 (SELECT id
                                                                      FROM general_accc.agrupacion_anio
                                                                      WHERE interpreta_musical = OLD.obra_musical),
                                                                 OLD.autoria_musica,
                                                                 'Música ' || OLD.genero::character varying);
    RETURN OLD;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_funcionesentidad_disenio_in; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_funcionesentidad_disenio_in()
RETURNS trigger AS $$
DECLARE
    auxEntidad integer;
    auxAnio integer;
    auxAgrupacion integer;
BEGIN

    IF NEW.autoria_disenio IS NOT NULL THEN
    
        SELECT anio, id
                INTO auxAnio, auxAgrupacion
            FROM general_accc.agrupacion_anio
            WHERE utiliza_tipo = NEW.obra_grafica;

        PERFORM general_accc.__dwh_alta_funcionesentidad (NEW.autoria_disenio,
                                                          'Diseño ' || NEW.tipo_disenio::character varying,
                                                          auxAnio);
                                                          
        -- Verificar el estado de premios de las agrupaciones solo es necesario en las altas
        -- de autor, de cambiar los premios ya estará asociada la entidad mediante el componente
        IF auxAgrupacion IS NOT NULL THEN
        -- Si no hay agrupación interprete no hay premio
            PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (1,
                                                                         auxAgrupacion,
                                                                         NEW.autoria_disenio,
                                                                         'Diseño ' || NEW.tipo_disenio::character varying);
        END IF;
    END IF;

    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_funcionesentidad_disenio_up; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_funcionesentidad_disenio_up()
RETURNS trigger AS $$
DECLARE
    auxEntidad integer;
    auxAnio integer;
    auxAgrupacion integer;
BEGIN

    IF OLD.autoria_disenio <> NEW.autoria_disenio THEN
    
        PERFORM general_accc.__dwh_baja_funcionesentidad (OLD.autoria_disenio,
                                                          'Diseño ' || OLD.tipo_disenio::character varying);

        SELECT anio, id
                INTO auxAnio, auxAgrupacion
            FROM general_accc.agrupacion_anio
            WHERE utiliza_tipo = NEW.obra_grafica;
                                                          
        -- Verificar el estado de premios de las agrupaciones solo es necesario
        -- en las altas de autor, de cambiar los premios ya estará asociada la
        -- entidad mediante el componente
        IF auxAgrupacion IS NOT NULL THEN
        -- Si no hay agrupación interprete no hay premio
            PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (-1,
                                                                         auxAgrupacion,
                                                                         OLD.autoria_disenio,
                                                                         'Diseño ' || OLD.tipo_disenio::character varying);
        END IF;
        
        PERFORM general_accc.__dwh_alta_funcionesentidad (NEW.autoria_disenio,
                                                          'Diseño ' || NEW.tipo_disenio::character varying,
                                                          (SELECT anio
                                                               FROM general_accc.agrupacion_anio
                                                               WHERE utiliza_tipo = NEW.obra_grafica));
        IF auxAgrupacion IS NOT NULL THEN
        -- Si no hay agrupación interprete no hay premio
            PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (1,
                                                                         (SELECT id
                                                                              FROM general_accc.agrupacion_anio
                                                                              WHERE utiliza_tipo = NEW.obra_grafica),
                                                                         NEW.autoria_disenio,
                                                                         'Diseño ' || NEW.tipo_disenio::character varying);
        END IF;
    END IF;

    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_funcionesentidad_disenio_del; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_funcionesentidad_disenio_del(old_obra_grafica integer,
                                                                            old_autoria_disenio integer,
                                                                            old_genero general_accc.enum_disenio)
RETURNS void AS $$
DECLARE
    auxEntidad integer;
BEGIN

    PERFORM general_accc.__dwh_baja_funcionesentidad (old_autoria_disenio,
                                                      'Diseño ' || old_tipo_disenio::character varying);
    PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (-1,
                                                                 (SELECT id
                                                                      FROM general_accc.agrupacion_anio
                                                                      WHERE utiliza_tipo = old_obra_grafica),
                                                                 OLD.autoria_disenio,
                                                                 'Diseño ' || old_tipo_disenio::character varying);
    RETURN;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES ('disenio', 'dwh_tr_disenio_del', 'DELETE', SQLERRM);
    RETURN;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_funcionesentidad_crea_in; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_funcionesentidad_crea_in()
RETURNS trigger AS $$
DECLARE
    auxObra integer;
    auxTipoDisenio character varying;
    auxAnio integer;
    auxAgrupacion integer;
BEGIN

    SELECT obra_grafica, tipo_disenio
        INTO auxObra, auxTipoDisenio
        FROM general_accc.disenio
        WHERE id = NEW.disenio;
             
    IF FOUND THEN

        SELECT anio, id
                INTO auxAnio, auxAgrupacion
            FROM general_accc.agrupacion_anio
            WHERE utiliza_tipo = auxObra;
                                                               
        PERFORM general_accc.__dwh_alta_funcionesentidad (NEW.entidad,
                                                          'Crea ' || auxTipoDisenio::character varying,
                                                          auxAnio);
                                                          
        IF auxAgrupacion IS NOT NULL THEN
        -- Si no hay agrupación interprete no hay premio
            PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (1,
                                                                         auxAgrupacion,
                                                                         NEW.entidad,
                                                                         'Crea ' || auxTipoDisenio::character varying);
        END IF;
    END IF;

    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_funcionesentidad_crea_up; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_funcionesentidad_crea_up()
RETURNS trigger AS $$
DECLARE
    auxOldObra integer;
    auxOldTipoDisenio character varying;
    auxNewObra integer;
    auxNewTipoDisenio character varying;
    auxAnio integer;
    auxAgrupacion integer;
BEGIN

    IF OLD.entidad <> NEW.entidad OR OLD.disenio <> NEW.disenio THEN
    -- Ambos son valores obligatorios, de no existir relación se elimina el registro

        SELECT obra_grafica, tipo_disenio
            INTO auxOldObra, auxOldTipoDisenio
            FROM general_accc.disenio
            WHERE id = OLD.disenio;

        SELECT obra_grafica, tipo_disenio
            INTO auxNewObra, auxNewTipoDisenio
            FROM general_accc.disenio
            WHERE id = NEW.disenio;
                 
        PERFORM general_accc.__dwh_baja_funcionesentidad (OLD.entidad,
                                                          auxOldTipoDisenio);
                                                          
        SELECT anio, id
                INTO auxAnio, auxAgrupacion
            FROM general_accc.agrupacion_anio
            WHERE utiliza_tipo = auxOldObra;
                                                          
        IF auxAgrupacion IS NOT NULL THEN
        -- Si no hay agrupación asociada no hay premio (en el futuro los habrá
        -- por eventos de tipo premio)
            PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (-1,
                                                                         auxAgrupacion,
                                                                         OLD.entidad,
                                                                         'Crea ' || auxOldTipoDisenio::character varying);
        END IF;
        
        PERFORM general_accc.__dwh_alta_funcionesentidad (NEW.entidad,
                                                          auxNewTipoDisenio,
                                                          auxAnio);

                     
        SELECT anio, id
                INTO auxAnio, auxAgrupacion
            FROM general_accc.agrupacion_anio
            WHERE utiliza_tipo = auxNewObra;

        IF auxAgrupacion IS NOT NULL THEN
        -- Si no hay agrupación interprete no hay premio
            PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (1,
                                                                         auxAgrupacion,
                                                                         NEW.entidad,
                                                                         'Crea ' || auxNewTipoDisenio::character varying);
        END IF;
    END IF;

    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_funcionesentidad_crea_del; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_funcionesentidad_crea_del()
RETURNS trigger AS $$
DECLARE
    auxOldTipoDisenio character varying;
    auxOldObra integer;
    auxOldAutoriaDisenio integer;
BEGIN

    SELECT tipo_disenio, obra_grafica, autoria_disenio
        INTO auxOldTipoDisenio, auxOldObra, auxOldAutoriaDisenio
        FROM general_accc.disenio
        WHERE id = OLD.disenio;
             
    IF auxOldAutoriaDisenio IS NOT NULL THEN
    
        PERFORM general_accc.__dwh_baja_funcionesentidad (OLD.entidad,
                                                          auxOldTipoDisenio);
                                                          
        PERFORM general_accc.__dwh_cambiapremiados_funcionesentidad (-1,
                                                                     (SELECT id
                                                                          FROM general_accc.agrupacion_anio
                                                                          WHERE utiliza_tipo = auxOldObra),
                                                                     OLD.entidad,
                                                                     'Crea ' || auxOldTipoDisenio::character varying);
    END IF;

    RETURN OLD;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_cambiapremioagrupacion_funcionesentidad; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.__dwh_cambiapremioagrupacion_funcionesentidad (in_valor integer,
                                                                                       in_agrupacion integer,
                                                                                       in_premio integer)
RETURNS void AS $$
DECLARE
    auxFuncion character varying;
    auxRecord record;
BEGIN

    FOR auxRecord IN (SELECT persona, rol.id, tipo_rol
                          FROM general_accc.rol AS rol
                          INNER JOIN general_accc.agrupacion_anio AS agrup
                              ON agrup.id = rol.agrupacion_anio
                          WHERE rol.agrupacion_anio = in_agrupacion)
    LOOP

        IF auxRecord.tipo_rol = 'Cargo'::general_accc.enum_rol THEN
            SELECT tipo_cargo INTO auxFuncion
                FROM general_accc.cargo
                WHERE rol = auxRecord.id;

        ELSEIF auxRecord.tipo_rol = 'Orquesta'::general_accc.enum_rol THEN
            SELECT i.instrumento INTO auxFuncion
                FROM general_accc.orquesta AS o
                INNER JOIN general_accc.instrumento AS i
                    ON o.instrumento = i.id
                WHERE rol = auxRecord.id;

        ELSEIF auxRecord.tipo_rol = 'Voz'::general_accc.enum_rol THEN
            SELECT tipo_cargo INTO auxFuncion
                FROM general_accc.cargo
                WHERE rol = auxRecord.id;
        END IF;


        IF in_premio = 1 THEN
            UPDATE general_accc.dwh_funciones_entidad SET
                                                      primero = in_valor + (SELECT primero
                                                                                FROM general_accc.dwh_funciones_entidad
                                                                                WHERE entidad = auxRecord.persona
                                                                                    AND funcion = auxFuncion)
                WHERE entidad = auxRecord.persona
                    AND funcion = auxFuncion;

        ELSEIF in_premio = 2 THEN
            UPDATE general_accc.dwh_funciones_entidad SET
                                                      segundo = in_valor + (SELECT segundo
                                                                                FROM general_accc.dwh_funciones_entidad
                                                                                WHERE entidad = auxRecord.persona
                                                                                    AND funcion = auxFuncion)
                WHERE entidad = auxRecord.persona
                    AND funcion = auxFuncion;

        ELSEIF in_premio = 3 THEN
            UPDATE general_accc.dwh_funciones_entidad SET
                                                      tercero = in_valor + (SELECT tercero
                                                                                FROM general_accc.dwh_funciones_entidad
                                                                                WHERE entidad = auxRecord.persona
                                                                                    AND funcion = auxFuncion)
                WHERE entidad = auxRecord.persona
                    AND funcion = auxFuncion;

        ELSEIF in_premio = 4 THEN
            UPDATE general_accc.dwh_funciones_entidad SET
                                                      cuarto = in_valor + (SELECT cuarto
                                                                               FROM general_accc.dwh_funciones_entidad
                                                                               WHERE entidad = auxRecord.persona
                                                                                   AND funcion = auxFuncion)
                WHERE entidad = auxRecord.persona
                    AND funcion = auxFuncion;

        END IF;

    END LOOP;

    -- Premios por autoría literaria
    FOR auxRecord IN (SELECT DISTINCT ON (autoria_letra) autoria_letra, genero
                        FROM general_accc.letra AS l
                        INNER JOIN general_accc.agrupacion_anio AS agrup
                            ON l.obra_literaria = agrup.interpreta_literaria
                        WHERE agrup.id = in_agrupacion)

    LOOP
        IF in_premio = 1 THEN
            UPDATE general_accc.dwh_funciones_entidad SET
                                                      primero = in_valor + (SELECT primero
                                                                                FROM general_accc.dwh_funciones_entidad
                                                                                WHERE entidad = auxRecord.autoria_letra
                                                                                    AND funcion = 'Letra ' || auxRecord.genero)
                WHERE entidad = auxRecord.autoria_letra
                    AND funcion = 'Letra ' || auxRecord.genero;

        ELSEIF in_premio = 2 THEN
            UPDATE general_accc.dwh_funciones_entidad SET
                                                      segundo = in_valor + (SELECT segundo
                                                                                FROM general_accc.dwh_funciones_entidad
                                                                                WHERE entidad = auxRecord.autoria_letra
                                                                                    AND funcion = 'Letra ' || auxRecord.genero)
                WHERE entidad = auxRecord.autoria_letra
                    AND funcion = 'Letra ' || auxRecord.genero;

        ELSEIF in_premio = 3 THEN
            UPDATE general_accc.dwh_funciones_entidad SET
                                                      tercero = in_valor + (SELECT tercero
                                                                                FROM general_accc.dwh_funciones_entidad
                                                                                WHERE entidad = auxRecord.autoria_letra
                                                                                    AND funcion = 'Letra ' || auxRecord.genero)
                WHERE entidad = auxRecord.autoria_letra
                    AND funcion = 'Letra ' || auxRecord.genero;

        ELSEIF in_premio = 4 THEN
            UPDATE general_accc.dwh_funciones_entidad SET
                                                      cuarto = in_valor + (SELECT cuarto
                                                                               FROM general_accc.dwh_funciones_entidad
                                                                               WHERE entidad = auxRecord.autoria_letra
                                                                                   AND funcion = 'Letra ' || auxRecord.genero)
                WHERE entidad = auxRecord.autoria_letra
                    AND funcion = 'Letra ' || auxRecord.genero;

        END IF;

    END LOOP;
    
    -- Premios por autoría musical
    FOR auxRecord IN (SELECT DISTINCT ON (autoria_musica) autoria_musica, genero
                        FROM general_accc.musica AS musica
                        INNER JOIN general_accc.agrupacion_anio AS agrup
                            ON musica.obra_musical = agrup.interpreta_musical
                        WHERE agrup.id = in_agrupacion)

    LOOP
        IF in_premio = 1 THEN
            UPDATE general_accc.dwh_funciones_entidad SET
                                                      primero = in_valor + (SELECT primero
                                                                                FROM general_accc.dwh_funciones_entidad
                                                                                WHERE entidad = auxRecord.autoria_musica
                                                                                    AND funcion = 'Música ' || auxRecord.genero)
                WHERE entidad = auxRecord.autoria_musica
                    AND funcion = 'Música ' || auxRecord.genero;

        ELSEIF in_premio = 2 THEN
            UPDATE general_accc.dwh_funciones_entidad SET
                                                      segundo = in_valor + (SELECT segundo
                                                                                FROM general_accc.dwh_funciones_entidad
                                                                                WHERE entidad = auxRecord.autoria_musica
                                                                                    AND funcion = 'Música ' || auxRecord.genero)
                WHERE entidad = auxRecord.autoria_musica
                    AND funcion = 'Música ' || auxRecord.genero;

        ELSEIF in_premio = 3 THEN
            UPDATE general_accc.dwh_funciones_entidad SET
                                                      tercero = in_valor + (SELECT tercero
                                                                                FROM general_accc.dwh_funciones_entidad
                                                                                WHERE entidad = auxRecord.autoria_musica
                                                                                    AND funcion = 'Música ' || auxRecord.genero)
                WHERE entidad = auxRecord.autoria_musica
                    AND funcion = 'Música ' || auxRecord.genero;

        ELSEIF in_premio = 4 THEN
            UPDATE general_accc.dwh_funciones_entidad SET
                                                      cuarto = in_valor + (SELECT cuarto
                                                                               FROM general_accc.dwh_funciones_entidad
                                                                               WHERE entidad = auxRecord.autoria_musica
                                                                                   AND funcion = 'Música ' || auxRecord.genero)
                WHERE entidad = auxRecord.autoria_musica
                    AND funcion = 'Música ' || auxRecord.genero;

        END IF;

    END LOOP;
    
    -- Premios por autoría grafica
    FOR auxRecord IN (SELECT DISTINCT ON (autoria_disenio) autoria_disenio, tipo_disenio
                        FROM general_accc.disenio AS d
                        INNER JOIN general_accc.agrupacion_anio AS agrup
                            ON d.obra_grafica = agrup.utiliza_tipo
                        WHERE agrup.id = in_agrupacion)

    LOOP
        IF in_premio = 1 THEN
            UPDATE general_accc.dwh_funciones_entidad SET
                                                      primero = in_valor + (SELECT primero
                                                                                FROM general_accc.dwh_funciones_entidad
                                                                                WHERE entidad = auxRecord.autoria_disenio
                                                                                    AND funcion = 'Diseño ' || auxRecord.tipo_disenio)
                WHERE entidad = auxRecord.autoria_disenio
                    AND funcion = 'Diseño ' || auxRecord.tipo_disenio;

        ELSEIF in_premio = 2 THEN
            UPDATE general_accc.dwh_funciones_entidad SET
                                                      segundo = in_valor + (SELECT segundo
                                                                                FROM general_accc.dwh_funciones_entidad
                                                                                WHERE entidad = auxRecord.autoria_disenio
                                                                                    AND funcion = 'Diseño ' || auxRecord.tipo_disenio)
                WHERE entidad = auxRecord.autoria_disenio
                    AND funcion = 'Diseño ' || auxRecord.tipo_disenio;

        ELSEIF in_premio = 3 THEN
            UPDATE general_accc.dwh_funciones_entidad SET
                                                      tercero = in_valor + (SELECT tercero
                                                                                FROM general_accc.dwh_funciones_entidad
                                                                                WHERE entidad = auxRecord.autoria_disenio
                                                                                    AND funcion = 'Diseño ' || auxRecord.tipo_disenio)
                WHERE entidad = auxRecord.autoria_disenio
                    AND funcion = 'Diseño ' || auxRecord.tipo_disenio;

        ELSEIF in_premio = 4 THEN
            UPDATE general_accc.dwh_funciones_entidad SET
                                                      cuarto = in_valor + (SELECT cuarto
                                                                               FROM general_accc.dwh_funciones_entidad
                                                                               WHERE entidad = auxRecord.autoria_disenio
                                                                                   AND funcion = 'Diseño ' || auxRecord.tipo_disenio)
                WHERE entidad = auxRecord.autoria_disenio
                    AND funcion = 'Diseño ' || auxRecord.tipo_disenio;

        END IF;
    END LOOP;


    -- Premios por creación    
    FOR auxRecord IN (SELECT crea.entidad, tipo_disenio
                          FROM general_accc.disenio AS d
                          INNER JOIN general_accc.agrupacion_anio AS agrup
                              ON d.obra_grafica = agrup.utiliza_tipo
                          INNER JOIN general_accc.crea AS crea
                              ON d.id = crea.disenio
                          WHERE agrup.id = in_agrupacion)

    LOOP
        IF in_premio = 1 THEN
            UPDATE general_accc.dwh_funciones_entidad SET
                                                      primero = in_valor + (SELECT primero
                                                                                FROM general_accc.dwh_funciones_entidad
                                                                                WHERE entidad = auxRecord.entidad
                                                                                    AND funcion = 'Crea ' || auxRecord.tipo_disenio)
                WHERE entidad = auxRecord.entidad
                    AND funcion = 'Crea ' || auxRecord.tipo_disenio;

        ELSEIF in_premio = 2 THEN
            UPDATE general_accc.dwh_funciones_entidad SET
                                                      segundo = in_valor + (SELECT segundo
                                                                                FROM general_accc.dwh_funciones_entidad
                                                                                WHERE entidad = auxRecord.entidad
                                                                                    AND funcion = 'Crea ' || auxRecord.tipo_disenio)
                WHERE entidad = auxRecord.entidad
                    AND funcion = 'Crea ' || auxRecord.tipo_disenio;

        ELSEIF in_premio = 3 THEN
            UPDATE general_accc.dwh_funciones_entidad SET
                                                      tercero = in_valor + (SELECT tercero
                                                                                FROM general_accc.dwh_funciones_entidad
                                                                                WHERE entidad = auxRecord.entidad
                                                                                    AND funcion = 'Crea ' || auxRecord.tipo_disenio)
                WHERE entidad = auxRecord.entidad
                    AND funcion = 'Crea ' || auxRecord.tipo_disenio;

        ELSEIF in_premio = 4 THEN
            UPDATE general_accc.dwh_funciones_entidad SET
                                                      cuarto = in_valor + (SELECT cuarto
                                                                               FROM general_accc.dwh_funciones_entidad
                                                                               WHERE entidad = auxRecord.entidad
                                                                                   AND funcion = 'Crea ' || auxRecord.tipo_disenio)
                WHERE entidad = auxRecord.entidad
                    AND funcion = 'Crea ' || auxRecord.tipo_disenio;

        END IF;

    END LOOP;

    RETURN;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES ('dwh_funciones_entidad', '__dwh_cambiapremioagrupacion_funcionesentidad', 'function', SQLERRM);
    RETURN;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_funcionesentidad_dwhpremiosconcurso_in; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_funcionesentidad_dwhpremiosconcurso_in()
RETURNS trigger AS $$
BEGIN

    PERFORM general_accc.__dwh_cambiapremioagrupacion_funcionesentidad(1, NEW.id_agrupacion, NEW.premio);

    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_funcionesentidad_dwhpremiosconcurso_up; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_funcionesentidad_dwhpremiosconcurso_up()
RETURNS trigger AS $$
BEGIN

    IF OLD.premio <> NEW.premio THEN
        PERFORM general_accc.__dwh_cambiapremioagrupacion_funcionesentidad(-1,
                                                                           OLD.id_agrupacion,
                                                                           OLD.premio);
        PERFORM general_accc.__dwh_cambiapremioagrupacion_funcionesentidad(1,
                                                                           NEW.id_agrupacion,
                                                                           NEW.premio);
    END IF;
    
    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_funcionesentidad_dwhpremiosconcurso_up; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_funcionesentidad_dwhpremiosconcurso_del()
RETURNS trigger AS $$
BEGIN

    PERFORM general_accc.__dwh_cambiapremioagrupacion_funcionesentidad(-1, OLD.id_agrupacion, OLD.premio);

    RETURN OLD;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;







-------------------------------------------------------------------------------------------
--                      Creación de disparadores para: dwh_entidad                       --
-------------------------------------------------------------------------------------------

--
-- Nombre: dwh_tr_entidad_persona; Tipo: TRIGGER; Schema: general_accc
--
CREATE TRIGGER dwh_tr_entidad_persona_in
  AFTER INSERT
  ON general_accc.persona
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_entidad_persona_in();

CREATE TRIGGER dwh_tr_entidad_persona_up
  AFTER UPDATE
  ON general_accc.persona
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_entidad_persona_up();


--
-- Nombre: dwh_tr_entidad_agrupacion; Tipo: TRIGGER; Schema: general_accc
--
CREATE TRIGGER dwh_tr_entidad_agrupacion_in
  AFTER INSERT
  ON general_accc.agrupacion
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_entidad_agrupacion_in();

CREATE TRIGGER dwh_tr_entidad_agrupacion_up
  AFTER UPDATE
  ON general_accc.agrupacion
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_entidad_agrupacion_up();


--
-- Nombre: dwh_tr_entidad_empresa; Tipo: TRIGGER; Schema: general_accc
--
CREATE TRIGGER dwh_tr_entidad_empresa_in
  AFTER INSERT
  ON general_accc.empresa
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_entidad_empresa_in();

CREATE TRIGGER dwh_tr_entidad_empresa_up
  AFTER UPDATE
  ON general_accc.empresa
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_entidad_empresa_up();

--
-- Nombre: dwh_tr_entidad_entidad; Tipo: TRIGGER; Schema: general_accc
--
CREATE TRIGGER dwh_tr_entidad_entidad_del
  AFTER DELETE
  ON general_accc.entidad
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_entidad_entidad_del();




-------------------------------------------------------------------------------------------
--                 Creación de disparadores para: dwh_funciones_entidad                  --
-------------------------------------------------------------------------------------------

--
-- Nombre: dwh_tr_funcionesentidad_cargo; Tipo: TRIGGER; Schema: general_accc
--
CREATE TRIGGER dwh_tr_funcionesentidad_cargo_in
  AFTER INSERT
  ON general_accc.cargo
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_funcionesentidad_cargo_in();

CREATE TRIGGER dwh_tr_funcionesentidad_cargo_up
  AFTER UPDATE
  ON general_accc.cargo
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_funcionesentidad_cargo_up();

CREATE TRIGGER dwh_tr_funcionesentidad_cargo_del
  AFTER DELETE
  ON general_accc.cargo
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_funcionesentidad_cargo_del();


--
-- Nombre: dwh_tr_funcionesentidad_voz; Tipo: TRIGGER; Schema: general_accc
--
CREATE TRIGGER dwh_tr_funcionesentidad_voz_in
  AFTER INSERT
  ON general_accc.voz
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_funcionesentidad_voz_in();

CREATE TRIGGER dwh_tr_funcionesentidad_voz_up
  AFTER UPDATE
  ON general_accc.voz
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_funcionesentidad_voz_up();

CREATE TRIGGER dwh_tr_funcionesentidad_voz_del
  AFTER DELETE
  ON general_accc.voz
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_funcionesentidad_voz_del();


--
-- Nombre: dwh_tr_funcionesentidad_orquesta; Tipo: TRIGGER; Schema: general_accc
--
CREATE TRIGGER dwh_tr_funcionesentidad_orquesta_in
  AFTER INSERT
  ON general_accc.orquesta
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_funcionesentidad_orquesta_in();

CREATE TRIGGER dwh_tr_funcionesentidad_orquesta_up
  AFTER UPDATE
  ON general_accc.orquesta
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_funcionesentidad_orquesta_up();

CREATE TRIGGER dwh_tr_funcionesentidad_orquesta_del
  AFTER DELETE
  ON general_accc.orquesta
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_funcionesentidad_orquesta_del();


--
-- Nombre: dwh_tr_funcionesentidad_letra; Tipo: TRIGGER; Schema: general_accc
--
CREATE TRIGGER dwh_tr_funcionesentidad_letra_in
  AFTER INSERT
  ON general_accc.letra
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_funcionesentidad_letra_in();

CREATE TRIGGER dwh_tr_funcionesentidad_letra_up
  AFTER UPDATE
  ON general_accc.letra
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_funcionesentidad_letra_up();

  

--
-- Nombre: dwh_tr_funcionesentidad_musica; Tipo: TRIGGER; Schema: general_accc
--
CREATE TRIGGER dwh_tr_funcionesentidad_musica_in
  AFTER INSERT
  ON general_accc.musica
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_funcionesentidad_musica_in();

CREATE TRIGGER dwh_tr_funcionesentidad_musica_up
  AFTER UPDATE
  ON general_accc.musica
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_funcionesentidad_musica_up();

CREATE TRIGGER dwh_tr_funcionesentidad_musica_del
  AFTER DELETE
  ON general_accc.musica
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_funcionesentidad_musica_del();


--
-- Nombre: dwh_tr_funcionesentidad_cargo; Tipo: TRIGGER; Schema: general_accc
--
CREATE TRIGGER dwh_tr_funcionesentidad_disenio_in
  AFTER INSERT
  ON general_accc.disenio
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_funcionesentidad_disenio_in();

CREATE TRIGGER dwh_tr_funcionesentidad_disenio_up
  AFTER UPDATE
  ON general_accc.disenio
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_funcionesentidad_disenio_up();


--
-- Nombre: dwh_tr_funcionesentidad_crea; Tipo: TRIGGER; Schema: general_accc
--
CREATE TRIGGER dwh_tr_funcionesentidad_crea_in
  AFTER INSERT
  ON general_accc.crea
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_funcionesentidad_crea_in();

CREATE TRIGGER dwh_tr_funcionesentidad_crea_up
  AFTER UPDATE
  ON general_accc.crea
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_funcionesentidad_crea_up();

CREATE TRIGGER dwh_tr_funcionesentidad_crea_del
  BEFORE DELETE
  ON general_accc.crea
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_funcionesentidad_crea_del();


--
-- Nombre: dwh_tr_funcionesentidad_dwhpremiosconcurso; Tipo: TRIGGER; Schema: general_accc
--
CREATE TRIGGER dwh_tr_funcionesentidad_dwhpremiosconcurso_in
  AFTER INSERT
  ON general_accc.dwh_premios_concurso
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_funcionesentidad_dwhpremiosconcurso_in();

CREATE TRIGGER dwh_tr_funcionesentidad_dwhpremiosconcurso_up
  AFTER UPDATE
  ON general_accc.dwh_premios_concurso
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_funcionesentidad_dwhpremiosconcurso_up();

CREATE TRIGGER dwh_tr_funcionesentidad_dwhpremiosconcurso_del
  BEFORE DELETE
  ON general_accc.dwh_premios_concurso
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_funcionesentidad_dwhpremiosconcurso_del();



