-- TFG BBDD Carnaval de Cádiz - Paloma Cortés


-- Creación de funciones para el data warehouse




-------------------------------------------------------------------------------------------
--                Creación de procedimientos para: dwh_premios_concurso                  --
-------------------------------------------------------------------------------------------

--
-- Nombre: dwh_sp_premiosconcurso_evento_up; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_premiosconcurso_evento_up()
RETURNS trigger AS $$
BEGIN

    IF NEW.nombre <> OLD.nombre OR NEW.localidad <> OLD.localidad THEN
    -- En caso de cambiar la cadena que identifica el concurso se actualizan
    -- los registros que lo utilizan
        UPDATE general_accc.dwh_premios_concurso SET
                                                 concurso = NEW.nombre,
                                                 localidad = NEW.localidad
            WHERE id_concurso = NEW.id;
    END IF;
    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_premiosconcurso_agrupacion_up; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_premiosconcurso_agrupacionanio_up (new_agrupacion integer,
                                                                                  new_nombre character varying,
                                                                                  old_nombre character varying)
RETURNS void AS $$
BEGIN

    IF new_nombre <> old_nombre THEN
    -- En caso de cambiar la cadena que identifica la agrupación se actualizan
    -- los registros que lo utilizan
        UPDATE general_accc.dwh_premios_concurso SET
                                                 agrupacion = new_nombre
            WHERE id_agrupacion = new_agrupacion;
    END IF;
    RETURN;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES ('agrupacion_anio', 'dwh_sp_premiosconcurso_agrupacionanio_up', 'UPDATE', SQLERRM);
    RETURN;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: __dwh_alta_premiosconcurso; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.__dwh_alta_premiosconcurso(in_agrupacion_anio integer,
                                                                   in_concurso integer,
                                                                   in_clasificacion integer)
RETURNS void AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
    auxAnio integer;
    auxModalidad integer;
    auxCategoria general_accc.enum_categoria;
    auxAgrupacion character varying;
    auxLocalidadAgrup character varying;
    auxLiteraria integer;
    auxMusical integer;
    auxAutor integer;
    auxAutorL character varying;
    auxAutorM character varying;
    auxConcurso character varying;
    auxLocalidad character varying;
BEGIN

    -- Valores necesarios de la agrupación
    SELECT anio, modalidad, categoria, nombre, localidad, interpreta_literaria, interpreta_musical
            INTO auxAnio, auxModalidad, auxCategoria, auxAgrupacion, auxLocalidadAgrup, auxLiteraria, auxMusical
        FROM general_accc.agrupacion_anio AS a_a
        INNER JOIN general_accc.agrupacion AS agrup
            ON a_a.entidad = agrup.entidad
        WHERE a_a.id = in_agrupacion_anio;

    IF FOUND THEN
        IF auxLiteraria IS NOT NULL THEN
        -- Calcular la autoría por mayoría
            SELECT l.autoria_letra, COUNT(l.id) AS total
                    INTO auxAutor, auxInteger
                FROM general_accc.letra AS l
                INNER JOIN general_accc.agrupacion_anio AS agrup
                    ON l.obra_literaria = agrup.interpreta_literaria
                WHERE agrup.id = in_agrupacion_anio
                GROUP BY l.autoria_letra
                ORDER BY total DESC
                LIMIT 1;

            SELECT id, tipo_entidad
                    INTO auxAutor, auxCharacter
                FROM general_accc.entidad
                WHERE id = auxAutor;

            IF auxCharacter = 'Persona' THEN
                SELECT nombre || ' ' || apellido1 || ' ' || COALESCE(apellido2, '')
                        INTO auxAutorL
                    FROM general_accc.persona
                    WHERE entidad = auxAutor;
            ELSEIF auxCharacter = 'Agrupacion' THEN
                auxAutorL := 'La agrupación';
            END IF;
        END IF;


        IF auxMusical IS NOT NULL THEN
        -- Calcular la autoría
            SELECT musica.autoria_musica, COUNT(musica.id) AS total
                    INTO auxAutor, auxInteger
                FROM general_accc.musica AS musica
                INNER JOIN general_accc.agrupacion_anio AS agrup
                    ON musica.obra_musical = agrup.interpreta_musical
                WHERE agrup.id = in_agrupacion_anio
                GROUP BY musica.autoria_musica
                ORDER BY total DESC
                LIMIT 1;

            SELECT id, tipo_entidad
                    INTO auxAutor, auxCharacter
                FROM general_accc.entidad
                WHERE id =  auxAutor;

            IF auxCharacter = 'Persona' THEN
                SELECT nombre || ' ' || apellido1 || ' ' || COALESCE(apellido2, '')
                        INTO auxAutorL
                    FROM general_accc.persona
                    WHERE entidad = auxAutor;
            ELSEIF auxCharacter = 'Agrupacion' THEN
                auxAutorL := 'La agrupación';
            END IF;
        END IF;

        -- Valores necesarios del concurso, la categoría ya coincide por restricciones
        -- semánticas del modelo
        SELECT nombre, localidad
                INTO auxConcurso, auxLocalidad
            FROM general_accc.evento
            WHERE id = in_concurso;
        IF FOUND THEN
            INSERT INTO general_accc.dwh_premios_concurso (id_concurso, concurso,
                                                           localidad, categoria,
                                                           anio, modalidad, premio,
                                                           id_agrupacion, agrupacion,
                                                           autor_letra,
                                                           autor_musica, localidad_agrup)
                VALUES (
                    in_concurso, auxConcurso, auxLocalidad, auxCategoria, auxAnio,
                    (SELECT modalidad FROM general_accc.modalidad WHERE id = auxModalidad),
                    in_clasificacion, in_agrupacion_anio, auxAgrupacion, auxAutorL,
                    auxAutorM, auxLocalidadAgrup);
        END IF;
    END IF;

    RETURN;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES ('dwh_premios_concurso', '__dwh_alta_premiosconcurso', 'function', SQLERRM);
    RETURN;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: dwh_sp_premiosconcurso_participa_in; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_premiosconcurso_participa_in()
RETURNS trigger AS $$
BEGIN

    -- La inserción en dwh_premios_concurso solo se da para premios menores que 4
    -- es decir, para el primero, segundo y tercero
    IF NEW.clasificacion IS NOT NULL AND NEW.clasificacion > 4 THEN
        RETURN NEW;
    END IF;

    PERFORM general_accc.__dwh_alta_premiosconcurso (NEW.agrupacion_anio, NEW.concurso, NEW.clasificacion);

    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;


--
-- Nombre: dwh_sp_premiosconcurso_participa_up; Tipo: FUNCTION; Schema: general_accc
--
-- Desde la tabla participa ya se accede al año, la modalidad y el concurso
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_premiosconcurso_participa_up()
RETURNS trigger AS $$
BEGIN

    IF (OLD.clasificacion IS NULL OR COALESCE(OLD.clasificacion, 5) > 4)
            AND COALESCE(NEW.clasificacion, 6) < 5 THEN
            -- Las conversiones del null se hacen para que no satisfaga la
            -- condición en caso de serlo
            
        -- Insertar el registro
        PERFORM general_accc.__dwh_alta_premiosconcurso(NEW.agrupacion_anio, NEW.concurso, NEW.clasificacion);
        RETURN NEW;
    END IF;

    IF COALESCE(OLD.clasificacion, 0) <> COALESCE(NEW.clasificacion, 0)
            AND COALESCE(OLD.clasificacion, 6) < 5
            AND COALESCE(NEW.clasificacion, 0) > 4 THEN
            
        -- Borrar el registro
        DELETE FROM general_accc.dwh_premios_concurso
            WHERE id_concurso = NEW.concurso
                AND id_agrupacion = NEW.agrupacion_anio;
        RETURN NEW;
    END IF;

    IF COALESCE(OLD.clasificacion, 0) <> COALESCE(NEW.clasificacion, 0)
            AND COALESCE(OLD.clasificacion, 6) < 5
            AND COALESCE(NEW.clasificacion, 6) < 5 THEN
        -- Actualizar el registro
        UPDATE general_accc.dwh_premios_concurso SET premio = NEW.clasificacion
            WHERE id_concurso = NEW.concurso
                AND id_agrupacion = NEW.agrupacion_anio;
    END IF;
    RETURN NEW;


EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_premiosconcurso_participa_del; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_premiosconcurso_participa_del()
RETURNS trigger AS $$
BEGIN

    DELETE FROM general_accc.dwh_premios_concurso
        WHERE id_concurso = OLD.concurso
            AND id_agrupacion = OLD.agrupacion_anio;
    RETURN OLD;


EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;









-------------------------------------------------------------------------------------------
--         Creación de disparadores para: dwh_premios_concurso          --
-------------------------------------------------------------------------------------------

--
-- Nombre: dwh_tr_premiosconcurso_evento_up; Tipo: TRIGGER; Schema: general_accc
--
CREATE TRIGGER dwh_tr_premiosconcurso_evento_up
  AFTER UPDATE
  ON general_accc.evento
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_premiosconcurso_evento_up();


--
-- Nombre: dwh_tr_premiosconcurso_participa; Tipo: TRIGGER; Schema: general_accc
--
CREATE TRIGGER dwh_tr_premiosconcurso_participa_in
  AFTER INSERT
  ON general_accc.participa
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_premiosconcurso_participa_in();

CREATE TRIGGER dwh_tr_premiosconcurso_participa_up
  AFTER UPDATE
  ON general_accc.participa
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_premiosconcurso_participa_up();

CREATE TRIGGER dwh_tr_premiosconcurso_participa_del
  BEFORE DELETE
  ON general_accc.participa
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_premiosconcurso_participa_del();

