-- TFG BBDD Carnaval de Cádiz - Paloma Cortés


-- Creación de funciones para el data warehouse



-------------------------------------------------------------------------------------------
--        Creación de procedimientos y disparadores para: dwh_soporte_documental         --
-------------------------------------------------------------------------------------------


--
-- Nombre: __dwh_alta_soportedocumental; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.__dwh_alta_soportedocumental (in_agrupacionanio integer,
                                                                      in_agrupacionanio_nombre character varying,
                                                                      in_tipo_obra general_accc.enum_obra,
                                                                      in_tipo_documento general_accc.enum_tipodocumento,
                                                                      in_tipo_soporte general_accc.enum_tiposoporte)
RETURNS void AS $$
DECLARE
    auxInteger integer;
BEGIN
    -- comprobar si existe el registro para el tipo de obra, de documento y de soporte
    IF in_tipo_soporte <> 'No disponible'::general_accc.enum_tiposoporte THEN
    -- tipo_soporte no admite nulos, aquí siempre llegará un valor
        SELECT total INTO auxInteger
            FROM general_accc.dwh_soporte_documental
            WHERE id_agrupacion = in_agrupacionanio
                AND tipo_obra = in_tipo_obra
                AND tipo_documento = in_tipo_documento
                AND tipo_soporte = in_tipo_soporte;
                
        IF FOUND THEN
            UPDATE general_accc.dwh_soporte_documental SET
                                                       total = auxInteger + 1
                WHERE id_agrupacion = in_agrupacionanio
                    AND tipo_obra = in_tipo_obra
                    AND tipo_documento = in_tipo_documento
                    AND tipo_soporte = in_tipo_soporte;
                    
        ELSEIF in_agrupacionanio IS NOT NULL AND in_agrupacionanio_nombre IS NOT NULL AND in_tipo_obra IS NOT NULL THEN
        
            INSERT INTO general_accc.dwh_soporte_documental (id_agrupacion,
                                                             agrupacion,
                                                             tipo_obra,
                                                             tipo_documento,
                                                             tipo_soporte,
                                                             total)
                VALUES (in_agrupacionanio, in_agrupacionanio_nombre,
                        in_tipo_obra, in_tipo_documento, in_tipo_soporte, 1);
                
        -- ELSE se gestiona al actualizar la tabla documento
        END IF;

    END IF;
    RETURN;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES ('dwh_soporte_documental', '__dwh_alta_soportedocumental', 'function', SQLERRM);
    RETURN;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: __dwh_baja_soportedocumental; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.__dwh_baja_soportedocumental (in_agrupacionanio integer,
                                                                      in_tipo_obra general_accc.enum_obra,
                                                                      in_tipo_documento general_accc.enum_tipodocumento,
                                                                      in_tipo_soporte general_accc.enum_tiposoporte)
RETURNS void AS $$
DECLARE
    auxInteger integer;
BEGIN
    SELECT total INTO auxInteger
        FROM general_accc.dwh_soporte_documental
        WHERE id_agrupacion = in_agrupacionanio
            AND tipo_obra = in_tipo_obra
            AND tipo_documento = in_tipo_documento
            AND tipo_soporte = in_tipo_soporte;
            
    IF auxInteger > 1 THEN
        UPDATE general_accc.dwh_soporte_documental SET total = total - 1
            WHERE id_agrupacion = in_agrupacionanio
                AND tipo_obra = in_tipo_obra
                AND tipo_documento = in_tipo_documento
                AND tipo_soporte = in_tipo_soporte;
                    
    ELSEIF auxInteger = 1 THEN
        DELETE FROM general_accc.dwh_soporte_documental
            WHERE id_agrupacion = in_agrupacionanio
                AND tipo_obra = in_tipo_obra
                AND tipo_documento = in_tipo_documento
                AND tipo_soporte = in_tipo_soporte;
                    
    -- ELSE no debe haber recibido los datos de agrupación
    END IF;
    RETURN;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES ('dwh_soporte_documental', '__dwh_baja_soportedocumental', 'function', SQLERRM);
    RETURN;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: __dwh_alta_soportedocumental_obra; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.__dwh_alta_soportedocumental_obra (in_agrupacionanio integer,
                                                                           in_agrupacionanio_nombre character varying,
                                                                           in_tipo_obra general_accc.enum_obra,
                                                                           in_obra integer)
RETURNS void AS $$
DECLARE
    auxInteger integer;
    auxRecord record;
BEGIN

    FOR auxRecord IN (SELECT d.tipo_documento, d.tipo_soporte
                          FROM general_accc.documento AS d
                          INNER JOIN general_accc.documento_obra AS d_o
                              ON d.id = d_o.documento
                          WHERE d_o.obra = in_obra)
    LOOP
        PERFORM general_accc.__dwh_alta_soportedocumental (in_agrupacionanio, 
                                                           in_agrupacionanio_nombre,
                                                           in_tipo_obra,
                                                           auxRecord.tipo_documento,
                                                           auxRecord.tipo_soporte);
    END LOOP;


    IF in_tipo_obra = 'Literaria'::general_accc.enum_obra THEN
               
        FOR auxRecord IN (SELECT doc.tipo_documento, doc.tipo_soporte
                              FROM general_accc.documento AS doc
                              INNER JOIN general_accc.documento_letra AS d_l
                                  ON doc.id = d_l.documento
                              INNER JOIN general_accc.letra AS l
                                  ON l.id = d_l.letra
                              WHERE l.obra_literaria = in_obra)
        LOOP
            PERFORM general_accc.__dwh_alta_soportedocumental (in_agrupacionanio, 
                                                               in_agrupacionanio_nombre,
                                                               in_tipo_obra,
                                                               auxRecord.tipo_documento,
                                                               auxRecord.tipo_soporte);

        END LOOP;

    ELSEIF in_tipo_obra = 'Musical'::general_accc.enum_obra THEN
                   
        FOR auxRecord IN (SELECT doc.tipo_documento, doc.tipo_soporte
                              FROM general_accc.documento AS doc
                              INNER JOIN general_accc.documento_musica AS d_m
                                  ON doc.id = d_m.documento
                              INNER JOIN general_accc.musica AS musica
                                  ON musica.id = d_m.musica
                              WHERE musica.obra_musical = in_obra)
        LOOP
            PERFORM general_accc.__dwh_alta_soportedocumental (in_agrupacionanio, 
                                                               in_agrupacionanio_nombre,
                                                               in_tipo_obra,
                                                               auxRecord.tipo_documento,
                                                               auxRecord.tipo_soporte);

        END LOOP;
    ELSE
                   
        FOR auxRecord IN (SELECT doc.tipo_documento, doc.tipo_soporte
                              FROM general_accc.documento AS doc
                              INNER JOIN general_accc.documento_disenio AS d_d
                                  ON doc.id = d_d.documento
                              INNER JOIN general_accc.disenio AS d
                                  ON d.id = d_d.disenio
                              WHERE d.obra_grafica = in_obra)
        LOOP
            PERFORM general_accc.__dwh_alta_soportedocumental (in_agrupacionanio, 
                                                               in_agrupacionanio_nombre,
                                                               in_tipo_obra,
                                                               auxRecord.tipo_documento,
                                                               auxRecord.tipo_soporte);

        END LOOP;

    END IF;
    RETURN;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES ('dwh_soporte_documental', '__dwh_alta_soportedocumental_obra', 'function', SQLERRM);
    RETURN;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_soportedocumental_agrupacion_in; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_soportedocumental_agrupacionanio_in (new_id integer,
                                                                                    new_nombre character varying,
                                                                                    new_interpreta_literaria integer,
                                                                                    new_interpreta_musical integer,
                                                                                    new_utiliza_tipo integer)
RETURNS void AS $$
DECLARE
    auxInteger integer;
BEGIN
    IF new_interpreta_literaria IS NOT NULL THEN
        PERFORM general_accc.__dwh_alta_soportedocumental_obra (new_id,
                                                                new_nombre,
                                                                'Literaria',
                                                                new_interpreta_literaria);
    END IF;

    IF new_interpreta_musical IS NOT NULL THEN
        PERFORM general_accc.__dwh_alta_soportedocumental_obra (new_id,
                                                                new_nombre,
                                                                'Musical',
                                                                new_interpreta_musical);
    END IF;

    IF new_utiliza_tipo IS NOT NULL THEN
        PERFORM general_accc.__dwh_alta_soportedocumental_obra (new_id, 
                                                                new_nombre,
                                                                'Grafica',
                                                                new_utiliza_tipo);
    END IF;

    RETURN;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES ('agrupacion_anio', 'dwh_tr_agrupacionanio_in', 'INSERT', SQLERRM);
    RETURN;
END;
$$ LANGUAGE plpgsql;


--
-- Nombre: dwh_sp_soportedocumental_agrupacion_up; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_soportedocumental_agrupacionanio_up (new_id integer,
                                                                                    old_id integer,
                                                                                    new_nombre character varying,
                                                                                    old_nombre character varying,
                                                                                    new_interpreta_literaria integer,
                                                                                    old_interpreta_literaria integer,
                                                                                    new_interpreta_musical integer,
                                                                                    old_interpreta_musical integer,
                                                                                    new_utiliza_tipo integer,
                                                                                    old_utiliza_tipo integer)
RETURNS void AS $$
DECLARE
    auxInteger integer;
BEGIN
    IF COALESCE(new_interpreta_literaria, 0) <> COALESCE(old_interpreta_literaria, 0) THEN

        IF old_interpreta_literaria IS NOT NULL THEN
            DELETE FROM general_accc.dwh_soporte_documental
                WHERE id_agrupacion = old_id
                    AND tipo_obra = 'Literaria'::general_accc.enum_obra;
        END IF;

        IF new_interpreta_literaria IS NOT NULL THEN
            PERFORM general_accc.__dwh_alta_soportedocumental_obra (new_id,
                                                                    new_nombre,
                                                                    'Literaria',
                                                                    new_interpreta_literaria);
        END IF;

    END IF;

    IF COALESCE(new_interpreta_musical, 0) <> COALESCE(old_interpreta_musical, 0) THEN

        IF old_interpreta_literaria IS NOT NULL THEN
            DELETE FROM general_accc.dwh_soporte_documental
                WHERE id_agrupacion = old_id
                    AND tipo_obra = 'Musical'::general_accc.enum_obra;
        END IF;
        
        IF new_interpreta_musical IS NOT NULL THEN
            PERFORM general_accc.__dwh_alta_soportedocumental_obra (new_id,
                                                                    new_nombre,
                                                                    'Musical',
                                                                    new_interpreta_musical);
        END IF;

    END IF;

    IF COALESCE(new_utiliza_tipo, 0) <> COALESCE(old_utiliza_tipo, 0) THEN
    
        DELETE FROM general_accc.dwh_soporte_documental
            WHERE id_agrupacion = new_id
                AND tipo_obra = 'Grafica'::general_accc.enum_obra;
                
        IF new_utiliza_tipo IS NOT NULL THEN
            PERFORM general_accc.__dwh_alta_soportedocumental_obra (new_id,
                                                                    new_nombre,
                                                                    'Grafica',
                                                                    new_utiliza_tipo);
        END IF;
    END IF;

    RETURN;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES ('agrupacion_anio', 'dwh_sp_soportedocumental_agrupacionanio_up', 'UPDATE', SQLERRM);
    RETURN;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_soportedocumental_agrupacion_del; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_soportedocumental_agrupacionanio_del(old_id integer)
RETURNS void AS $$
DECLARE
    auxInteger integer;
BEGIN
    DELETE FROM general_accc.dwh_soporte_documental
        WHERE id_agrupacion = old_id;

    RETURN;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES ('agrupacion_anio', 'dwh_tr_agrupacionanio_del', 'DELETE', SQLERRM);
    RETURN;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: dwh_sp_soportedocumental_documento_up; Tipo: FUNCTION; Schema: general_accc
--
-- La creación de documentos se controla a partir de las tabas de N:M respecto
-- de obras y componentes de las mismas.
-- En este caso se gestionan los que durante el alta no se contaron por tener
-- estado no disponible.
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_soportedocumental_documento_up()
RETURNS trigger AS $$
DECLARE
    auxInteger integer;
    auxCursor refcursor;
    auxRecord record;
    auxIdAgrupacion integer;
    auxAgrupacion character varying;
BEGIN


    IF OLD.tipo_documento <> NEW.tipo_documento OR OLD.tipo_soporte <> NEW.tipo_soporte THEN
    -- tipo_documento y tipo_soporte siempre contienen un valor

        -- La tabla documento_obra representa una relación N:M, por lo que un
        -- documento puede aparecer en varios registros
        FOR auxRecord IN (SELECT agrup.id AS id_agrupacion, agrup.nombre, tipo_obra
                              FROM general_accc.documento AS d
                              INNER JOIN general_accc.documento_obra AS d_o
                                  ON d.id = d_o.documento
                              INNER JOIN general_accc.obra AS obra
                                  ON obra.signatura = d_o.obra
                              INNER JOIN general_accc.agrupacion_anio AS agrup
                                  ON obra.signatura = agrup.interpreta_literaria
                                      OR obra.signatura = agrup.interpreta_musical
                                      OR obra.signatura = agrup.utiliza_tipo
                              WHERE d_o.documento = NEW.id)
        LOOP
            PERFORM general_accc.__dwh_baja_soportedocumental (auxRecord.id_agrupacion,
                                                               auxRecord.tipo_obra,
                                                               OLD.tipo_documento,
                                                               OLD.tipo_soporte);
                                                               
            IF NEW.tipo_soporte <> 'No disponible'::general_accc.enum_tiposoporte THEN
                PERFORM general_accc.__dwh_alta_soportedocumental (auxRecord.id_agrupacion,
                                                                   auxRecord.nombre,
                                                                   auxRecord.tipo_obra,
                                                                   NEW.tipo_documento,
                                                                   NEW.tipo_soporte);
            END IF;

        END LOOP;

        -- En las relaciones documento_letra, documento_musica y documento_disenio
        -- si aparece el documento lo hará solo una vez
        SELECT agrup.id, agrup.nombre
                INTO auxIdAgrupacion, auxAgrupacion
            FROM general_accc.documento_letra AS d_l
            INNER JOIN general_accc.letra AS l
                ON d_l.letra = l.id
            INNER JOIN general_accc.agrupacion_anio AS agrup
                ON agrup.interpreta_literaria = l.obra_literaria
            WHERE d_l.documento = NEW.id;
            
        IF FOUND THEN
            PERFORM general_accc.__dwh_baja_soportedocumental (auxIdAgrupacion,
                                                               'Literaria'::general_accc.enum_obra,
                                                               OLD.tipo_documento,
                                                               OLD.tipo_soporte);
                        
            IF NEW.tipo_soporte <> 'No disponible'::general_accc.enum_tiposoporte THEN
                PERFORM general_accc.__dwh_alta_soportedocumental (auxIdAgrupacion,
                                                                   auxAgrupacion,
                                                                   'Literaria',
                                                                   NEW.tipo_documento,
                                                                   NEW.tipo_soporte);
            END IF;
        END IF;

        SELECT agrup.id, agrup.nombre
                INTO auxIdAgrupacion, auxAgrupacion
            FROM general_accc.documento_musica AS d_m
            INNER JOIN general_accc.musica AS musica
                ON d_m.musica = musica.id
            INNER JOIN general_accc.agrupacion_anio AS agrup
                ON agrup.interpreta_musical = musica.obra_musical
            WHERE d_m.documento = NEW.id;
            
        IF FOUND THEN
            PERFORM general_accc.__dwh_baja_soportedocumental (auxIdAgrupacion,
                                                               'Musical'::general_accc.enum_obra,
                                                               OLD.tipo_documento,
                                                               OLD.tipo_soporte);
                        
            IF NEW.tipo_soporte <> 'No disponible'::general_accc.enum_tiposoporte THEN
                PERFORM general_accc.__dwh_alta_soportedocumental (auxIdAgrupacion,
                                                                   auxAgrupacion,
                                                                   'Musical',
                                                                   NEW.tipo_documento,
                                                                   NEW.tipo_soporte);
            END IF;
        END IF;

        SELECT agrup.id, agrup.nombre
                INTO auxIdAgrupacion, auxAgrupacion
            FROM general_accc.documento_disenio AS d_d
                INNER JOIN general_accc.disenio AS d
                    ON d_d.disenio = d.id
                INNER JOIN general_accc.agrupacion_anio AS agrup
                    ON agrup.utiliza_tipo = d.obra_grafica
            WHERE d_d.documento = NEW.id;
        IF FOUND THEN
            PERFORM general_accc.__dwh_baja_soportedocumental (auxIdAgrupacion,
                                                               'Grafica',
                                                               OLD.tipo_documento,
                                                               OLD.tipo_soporte);
            IF NEW.tipo_soporte <> 'No disponible'::general_accc.enum_tiposoporte THEN
            -- tipo_soporte es un valor obligatorio
                PERFORM general_accc.__dwh_alta_soportedocumental (auxIdAgrupacion,
                                                                   auxAgrupacion,
                                                                   'Grafica',
                                                                   NEW.tipo_documento,
                                                                   NEW.tipo_soporte);
            END IF; 
        END IF;

    END IF;

    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_soportedocumental_documentoobra_in; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_soportedocumental_documentoobra_in()
RETURNS trigger AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
    auxIdAgrupacion integer;
    auxAgrupacion character varying;
    auxTipoDocumento general_accc.enum_tipodocumento;
    auxTipoSoporte general_accc.enum_tiposoporte;
    auxTipoObra general_accc.enum_obra;
BEGIN
                    
    SELECT agrup.id, agrup.nombre, tipo_documento, tipo_soporte, tipo_obra
            INTO auxIdAgrupacion, auxAgrupacion, auxTipoDocumento, auxTipoSoporte, auxTipoObra
        FROM general_accc.documento AS doc, general_accc.obra, general_accc.agrupacion_anio AS agrup
        WHERE doc.id = NEW.documento
            AND signatura = NEW.obra
            AND (agrup.interpreta_literaria = NEW.obra
                    OR agrup.interpreta_musical = NEW.obra
                    OR agrup.utiliza_tipo = NEW.obra);

    IF FOUND AND auxTipoSoporte <> 'No disponible'::general_accc.enum_tiposoporte THEN
        -- La tabla documento_obra representa una relación N:M, por lo que un
        -- documento puede aparecer en varios registros
        PERFORM general_accc.__dwh_alta_soportedocumental (auxIdAgrupacion,
                                                           auxAgrupacion,
                                                           auxTipoObra,
                                                           auxTipoDocumento,
                                                           auxTipoSoporte);
    END IF;

    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_soportedocumental_documentoobra_del; Tipo: FUNCTION; Schema: general_accc
--
-- Este solo decrementa el contador en caso de que el documento y la obra aún existan.
-- En caso cantrario se decrementan al desasociar obras de agrupaciones o eliminar documentos.
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_soportedocumental_documentoobra_del()
RETURNS trigger AS $$
DECLARE
    auxIdAgrupacion integer;
    auxTipoDocumento general_accc.enum_tipodocumento;
    auxTipoSoporte general_accc.enum_tiposoporte;
    auxTipoObra general_accc.enum_obra;
BEGIN
            
    SELECT doc.tipo_documento, doc.tipo_soporte, tipo_obra
            INTO auxTipoDocumento, auxTipoSoporte, auxTipoObra
        FROM general_accc.documento AS doc, general_accc.obra, general_accc.agrupacion_anio AS agrup
        WHERE doc.id = OLD.documento
            AND signatura = OLD.obra
            AND (interpreta_literaria = OLD.obra
                 OR interpreta_musical = OLD.obra
                 OR utiliza_tipo = OLD.obra);

    IF FOUND THEN
        PERFORM general_accc.__dwh_baja_soportedocumental (auxIdAgrupacion,
                                                           auxTipoObra,
                                                           auxTipoDocumento,
                                                           auxTipoSoporte);
    END IF;

    RETURN OLD;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: dwh_sp_soportedocumental_documentoletra_in; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_soportedocumental_documentoletra_in()
RETURNS trigger AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
    auxIdAgrupacion integer;
    auxAgrupacion character varying;
    auxTipoDocumento general_accc.enum_tipodocumento;
    auxTipoSoporte general_accc.enum_tiposoporte;
BEGIN           
    SELECT tipo_documento, tipo_soporte, agrup.id, agrup.nombre
            INTO auxTipoDocumento, auxTipoSoporte, auxIdAgrupacion, auxAgrupacion
        FROM general_accc.documento_letra AS d_l
        INNER JOIN general_accc.letra AS l
            ON d_l.letra = l.id
        INNER JOIN general_accc.agrupacion_anio AS agrup
            ON agrup.interpreta_literaria = l.obra_literaria
        INNER JOIN general_accc.documento AS doc
            ON d_l.documento = doc.id
        WHERE d_l.documento = NEW.documento
            AND l.id = NEW.letra;
            
    IF FOUND AND auxTipoSoporte <> 'No disponible'::general_accc.enum_tiposoporte THEN
        PERFORM general_accc.__dwh_alta_soportedocumental (auxIdAgrupacion,
                                                           auxAgrupacion,
                                                           'Literaria',
                                                           auxTipoDocumento,
                                                           auxTipoSoporte);
    END IF;

    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_soportedocumental_documentoletra_del; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_soportedocumental_documentoletra_del()
RETURNS trigger AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
    auxIdAgrupacion integer;
    auxTipoDocumento general_accc.enum_tipodocumento;
    auxTipoSoporte general_accc.enum_tiposoporte;
BEGIN
    SELECT tipo_documento, tipo_soporte, agrup.id
            INTO auxTipoDocumento, auxTipoSoporte, auxIdAgrupacion
        FROM general_accc.documento_letra AS d_l
        INNER JOIN general_accc.letra AS l
            ON d_l.letra = l.id
        INNER JOIN general_accc.agrupacion_anio AS agrup
            ON agrup.interpreta_literaria = l.obra_literaria
        INNER JOIN general_accc.documento AS doc
            ON d_l.documento = doc.id
        WHERE d_l.documento = OLD.documento
            AND l.id = OLD.letra;
    IF FOUND THEN
        PERFORM general_accc.__dwh_baja_soportedocumental (auxIdAgrupacion,
                                                           'Literaria'::general_accc.enum_obra,
                                                           auxTipoDocumento,
                                                           auxTipoSoporte);
    END IF;

    RETURN OLD;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_soportedocumental_documentomusica_in; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_soportedocumental_documentomusica_in()
RETURNS trigger AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
    auxIdAgrupacion integer;
    auxAgrupacion character varying;
    auxTipoDocumento general_accc.enum_tipodocumento;
    auxTipoSoporte general_accc.enum_tiposoporte;
BEGIN
    SELECT tipo_documento, tipo_soporte, agrup.id, agrup.nombre
            INTO auxTipoDocumento, auxTipoSoporte, auxIdAgrupacion, auxAgrupacion
        FROM general_accc.documento_musica AS d_m
        INNER JOIN general_accc.musica AS musica
            ON d_m.musica = musica.id
        INNER JOIN general_accc.agrupacion_anio AS agrup
            ON agrup.interpreta_musical = musica.obra_musical
        INNER JOIN general_accc.documento AS doc
            ON d_m.documento = doc.id
        WHERE d_m.documento = NEW.documento
            AND musica.id = NEW.musica;
            
    IF FOUND AND auxTipoSoporte <> 'No disponible'::general_accc.enum_tiposoporte THEN
        PERFORM general_accc.__dwh_alta_soportedocumental (auxIdAgrupacion,
                                                           auxAgrupacion,
                                                           'Musical',
                                                           auxTipoDocumento,
                                                           auxTipoSoporte);
    END IF;

    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_soportedocumental_documentomusica_in; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_soportedocumental_documentomusica_del()
RETURNS trigger AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
    auxIdAgrupacion integer;
    auxTipoDocumento general_accc.enum_tipodocumento;
    auxTipoSoporte general_accc.enum_tiposoporte;
BEGIN

    SELECT tipo_documento, tipo_soporte, agrup.id
            INTO auxTipoDocumento, auxTipoSoporte, auxIdAgrupacion
        FROM general_accc.documento_musica AS d_m
        INNER JOIN general_accc.musica AS musica
            ON d_m.musica = musica.id
        INNER JOIN general_accc.agrupacion_anio AS agrup
            ON agrup.interpreta_musical = musica.obra_musical
        INNER JOIN general_accc.documento AS doc
            ON d_m.documento = doc.id
        WHERE d_m.documento = OLD.documento
            AND musica.id = OLD.musica;
            
    IF FOUND THEN
        PERFORM general_accc.__dwh_baja_soportedocumental (auxIdAgrupacion,
                                                           'Musical'::general_accc.enum_obra,
                                                           auxTipoDocumento,
                                                           auxTipoSoporte);
    END IF;

    RETURN OLD;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_soportedocumental_documentodisenio_in; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_soportedocumental_documentodisenio_in()
RETURNS trigger AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
    auxIdAgrupacion integer;
    auxAgrupacion character varying;
    auxTipoDocumento general_accc.enum_tipodocumento;
    auxTipoSoporte general_accc.enum_tiposoporte;
BEGIN
    SELECT tipo_documento, tipo_soporte, agrup.id, agrup.nombre
            INTO auxTipoDocumento, auxTipoSoporte, auxIdAgrupacion, auxAgrupacion
        FROM general_accc.documento_disenio AS d_d
        INNER JOIN general_accc.disenio AS d
            ON d_d.disenio = d.id
        INNER JOIN general_accc.agrupacion_anio AS agrup
            ON agrup.utiliza_tipo = d.obra_grafica
        INNER JOIN general_accc.documento AS doc
            ON d_d.documento = doc.id
        WHERE d_d.documento = NEW.documento
            AND d.id = NEW.disenio;
    IF FOUND AND auxTipoSoporte <> 'No disponible'::general_accc.enum_tiposoporte THEN
        PERFORM general_accc.__dwh_alta_soportedocumental (auxIdAgrupacion,
                                                           auxAgrupacion,
                                                           'Grafica',
                                                           auxTipoDocumento,
                                                           auxTipoSoporte);
    END IF;

    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_soportedocumental_documentodisenio_del; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_soportedocumental_documentodisenio_del()
RETURNS trigger AS $$
DECLARE
    auxInteger integer;
    auxCharacter character varying;
    auxIdAgrupacion integer;
    auxTipoDocumento general_accc.enum_tipodocumento;
    auxTipoSoporte general_accc.enum_tiposoporte;
BEGIN
    SELECT tipo_documento, tipo_soporte, agrup.id
            INTO auxTipoDocumento, auxTipoSoporte, auxIdAgrupacion
        FROM general_accc.documento_disenio AS d_d
        INNER JOIN general_accc.disenio AS d
            ON d_d.disenio = d.id
        INNER JOIN general_accc.agrupacion_anio AS agrup
            ON agrup.utiliza_tipo = d.obra_grafica
        INNER JOIN general_accc.documento AS doc
            ON d_d.documento = doc.id
        WHERE d_d.documento = OLD.documento
            AND d.id = OLD.disenio;
    IF FOUND THEN
        PERFORM general_accc.__dwh_baja_soportedocumental (auxIdAgrupacion,
                                                           'Grafica'::general_accc.enum_obra,
                                                           auxTipoDocumento,
                                                           auxTipoSoporte);
    END IF;

    RETURN OLD;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;











-------------------------------------------------------------------------------------------
--                 Creación de disparadores para: dwh_soporte_documental                 --
-------------------------------------------------------------------------------------------

--
-- Nombre: dwh_tr_soportedocumental_documento; Tipo: TRIGGER; Schema: general_accc
--
CREATE TRIGGER dwh_tr_soportedocumental_documento_up
    BEFORE UPDATE
    ON general_accc.documento
    FOR EACH ROW
    EXECUTE PROCEDURE general_accc.dwh_sp_soportedocumental_documento_up();


--
-- Nombre: dwh_tr_soportedocumental_documentoobra; Tipo: TRIGGER; Schema: general_accc
--
CREATE TRIGGER dwh_tr_soportedocumental_documentoobra_in
    BEFORE INSERT
    ON general_accc.documento_obra
    FOR EACH ROW
    EXECUTE PROCEDURE general_accc.dwh_sp_soportedocumental_documentoobra_in();

CREATE TRIGGER dwh_tr_soportedocumental_documentoobra_del
    BEFORE DELETE
    ON general_accc.documento_obra
    FOR EACH ROW
    EXECUTE PROCEDURE general_accc.dwh_sp_soportedocumental_documentoobra_del();


--
-- Nombre: dwh_tr_soportedocumental_documento; Tipo: TRIGGER; Schema: general_accc
--
CREATE TRIGGER dwh_tr_soportedocumental_documentoletra_in
    BEFORE INSERT
    ON general_accc.documento_letra
    FOR EACH ROW
    EXECUTE PROCEDURE general_accc.dwh_sp_soportedocumental_documentoletra_in();

CREATE TRIGGER dwh_tr_soportedocumental_documentoletra_del
    BEFORE DELETE
    ON general_accc.documento_letra
    FOR EACH ROW
    EXECUTE PROCEDURE general_accc.dwh_sp_soportedocumental_documentoletra_del();


--
-- Nombre: dwh_tr_soportedocumental_documento; Tipo: TRIGGER; Schema: general_accc
--
CREATE TRIGGER dwh_tr_soportedocumental_documentomusica_in
    BEFORE INSERT
    ON general_accc.documento_musica
    FOR EACH ROW
    EXECUTE PROCEDURE general_accc.dwh_sp_soportedocumental_documentomusica_in();

CREATE TRIGGER dwh_tr_soportedocumental_documentomusica_del
    BEFORE DELETE
    ON general_accc.documento_musica
    FOR EACH ROW
    EXECUTE PROCEDURE general_accc.dwh_sp_soportedocumental_documentomusica_del();


--
-- Nombre: dwh_tr_soportedocumental_documento; Tipo: TRIGGER; Schema: general_accc
--
CREATE TRIGGER dwh_tr_soportedocumental_documentodisenio_in
    BEFORE INSERT
    ON general_accc.documento_disenio
    FOR EACH ROW
    EXECUTE PROCEDURE general_accc.dwh_sp_soportedocumental_documentodisenio_in();

CREATE TRIGGER dwh_tr_soportedocumental_documentodisenio_del
    BEFORE DELETE
    ON general_accc.documento_disenio
    FOR EACH ROW
    EXECUTE PROCEDURE general_accc.dwh_sp_soportedocumental_documentodisenio_del();

