-- TFG BBDD Carnaval de Cádiz - Paloma Cortés


-- Creación de funciones para el data warehouse




--------------------------------------------------------------------------------
--           Creación de procedimientos para: dwh_temas_recurrentes           --
--------------------------------------------------------------------------------


--
-- Nombre: __dwh_alta_temasrecurrentes; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.__dwh_alta_temasrecurrentes(in_tema integer,
                                                                    in_tipo_obra general_accc.enum_obra,
                                                                    in_anio integer,
                                                                    in_modalidad integer,
                                                                    in_categoria general_accc.enum_categoria)
RETURNS void AS $$
DECLARE
    auxInteger integer;
BEGIN

    -- Comprobar si existe el tema para el año, el tipo de obra, la modalidad y
    -- la categoría
    SELECT total INTO auxInteger
        FROM general_accc.dwh_temas_recurrentes
        WHERE id_tema = in_tema
            AND tipo_obra = in_tipo_obra
            AND anio = in_anio
            AND modalidad = (SELECT modalidad FROM general_accc.modalidad WHERE id = in_modalidad)
            AND categoria = in_categoria;

    IF FOUND THEN
        UPDATE general_accc.dwh_temas_recurrentes SET
                                                  total = auxInteger + 1
            WHERE id_tema = in_tema
                AND tipo_obra = in_tipo_obra
                AND anio = in_anio
                AND modalidad = (SELECT modalidad FROM general_accc.modalidad WHERE id = in_modalidad)
                AND categoria = in_categoria;

    ELSEIF in_anio IS NOT NULL AND in_modalidad IS NOT NULL AND in_categoria IS NOT NULL THEN
        INSERT INTO general_accc.dwh_temas_recurrentes (id_tema, tema, tipo_obra, anio, modalidad, categoria, total)
            VALUES (in_tema,
                    (SELECT tema FROM general_accc.tema WHERE id = in_tema),
                    in_tipo_obra,
                    in_anio,
                    (SELECT modalidad FROM general_accc.modalidad WHERE id = in_modalidad),
                    in_categoria,
                    1);

    --  ELSE se recuperarán los datos cuando se asocie la obra a la que
        -- pertenece a una agrupación
    END IF;

    RETURN;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES ('dwh_temas_recurrentes', '__dwh_alta_temasrecurrentes', 'function', SQLERRM);
    RETURN;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: __dwh_baja_temasrecurrentes; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.__dwh_baja_temasrecurrentes(in_tema integer,
                                                                    in_tipo_obra general_accc.enum_obra,
                                                                    in_anio integer,
                                                                    in_modalidad integer,
                                                                    in_categoria general_accc.enum_categoria)
RETURNS void AS $$
DECLARE
    auxInteger integer;
BEGIN

    -- Comprobar si existe el tema para el año, el tipo de obra, la modalidad
    -- y la categoría
    SELECT total INTO auxInteger
        FROM general_accc.dwh_temas_recurrentes
        WHERE id_tema = in_tema
            AND tipo_obra = in_tipo_obra
            AND anio = in_anio
            AND modalidad = (SELECT modalidad FROM general_accc.modalidad WHERE id = in_modalidad)
            AND categoria = in_categoria;

    -- Si el contador es mayor que 1 se resta una unidad, en caso contrario
    -- se elimina el registro
    IF auxInteger > 1 THEN
        UPDATE general_accc.dwh_temas_recurrentes SET
                                                  total = auxInteger - 1
            WHERE id_tema = in_tema
                AND anio = in_anio
                AND modalidad = (SELECT modalidad FROM general_accc.modalidad WHERE id = in_modalidad)
                AND categoria = in_categoria
                AND tipo_obra = in_tipo_obra;
                
    ELSEIF auxInteger = 1 THEN
        DELETE FROM general_accc.dwh_temas_recurrentes
            WHERE id_tema = in_tema
                AND anio = in_anio
                AND modalidad = (SELECT modalidad FROM general_accc.modalidad WHERE id = in_modalidad)
                AND categoria = in_categoria
                AND tipo_obra = in_tipo_obra;
    END IF;

    RETURN;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES ('dwh_temas_recurrentes', '__dwh_baja_temasrecurrentes', 'function', SQLERRM);
    RETURN;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_temasrecurrentes_tema_up; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_temasrecurrentes_tema_up()
RETURNS trigger AS $$
BEGIN

    -- En caso de cambiar la cadena que identifica el tema se actualzan los
    -- registros que lo utilizan
    UPDATE general_accc.dwh_temas_recurrentes SET
                                              tema = NEW.tema
            WHERE id_tema = OLD.id;
    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_temasrecurrentes_disenio_del; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_temasrecurrentes_letra_del(old_id integer,
                                                                          old_obra_literaria integer)
RETURNS void AS $$
DECLARE
    auxAnio integer;
    auxModalidad integer;
    auxCategoria general_accc.enum_categoria;
    auxRecord record;
BEGIN

    SELECT anio, modalidad, categoria
        INTO auxAnio, auxModalidad, auxCategoria
        FROM general_accc.agrupacion_anio
        WHERE interpreta_literaria = old_obra_literaria;

    IF FOUND THEN
        FOR auxRecord IN (SELECT tema
                              FROM general_accc.tema_letra
                              WHERE letra = old_id)
        LOOP
            -- Resta una unidad a cada tema afectado por la letra eliminada
            PERFORM general_accc.__dwh_baja_temasrecurrentes (auxRecord.tema,
                                                              'Literaria',
                                                              auxAnio,
                                                              auxModalidad,
                                                              auxCategoria);
        END LOOP;
    END IF;
    
    RETURN;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES ('letra', 'dwh_sp_letra_del', 'DELETE', SQLERRM);
    RETURN;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_temasrecurrentes_letra_in; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_temasrecurrentes_temaletra_in()
RETURNS trigger AS $$
DECLARE
    auxAnio integer;
    auxModalidad integer;
    auxCategoria general_accc.enum_categoria;
BEGIN

    SELECT anio, modalidad, categoria
        INTO auxAnio, auxModalidad, auxCategoria
        FROM general_accc.agrupacion_anio AS agrup
        INNER JOIN general_accc.literaria AS obra
            ON agrup.interpreta_literaria = (SELECT obra_literaria
                                                 FROM general_accc.letra
                                                 WHERE id = NEW.letra);
    IF FOUND THEN
        PERFORM general_accc.__dwh_alta_temasrecurrentes (NEW.tema,
                                                          'Literaria',
                                                          auxAnio,
                                                          auxModalidad,
                                                          auxCategoria);
    END IF;
    
    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: dwh_sp_temasrecurrentes_letra_up; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_temasrecurrentes_temaletra_up()
RETURNS trigger AS $$
DECLARE
    auxAnio integer;
    auxModalidad integer;
    auxCategoria general_accc.enum_categoria;
BEGIN

    SELECT anio, modalidad, categoria
        INTO auxAnio, auxModalidad, auxCategoria
        FROM general_accc.agrupacion_anio AS agrup
        INNER JOIN general_accc.literaria AS obra
            ON agrup.interpreta_literaria = (SELECT obra_literaria
                                                 FROM general_accc.letra
                                                 WHERE id = OLD.letra);
    -- En las modificaciones la letra no cambia, solo puede cambiar el tema
    -- por tanto los valores de agrupación recogidos son válidos
    
    IF FOUND THEN
        PERFORM general_accc.__dwh_baja_temasrecurrentes (OLD.tema,
                                                          'Literaria',
                                                          auxAnio,
                                                          auxModalidad,
                                                          auxCategoria);

        -- Para aumentar el contador se procede como en el caso del trigger para
        -- inserts en tema_letra si existe el tema para el año, el tipo de obra,
        -- la modalidad y la categoría se actualiza y en caso contrario se crea
        PERFORM general_accc.__dwh_alta_temasrecurrentes (NEW.tema, 
                                                          'Literaria',
                                                          auxAnio,
                                                          auxModalidad,
                                                          auxCategoria);
    END IF;
    
    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_temasrecurrentes_letra_del; Tipo: FUNCTION; Schema: general_accc
--

CREATE OR REPLACE FUNCTION general_accc.dwh_sp_temasrecurrentes_temaletra_del()
RETURNS trigger AS $$
DECLARE
    auxAnio integer;
    auxModalidad integer;
    auxCategoria general_accc.enum_categoria;
BEGIN

    -- De ser la última aparición de la letra ya no existe y no se puede
    -- localizar la obra
    SELECT anio, modalidad, categoria
        INTO auxAnio, auxModalidad, auxCategoria
        FROM general_accc.agrupacion_anio AS agrup
        INNER JOIN general_accc.literaria AS obra
            ON agrup.interpreta_literaria =
                    (SELECT obra_literaria FROM general_accc.letra WHERE id = OLD.letra);
    IF FOUND THEN
        PERFORM general_accc.__dwh_baja_temasrecurrentes (OLD.tema, 
                                                          'Literaria',
                                                          auxAnio,
                                                          auxModalidad,
                                                          auxCategoria);
    END IF;

    RETURN OLD;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: dwh_sp_temasrecurrentes_disenio_del; Tipo: FUNCTION; Schema: general_accc
--

CREATE OR REPLACE FUNCTION general_accc.dwh_sp_temasrecurrentes_disenio_del(old_id integer,
                                                                            old_obra_grafica integer)
RETURNS void AS $$
DECLARE
    auxAnio integer;
    auxModalidad integer;
    auxCategoria general_accc.enum_categoria;
    auxRecord record;
BEGIN

    SELECT anio, modalidad, categoria
        INTO auxAnio, auxModalidad, auxCategoria
        FROM general_accc.agrupacion_anio
        WHERE utiliza_tipo = old_obra_grafica;

    IF FOUND THEN
        -- Resta una unidad a cada tema afectado por el diseño eliminado
        FOR auxRecord IN (SELECT tema
                              FROM general_accc.tema_disenio
                              WHERE disenio = old_id)
        LOOP
            PERFORM general_accc.__dwh_baja_temasrecurrentes (auxRecord.tema,
                                                              'Grafica',
                                                              auxAnio,
                                                              auxModalidad,
                                                              auxCategoria);
        END LOOP;
    END IF;
    
    RETURN;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES ('disenio', 'dwh_sp_disenio_del', 'DELETE', SQLERRM);
    RETURN;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: dwh_sp_temasrecurrentes_temadisenio_in; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_temasrecurrentes_temadisenio_in()
RETURNS trigger AS $$
DECLARE
    auxAnio integer;
    auxModalidad integer;
    auxCategoria general_accc.enum_categoria;
BEGIN

    SELECT anio, modalidad, categoria
        INTO auxAnio, auxModalidad, auxCategoria
        FROM general_accc.agrupacion_anio AS agrup
        INNER JOIN general_accc.grafica AS obra
            ON agrup.utiliza_tipo =
                    (SELECT obra_grafica FROM general_accc.disenio WHERE id = NEW.disenio);
    IF FOUND THEN
        PERFORM general_accc.__dwh_alta_temasrecurrentes (NEW.tema,
                                                          'Grafica',
                                                          auxAnio,
                                                          auxModalidad,
                                                          auxCategoria);

    -- ELSE se recuperarán los datos cuando se asocie la obra a la que pertenece
        -- a una agrupación
    END IF;

    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;




--
-- Nombre: dwh_sp_temasrecurrentes_temadisenio_up; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_temasrecurrentes_temadisenio_up()
RETURNS trigger AS $$
DECLARE
    auxInteger integer;
    auxAnio integer;
    auxModalidad integer;
    auxCategoria general_accc.enum_categoria;
BEGIN

    -- El contador existirá en DWH_temas_recurrentes para el tema que se retira
    -- si tenía alguna agrupación indirectamente vinculada
    SELECT anio, modalidad, categoria
        INTO auxAnio, auxModalidad, auxCategoria
        FROM general_accc.agrupacion_anio AS agrup
        INNER JOIN general_accc.grafica AS obra
            ON agrup.utiliza_tipo = (SELECT obra_grafica
                                         FROM general_accc.disenio
                                         WHERE id = OLD.disenio);

    IF FOUND THEN
        PERFORM general_accc.__dwh_baja_temasrecurrentes (OLD.tema,
                                                          'Grafica',
                                                          auxAnio,
                                                          auxModalidad,
                                                          auxCategoria);

        -- Para aumentar el contador se procede como en el caso del trigger para
        -- inserts en tema_letra, si existe el tema para el año, el tipo de obra,
        -- la modalidad y la categoría se actualiza.
        -- En caso contrario se crea se da por hecho que el diseño no ha cambiado 
        -- en el UPDATE de tema_letra

        PERFORM general_accc.__dwh_alta_temasrecurrentes (NEW.tema, 
                                                          'Grafica',
                                                          auxAnio,
                                                          auxModalidad,
                                                          auxCategoria);
    END IF;

    RETURN NEW;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_temasrecurrentes_disenio_del; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_temasrecurrentes_temadisenio_del()
RETURNS trigger AS $$
DECLARE
    auxAnio integer;
    auxModalidad integer;
    auxCategoria general_accc.enum_categoria;
BEGIN

    SELECT anio, modalidad, categoria
        INTO auxAnio, auxModalidad, auxCategoria
        FROM general_accc.agrupacion_anio AS agrup
        INNER JOIN general_accc.grafica AS obra
            ON agrup.utiliza_tipo =
                    (SELECT obra_grafica FROM general_accc.disenio WHERE id = OLD.disenio);
    IF FOUND THEN
        PERFORM general_accc.__dwh_baja_temasrecurrentes (OLD.tema,
                                                          'Grafica',
                                                          auxAnio,
                                                          auxModalidad,
                                                          auxCategoria);
    END IF;

    RETURN OLD;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES (TG_TABLE_NAME, TG_NAME, TG_OP, SQLERRM);
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_temasrecurrentes_agrupacionanio_in; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_temasrecurrentes_agrupacionanio_in (new_interpreta_literaria integer,
                                                                                   new_utiliza_tipo integer,
                                                                                   new_anio integer,
                                                                                   new_modalidad integer,
                                                                                   new_categoria general_accc.enum_categoria)
RETURNS void AS $$
DECLARE
    auxRecord record;
BEGIN

    IF new_interpreta_literaria IS NOT NULL THEN

        FOR auxRecord IN (SELECT t_l.id
                              FROM general_accc.tema_letra AS t_l
                              INNER JOIN general_accc.letra AS l
                                  ON t_l.letra = l.id
                              WHERE l.obra_literaria = new_interpreta_literaria)
        LOOP
            PERFORM general_accc.__dwh_alta_temasrecurrentes (auxRecord.id,
                                                              'Literaria',
                                                              new_anio,
                                                              new_modalidad,
                                                              new_categoria);
        END LOOP;

    END IF;

    IF new_utiliza_tipo IS NOT NULL THEN

        FOR auxRecord IN (SELECT t_d.id
                              FROM general_accc.tema_disenio AS t_d
                                  INNER JOIN general_accc.disenio AS d
                                      ON t_d.disenio = d.id
                              WHERE d.obra_grafica = new_utiliza_tipo)
        LOOP
            PERFORM general_accc.__dwh_alta_temasrecurrentes (auxRecord.id,
                                                              'Grafica',
                                                              new_anio,
                                                              new_modalidad,
                                                              new_categoria);
        END LOOP;

    END IF;

    RETURN;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES ('agrupacion_anio', 'dwh_tr_agrupacionanio_in', 'INSERT', SQLERRM);
    RETURN;
END;
$$ LANGUAGE plpgsql;



--
-- Nombre: dwh_sp_temasrecurrentes_agrupacionanio_up; Tipo: FUNCTION; Schema: general_accc
--
CREATE OR REPLACE FUNCTION general_accc.dwh_sp_temasrecurrentes_agrupacionanio_up (new_interpreta_literaria integer,
                                                                                   old_interpreta_literaria integer,
                                                                                   new_utiliza_tipo integer,
                                                                                   old_utiliza_tipo integer,
                                                                                   new_anio integer,
                                                                                   old_anio integer,
                                                                                   new_modalidad integer,
                                                                                   old_modalidad integer,
                                                                                   new_categoria general_accc.enum_categoria,
                                                                                   old_categoria general_accc.enum_categoria)
RETURNS void AS $$
DECLARE
    auxRecord record;
BEGIN

    IF COALESCE(new_interpreta_literaria,0) <> COALESCE(old_interpreta_literaria,0) THEN
        -- Decrementar en una unidad los temas asociados a la obra desvinculada

        IF old_interpreta_literaria IS NOT NULL THEN

            FOR auxRecord IN (SELECT t_l.tema
                                FROM general_accc.tema_letra AS t_l
                                    INNER JOIN general_accc.letra AS l
                                        ON t_l.letra = l.id
                                WHERE l.obra_literaria = old_interpreta_literaria)
            LOOP
                PERFORM general_accc.__dwh_baja_temasrecurrentes (auxRecord.tema,
                                                                  'Literaria',
                                                                  old_anio,
                                                                  old_modalidad,
                                                                  old_categoria);
            END LOOP;

        END IF;

        IF new_interpreta_literaria IS NOT NULL THEN

            FOR auxRecord IN (SELECT t.tema
                                  FROM general_accc.tema_letra AS t
                                  INNER JOIN general_accc.letra AS l
                                      ON t.letra = l.id
                                  WHERE l.obra_literaria = new_interpreta_literaria)
            LOOP
                 PERFORM general_accc.__dwh_alta_temasrecurrentes (auxRecord.tema,
                                                                  'Literaria',
                                                                  new_anio,
                                                                  new_modalidad,
                                                                  new_categoria);
            END LOOP;

        END IF;
    END IF;

    IF COALESCE(new_utiliza_tipo, 0) <> COALESCE(old_utiliza_tipo, 0) THEN
        -- Decrementar en una unidad los temas asociados a la obra desvinculada

        IF old_utiliza_tipo IS NOT NULL THEN
            FOR auxRecord IN (SELECT t_d.tema
                                  FROM general_accc.tema_disenio AS t_d
                                      INNER JOIN general_accc.disenio AS d
                                          ON t_d.disenio = d.id
                                  WHERE d.obra_grafica = old_utiliza_tipo)
            LOOP
                PERFORM general_accc.__dwh_baja_temasrecurrentes (auxRecord.tema,
                                                                  'Grafica',
                                                                  old_anio,
                                                                  old_modalidad,
                                                                  old_categoria);
            END LOOP;
        END IF;

        IF new_utiliza_tipo IS NOT NULL THEN
            FOR auxRecord IN (SELECT t_d.tema
                                  FROM general_accc.tema_disenio AS t_d
                                      INNER JOIN general_accc.disenio AS d
                                          ON t_d.disenio = d.id
                                  WHERE d.obra_grafica = new_utiliza_tipo)
            LOOP
                PERFORM general_accc.__dwh_alta_temasrecurrentes (auxRecord.tema,
                                                                  'Grafica',
                                                                  new_anio,
                                                                  new_modalidad,
                                                                  new_categoria);
            END LOOP;
        END IF;
    END IF;

    RETURN;

EXCEPTION WHEN others THEN
    INSERT INTO general_accc.dwh_error_log (tabla, disparador, evento, mensaje)
        VALUES ('agrupacion_anio', 'dwh_sp_temasrecurrentes_agrupacionanio_up', 'UPDATE', SQLERRM);
    RETURN;
END;
$$ LANGUAGE plpgsql;

-- No es necesario en caso de DELETE en agrupacion_anio porque antes de eliminar
-- las agrupaciones deben habérsele retirado las obras






-------------------------------------------------------------------------------------------
--                Creación de disparadores para: dwh_temas_recurrentes                   --
-------------------------------------------------------------------------------------------

--
-- Nombre: dwh_tr_temasrecurrentes_tema; Tipo: TRIGGER; Schema: general_accc
--
CREATE TRIGGER dwh_tr_temasrecurrentes_tema_up
  BEFORE UPDATE
  ON general_accc.tema
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_temasrecurrentes_tema_up();


--
-- Nombre: dwh_tr_temasrecurrentes_temaletra; Tipo: TRIGGER; Schema: general_accc
--
CREATE TRIGGER dwh_tr_temasrecurrentes_temaletra_in
  BEFORE INSERT
  ON general_accc.tema_letra
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_temasrecurrentes_temaletra_in();

CREATE TRIGGER dwh_tr_temasrecurrentes_temaletra_up
  BEFORE UPDATE
  ON general_accc.tema_letra
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_temasrecurrentes_temaletra_up();

CREATE TRIGGER dwh_tr_temasrecurrentes_temaletra_del
  BEFORE DELETE
  ON general_accc.tema_letra
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_temasrecurrentes_temaletra_del();

--
-- Nombre: dwh_tr_temasrecurrentes_temadisenio; Tipo: TRIGGER; Schema: general_accc
--
CREATE TRIGGER dwh_tr_temasrecurrentes_temadisenio_in
  BEFORE INSERT
  ON general_accc.tema_disenio
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_temasrecurrentes_temadisenio_in();

CREATE TRIGGER dwh_tr_temasrecurrentes_temadisenio_up
  BEFORE UPDATE
  ON general_accc.tema_disenio
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_temasrecurrentes_temadisenio_up();

CREATE TRIGGER dwh_tr_temasrecurrentes_temadisenio_del
  BEFORE DELETE
  ON general_accc.tema_disenio
  FOR EACH ROW
  EXECUTE PROCEDURE general_accc.dwh_sp_temasrecurrentes_temadisenio_del();




